@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2014  Nexedi SA
+# Copyright (C) 2006-2016  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -16,10 +16,10 @@
 
 from cPickle import dumps, loads
 from zlib import compress, decompress
-from neo.lib.locking import Empty
 from random import shuffle
 import heapq
 import time
+import weakref
 from functools import partial
 
 from ZODB.POSException import UndoError, StorageTransactionError, ConflictError
@@ -32,22 +32,20 @@ from neo.lib.protocol import NodeTypes, Packets, \
     INVALID_PARTITION, ZERO_HASH, ZERO_TID
 from neo.lib.event import EventManager
 from neo.lib.util import makeChecksum, dump
-from neo.lib.locking import Lock
+from neo.lib.locking import Empty, Lock, SimpleQueue
 from neo.lib.connection import MTClientConnection, ConnectionClosed
 from neo.lib.node import NodeManager
-from neo.lib.connector import getConnectorHandler
 from .exception import NEOStorageError, NEOStorageCreationUndoneError
 from .exception import NEOStorageNotFoundError
 from .handlers import storage, master
 from neo.lib.dispatcher import Dispatcher, ForgottenPacket
-from .poll import ThreadedPoll, psThreadedPoll
+from neo.lib.threaded_app import ThreadedApplication
 from .cache import ClientCache
 from .pool import ConnectionPool
 from neo.lib.util import p64, u64, parseMasterList
 from neo.lib.debug import register as registerLiveDebugger
-from .container import ThreadContainer, TransactionContainer
 
-CHECKED_SERIAL = master.CHECKED_SERIAL
+CHECKED_SERIAL = object()
 
 try:
     from Signals.Signals import SignalHandler
@@ -57,33 +55,54 @@ if SignalHandler:
     import signal
     SignalHandler.registerHandler(signal.SIGUSR2, logging.reopen)
 
-class Application(object):
+
+class TransactionContainer(dict):
+
+    def pop(self, txn):
+        return dict.pop(self, id(txn), None)
+
+    def get(self, txn):
+        try:
+            return self[id(txn)]
+        except KeyError:
+            raise StorageTransactionError("unknown transaction %r" % txn)
+
+    def new(self, txn):
+        key = id(txn)
+        if key in self:
+            raise StorageTransactionError("commit of transaction %r"
+                                          " already started" % txn)
+        context = self[key] = {
+            'queue': SimpleQueue(),
+            'txn': txn,
+            'ttid': None,
+            'data_dict': {},
+            'data_size': 0,
+            'cache_dict': {},
+            'cache_size': 0,
+            'object_base_serial_dict': {},
+            'object_serial_dict': {},
+            'object_stored_counter_dict': {},
+            'conflict_serial_dict': {},
+            'resolved_conflict_serial_dict': {},
+            'involved_nodes': set(),
+        }
+        return context
+
+
+class Application(ThreadedApplication):
     """The client node application."""
 
-    def __init__(self, master_nodes, name, compress=True,
-            dynamic_master_list=None):
-        # Start polling thread
-        self.em = EventManager()
-        self.poll_thread = ThreadedPoll(self.em, name=name)
-        psThreadedPoll()
+    def __init__(self, master_nodes, name, compress=True, **kw):
+        super(Application, self).__init__(parseMasterList(master_nodes),
+                                          name, **kw)
         # Internal Attributes common to all thread
         self._db = None
-        self.name = name
-        master_addresses, connector_name = parseMasterList(master_nodes)
-        self.connector_handler = getConnectorHandler(connector_name)
-        self.dispatcher = Dispatcher(self.poll_thread)
-        self.nm = NodeManager(dynamic_master_list)
         self.cp = ConnectionPool(self)
-        self.master_conn = None
         self.primary_master_node = None
         self.trying_master_node = None
 
-        # load master node list
-        for address in master_addresses:
-            self.nm.createMaster(address=address)
-
         # no self-assigned UUID, primary master will supply us one
-        self.uuid = None
         self._cache = ClientCache()
         self._loading_oid = None
         self.new_oid_list = ()
@@ -95,8 +114,6 @@ class Application(object):
         self.primary_handler = master.PrimaryAnswersHandler(self)
         self.primary_bootstrap_handler = master.PrimaryBootstrapHandler(self)
         self.notifications_handler = master.PrimaryNotificationsHandler( self)
-        # Internal attribute distinct between thread
-        self._thread_container = ThreadContainer()
         self._txn_container = TransactionContainer()
         # Lock definition :
         # _load_lock is used to make loading and storing atomic
@@ -115,64 +132,22 @@ class Application(object):
         # _connecting_to_master_node is used to prevent simultaneous master
         # node connection attemps
         self._connecting_to_master_node = Lock()
-        # _nm ensure exclusive access to the node manager
-        lock = Lock()
-        self._nm_acquire = lock.acquire
-        self._nm_release = lock.release
         self.compress = compress
-        registerLiveDebugger(on_log=self.log)
 
     def __getattr__(self, attr):
         if attr == 'pt':
             self._getMasterConnection()
         return self.__getattribute__(attr)
 
+    def log(self):
+        super(Application, self).log()
+        logging.info("%r", self._cache)
+
     @property
     def txn_contexts(self):
         # do not iter lazily to avoid race condition
         return self._txn_container.values
 
-    def getHandlerData(self):
-        return self._thread_container.answer
-
-    def setHandlerData(self, data):
-        self._thread_container.answer = data
-
-    def log(self):
-        self.em.log()
-        self.nm.log()
-        if self.pt is not None:
-            self.pt.log()
-
-    def _handlePacket(self, conn, packet, kw={}, handler=None):
-        """
-          conn
-            The connection which received the packet (forwarded to handler).
-          packet
-            The packet to handle.
-          handler
-            The handler to use to handle packet.
-            If not given, it will be guessed from connection's not type.
-        """
-        if handler is None:
-            # Guess the handler to use based on the type of node on the
-            # connection
-            node = self.nm.getByAddress(conn.getAddress())
-            if node is None:
-                raise ValueError, 'Expecting an answer from a node ' \
-                    'which type is not known... Is this right ?'
-            if node.isStorage():
-                handler = self.storage_handler
-            elif node.isMaster():
-                handler = self.primary_handler
-            else:
-                raise ValueError, 'Unknown node type: %r' % (node.__class__, )
-        conn.lock()
-        try:
-            handler.dispatch(conn, packet, kw)
-        finally:
-            conn.unlock()
-
     def _waitAnyMessage(self, queue, block=True):
         """
           Handle all pending packets.
@@ -210,29 +185,6 @@ class Application(object):
             # Don't leave access to thread context, even if a raise happens.
             self.setHandlerData(None)
 
-    def _ask(self, conn, packet, handler=None, **kw):
-        self.setHandlerData(None)
-        queue = self._thread_container.queue
-        msg_id = conn.ask(packet, queue=queue, **kw)
-        get = queue.get
-        _handlePacket = self._handlePacket
-        while True:
-            qconn, qpacket, kw = get(True)
-            is_forgotten = isinstance(qpacket, ForgottenPacket)
-            if conn is qconn:
-                # check fake packet
-                if qpacket is None:
-                    raise ConnectionClosed
-                if msg_id == qpacket.getId():
-                    if is_forgotten:
-                        raise ValueError, 'ForgottenPacket for an ' \
-                            'explicitely expected packet.'
-                    _handlePacket(qconn, qpacket, kw, handler)
-                    break
-            if not is_forgotten and qpacket is not None:
-                _handlePacket(qconn, qpacket, kw)
-        return self.getHandlerData()
-
     def _askStorage(self, conn, packet, **kw):
         """ Send a request to a storage node and process its answer """
         return self._ask(conn, packet, handler=self.storage_handler, **kw)
@@ -261,7 +213,8 @@ class Application(object):
             Lookup for the current primary master node
         """
         logging.debug('connecting to primary master...')
-        index = 0
+        self.start()
+        index = -1
         ask = self._ask
         handler = self.primary_bootstrap_handler
         while 1:
@@ -274,18 +227,12 @@ class Application(object):
                 else:
                     # Otherwise, check one by one.
                     master_list = self.nm.getMasterList()
-                    try:
-                        self.trying_master_node = master_list[index]
-                    except IndexError:
-                        time.sleep(1)
-                        index = 0
-                        self.trying_master_node = master_list[0]
-                    index += 1
+                    index = (index + 1) % len(master_list)
+                    self.trying_master_node = master_list[index]
                 # Connect to master
-                conn = MTClientConnection(self.em,
+                conn = MTClientConnection(self,
                         self.notifications_handler,
                         node=self.trying_master_node,
-                        connector=self.connector_handler(),
                         dispatcher=self.dispatcher)
                 # Query for primary master node
                 if conn.getConnector() is None:
@@ -435,9 +382,7 @@ class Application(object):
                 return (decompress(data) if compression else data,
                         tid, next_tid, data_tid)
             raise NEOStorageCreationUndoneError(dump(oid))
-        # We didn't got any object from all storage node because of
-        # connection error
-        raise NEOStorageError('connection failure')
+        raise NEOStorageError("storage down or corrupted data")
 
     def _loadFromCache(self, oid, at_tid=None, before_tid=None):
         """
@@ -671,18 +616,29 @@ class Application(object):
         packet = Packets.AskStoreTransaction(ttid, str(transaction.user),
             str(transaction.description), dumps(transaction._extension),
             txn_context['cache_dict'])
-        add_involved_nodes = txn_context['involved_nodes'].add
+        queue = txn_context['queue']
+        trans_nodes = []
         for node, conn in self.cp.iterateForObject(ttid):
             logging.debug("voting transaction %s on %s", dump(ttid),
                 dump(conn.getUUID()))
             try:
-                self._askStorage(conn, packet)
+                conn.ask(packet, queue=queue)
             except ConnectionClosed:
                 continue
-            add_involved_nodes(node)
-
+            trans_nodes.append(node)
         # check at least one storage node accepted
-        if txn_context['involved_nodes']:
+        if trans_nodes:
+            involved_nodes = txn_context['involved_nodes']
+            packet = Packets.AskVoteTransaction(ttid)
+            for node in involved_nodes.difference(trans_nodes):
+                conn = self.cp.getConnForNode(node)
+                if conn is not None:
+                    try:
+                        conn.ask(packet, queue=queue)
+                    except ConnectionClosed:
+                        pass
+            involved_nodes.update(trans_nodes)
+            self.waitResponses(queue)
             txn_context['voted'] = None
             # We must not go further if connection to master was lost since
             # tpc_begin, to lower the probability of failing during tpc_finish.
@@ -719,17 +675,47 @@ class Application(object):
         self.dispatcher.forget_queue(txn_context['queue'], flush_queue=False)
 
     def tpc_finish(self, transaction, tryToResolveConflict, f=None):
-        """Finish current transaction."""
+        """Finish current transaction
+
+        To avoid inconsistencies between several databases involved in the
+        same transaction, an IStorage implementation must do its best not to
+        fail in tpc_finish. In particular, making a transaction permanent
+        should ideally be as simple as switching a bit permanently.
+
+        In NEO, all the data (with the exception of the tid, simply because
+        it is not known yet) is already flushed on disk at the end on the vote.
+        During tpc_finish, all nodes storing the transaction metadata are asked
+        to commit by saving the new tid and flushing again: for SQL backends,
+        it's just an UPDATE of 1 cell. At last, the metadata is moved to
+        a final place so that the new transaction is readable, but this is
+        something that can always be replayed (during the verification phase)
+        if any failure happens.
+
+        TODO: We should recover from master failures when the transaction got
+              successfully committed. More precisely, we should not raise:
+              - if any failure happens after all storage nodes have processed
+                successfully the LockInformation packets from the master;
+              - and if we can reconnect to the cluster to check that the ttid
+                got successfuly committed, which is possible because storage
+                nodes remember the ttid of all transactions.
+              See neo.threaded.test.Test.testStorageFailureDuringTpcFinish
+              This bug exists in ZEO.
+        """
         txn_container = self._txn_container
         if 'voted' not in txn_container.get(transaction):
             self.tpc_vote(transaction, tryToResolveConflict)
+        checked_list = []
         self._load_lock_acquire()
         try:
             # Call finish on master
             txn_context = txn_container.pop(transaction)
             cache_dict = txn_context['cache_dict']
+            checked_list = [oid for oid, data  in cache_dict.iteritems()
+                                if data is CHECKED_SERIAL]
+            for oid in checked_list:
+                del cache_dict[oid]
             tid = self._askPrimary(Packets.AskFinishTransaction(
-                txn_context['ttid'], cache_dict),
+                txn_context['ttid'], cache_dict, checked_list),
                 cache_dict=cache_dict, callback=f)
             assert tid
             return tid
@@ -952,20 +938,6 @@ class Application(object):
         self._askPrimary(Packets.AskLastTransaction())
         return self.last_tid
 
-    def __del__(self):
-        """Clear all connection."""
-        # Due to bug in ZODB, close is not always called when shutting
-        # down zope, so use __del__ to close connections
-        for conn in self.em.getConnectionList():
-            conn.close()
-        self.cp.flush()
-        self.master_conn = None
-        # Stop polling thread
-        logging.debug('Stopping %s', self.poll_thread)
-        self.poll_thread.stop()
-        psThreadedPoll()
-    close = __del__
-
     def pack(self, t):
         tid = repr(TimeStamp(*time.gmtime(t)[:5] + (t % 60, )))
         if tid == ZERO_TID:
