@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2014  Nexedi SA
+# Copyright (C) 2006-2016  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -14,24 +14,52 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import os
+from optparse import OptionParser
 from ConfigParser import SafeConfigParser, NoOptionError
 from . import util
 from .util import parseNodeAddress
 
+
+def getOptionParser():
+    parser = OptionParser()
+    parser.add_option('-l', '--logfile',
+        help='log debugging information to specified SQLite DB')
+    parser.add_option('--ca', help='certificate authority in PEM format')
+    parser.add_option('--cert', help='certificate in PEM format')
+    parser.add_option('--key', help='private key in PEM format')
+    return parser
+
+def getServerOptionParser():
+    parser = getOptionParser()
+    parser.add_option('-f', '--file', help='specify a configuration file')
+    parser.add_option('-s', '--section', help='specify a configuration section')
+    parser.add_option('-c', '--cluster', help='the cluster name')
+    parser.add_option('-m', '--masters', help='master node list')
+    parser.add_option('-b', '--bind', help='the local address to bind to')
+    parser.add_option('-D', '--dynamic-master-list',
+        help='path of the file containing dynamic master node list')
+    return parser
+
+
 class ConfigurationManager(object):
     """
     Configuration manager that load options from a configuration file and
     command line arguments
     """
 
-    def __init__(self, defaults, config_file, section, argument_list):
+    def __init__(self, defaults, options, section):
+        self.argument_list = options = {k: v
+            for k, v in options.__dict__.iteritems()
+            if v is not None}
         self.defaults = defaults
-        self.argument_list = argument_list
-        self.parser = None
-        if config_file is not None:
+        config_file = options.pop('file', None)
+        if config_file:
             self.parser = SafeConfigParser(defaults)
             self.parser.read(config_file)
-        self.section = section
+        else:
+            self.parser = None
+        self.section = options.pop('section', section)
 
     def __get(self, key, optional=False):
         value = self.argument_list.get(key)
@@ -47,6 +75,19 @@ class ConfigurationManager(object):
             raise RuntimeError("Option '%s' is undefined'" % (key, ))
         return value
 
+    def __getPath(self, *args, **kw):
+        path = self.__get(*args, **kw)
+        if path:
+            return os.path.expanduser(path)
+
+    def getLogfile(self):
+        return self.__getPath('logfile', True)
+
+    def getSSL(self):
+        r = [self.__getPath(key, True) for key in ('ca', 'cert', 'key')]
+        if any(r):
+            return r
+
     def getMasters(self):
         """ Get the master node list except itself """
         masters = self.__get('masters')
@@ -61,12 +102,15 @@ class ConfigurationManager(object):
     def getDatabase(self):
         return self.__get('database')
 
+    def getEngine(self):
+        return self.__get('engine', True)
+
     def getWait(self):
         # BUG
         return self.__get('wait')
 
     def getDynamicMasterList(self):
-        return self.__get('dynamic_master_list', optional=True)
+        return self.__getPath('dynamic_master_list', optional=True)
 
     def getAdapter(self):
         return self.__get('adapter')
@@ -97,3 +141,8 @@ class ConfigurationManager(object):
 
     def getUpstreamMasters(self):
         return util.parseMasterList(self.__get('upstream_masters'))
+
+    def getAutostart(self):
+        n = self.__get('autostart', True)
+        if n:
+            return int(n)
