@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2014  Nexedi SA
+# Copyright (C) 2006-2016  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,9 +18,7 @@ from functools import wraps
 from time import time
 
 from . import attributeTracker, logging
-from .connector import ConnectorException, ConnectorTryAgainException, \
-        ConnectorInProgressException, ConnectorConnectionRefusedException, \
-        ConnectorConnectionClosedException
+from .connector import ConnectorException, ConnectorDelayedConnection
 from .locking import RLock
 from .protocol import uuid_str, Errors, \
         PacketMalformedError, Packets, ParserState
@@ -31,55 +29,20 @@ CRITICAL_TIMEOUT = 30
 class ConnectionClosed(Exception):
     pass
 
-def not_closed(func):
-    def decorator(self, *args, **kw):
-        if self.connector is None:
-            raise ConnectorConnectionClosedException
-        return func(self, *args, **kw)
-    return wraps(func)(decorator)
-
-
-def lockCheckWrapper(func):
-    """
-    This function is to be used as a wrapper around
-    MT(Client|Server)Connection class methods.
-
-    It uses a "_" method on RLock class, so it might stop working without
-    notice (sadly, RLock does not offer any "acquired" method, but that one
-    will do as it checks that current thread holds this lock).
-
-    It requires moniroted class to have an RLock instance in self._lock
-    property.
-    """
-    def wrapper(self, *args, **kw):
-        if not self._lock._is_owned():
-            import traceback
-            logging.warning('%s called on %s instance without being locked.'
-                ' Stack:\n%s', func.func_code.co_name,
-                self.__class__.__name__, ''.join(traceback.format_stack()))
-        # Call anyway
-        return func(self, *args, **kw)
-    return wraps(func)(wrapper)
-
 class HandlerSwitcher(object):
+    _is_handling = False
     _next_timeout = None
     _next_timeout_msg_id = None
     _next_on_timeout = None
+    _pending = ({}, None),
 
     def __init__(self, handler):
         # pending handlers and related requests
         self._pending = []
-        self._is_handling = False
         self.setHandler(handler)
 
-    def clear(self):
-        self.__init__(self.getLastHandler())
-        try:
-            del (self._next_timeout,
-                 self._next_timeout_msg_id,
-                 self._next_on_timeout)
-        except AttributeError:
-            pass
+    def close(self):
+        self.__dict__.clear()
 
     def isPending(self):
         return bool(self._pending[0][0])
@@ -226,6 +189,7 @@ class BaseConnection(object):
         Timeouts in HandlerSwitcher are only there to prioritize some packets.
     """
 
+    from .connector import SocketConnector as ConnectorClass
     KEEP_ALIVE = 60
 
     def __init__(self, event_manager, handler, connector, addr=None):
@@ -234,8 +198,6 @@ class BaseConnection(object):
         self.connector = connector
         self.addr = addr
         self._handlers = HandlerSwitcher(handler)
-        event_manager.register(self)
-        event_manager.addReader(self)
 
     # XXX: do not use getHandler
     getHandler      = property(lambda self: self._handlers.getHandler)
@@ -245,14 +207,11 @@ class BaseConnection(object):
     def cancelRequests(self, *args, **kw):
         return self._handlers.cancelRequests(self, *args, **kw)
 
-    def checkTimeout(self, t):
+    def getTimeout(self):
         pass
 
-    def lock(self):
-        return 1
-
-    def unlock(self):
-        return None
+    def lockWrapper(self, func):
+        return func
 
     def getConnector(self):
         return self.connector
@@ -269,37 +228,39 @@ class BaseConnection(object):
     def close(self):
         """Close the connection."""
         if self.connector is not None:
-            em = self.em
-            em.removeReader(self)
-            em.removeWriter(self)
-            em.unregister(self)
-            self.connector.shutdown()
+            self.em.unregister(self)
             self.connector.close()
             self.connector = None
             self.aborted = False
 
+    def _getReprInfo(self):
+        r = [
+            ('uuid', uuid_str(self.getUUID())),
+            ('address', '%s:%u' % self.addr if self.addr else '?'),
+            ('handler', self.getHandler()),
+        ]
+        connector = self.connector
+        if connector is None:
+            return r, ['closed']
+        r.append(('fd', connector.getDescriptor()))
+        return r, ['aborted'] if self.isAborted() else []
+
     def __repr__(self):
-        address = self.addr and '%s:%d' % self.addr or '?'
-        return '<%s(uuid=%s, address=%s, closed=%s, handler=%s) at %x>' % (
+        r, flags = self._getReprInfo()
+        r = map('%s=%s'.__mod__, r)
+        r += flags
+        return '<%s(%s) at %x>' % (
             self.__class__.__name__,
-            uuid_str(self.getUUID()),
-            address,
-            int(self.isClosed()),
-            self.getHandler(),
+            ', '.join(r),
             id(self),
         )
 
-    __del__ = close
-
     def setHandler(self, handler):
         if self._handlers.setHandler(handler):
             logging.debug('Set handler %r on %r', handler, self)
         else:
             logging.debug('Delay handler %r on %r', handler, self)
 
-    def getEventManager(self):
-        return self.em
-
     def getUUID(self):
         return None
 
@@ -337,29 +298,28 @@ attributeTracker.track(BaseConnection)
 class ListeningConnection(BaseConnection):
     """A listen connection."""
 
-    def __init__(self, event_manager, handler, addr, connector, **kw):
+    def __init__(self, app, handler, addr):
+        self._ssl = app.ssl
         logging.debug('listening to %s:%d', *addr)
-        BaseConnection.__init__(self, event_manager, handler,
-                                addr=addr, connector=connector)
-        self.connector.makeListeningConnection(addr)
+        connector = self.ConnectorClass(addr)
+        BaseConnection.__init__(self, app.em, handler, connector, addr)
+        connector.makeListeningConnection()
+        self.em.register(self)
 
     def readable(self):
-        try:
-            new_s, addr = self.connector.getNewConnection()
-            logging.debug('accepted a connection from %s:%d', *addr)
-            handler = self.getHandler()
-            new_conn = ServerConnection(self.getEventManager(), handler,
-                connector=new_s, addr=addr)
-            handler.connectionAccepted(new_conn)
-        except ConnectorTryAgainException:
-            pass
+        connector, addr = self.connector.accept()
+        logging.debug('accepted a connection from %s:%d', *addr)
+        conn = ServerConnection(self.em, self.getHandler(), connector, addr)
+        if self._ssl:
+            conn.connecting = True
+            connector.ssl(self._ssl, conn._connected)
+            self.em.addWriter(conn)
+        else:
+            conn._connected()
 
     def getAddress(self):
         return self.connector.getAddress()
 
-    def writable(self):
-        return False
-
     def isListening(self):
         return True
 
@@ -369,16 +329,16 @@ class Connection(BaseConnection):
 
     # XXX: rename isPending, hasPendingMessages & pending methods
 
-    connecting = False
+    connecting = True
     client = False
     server = False
     peer_id = None
-    _base_timeout = None
+    _next_timeout = None
+    _timeout = 0
 
     def __init__(self, event_manager, *args, **kw):
         BaseConnection.__init__(self, event_manager, *args, **kw)
         self.read_buf = ReadBuffer()
-        self.write_buf = []
         self.cur_id = 0
         self.aborted = False
         self.uuid = None
@@ -386,6 +346,16 @@ class Connection(BaseConnection):
         self._on_close = None
         self._parser_state = ParserState()
 
+    def _getReprInfo(self):
+        r, flags = super(Connection, self)._getReprInfo()
+        if self._queue:
+            r.append(('len(queue)', len(self._queue)))
+        if self._on_close is not None:
+            r.append(('on_close', getattr(self._on_close, '__name__', '?')))
+        flags.extend(x for x in ('aborted', 'connecting', 'client', 'server')
+                       if getattr(self, x))
+        return r, flags
+
     def setOnClose(self, callback):
         self._on_close = callback
 
@@ -440,87 +410,108 @@ class Connection(BaseConnection):
 
     def updateTimeout(self, t=None):
         if not self._queue:
-            if t:
-                self._base_timeout = t
+            if not t:
+                t = self._next_timeout - self._timeout
             self._timeout = self._handlers.getNextTimeout() or self.KEEP_ALIVE
+            self._next_timeout = t + self._timeout
 
-    def checkTimeout(self, t):
-        # first make sure we don't timeout on answers we already received
-        if self._base_timeout and not self._queue:
-            timeout = t - self._base_timeout
-            if self._timeout <= timeout:
-                handlers = self._handlers
-                if handlers.isPending():
-                    msg_id = handlers.timeout(self)
-                    if msg_id is None:
-                        self._base_timeout = t
-                    else:
-                        logging.info('timeout for #0x%08x with %r',
-                                     msg_id, self)
-                        self.close()
+    def getTimeout(self):
+        if not self._queue:
+            return self._next_timeout
+
+    def onTimeout(self):
+        handlers = self._handlers
+        if handlers.isPending():
+            # It is possible that another thread used ask() while getting a
+            # timeout from epoll, so we must check again the value of
+            # _next_timeout (we know that _queue is still empty).
+            # Although this test is only useful for MTClientConnection,
+            # it's not worth complicating the code more.
+            if self._next_timeout <= time():
+                msg_id = handlers.timeout(self)
+                if msg_id is None:
+                    self._next_timeout = time() + self._timeout
                 else:
-                    self.idle()
+                    logging.info('timeout for #0x%08x with %r', msg_id, self)
+                    self.close()
+        else:
+            self.idle()
 
     def abort(self):
         """Abort dealing with this connection."""
+        assert self.pending()
+        if self.connecting:
+            self.close()
+            return
         logging.debug('aborting a connector for %r', self)
         self.aborted = True
-        assert self.write_buf
         if self._on_close is not None:
             self._on_close()
             self._on_close = None
 
     def writable(self):
         """Called when self is writable."""
-        self._send()
-        if not self.write_buf and self.connector is not None:
-            if self.aborted:
-                self.close()
-            else:
-                self.em.removeWriter(self)
+        try:
+            if self.connector.send():
+                if self.aborted:
+                    self.close()
+                else:
+                    self.em.removeWriter(self)
+        except ConnectorException:
+            self._closure()
 
     def readable(self):
         """Called when self is readable."""
-        self._recv()
-        self.analyse()
-        if self.aborted:
-            self.em.removeReader(self)
-
-    def analyse(self):
-        """Analyse received data."""
+        # last known remote activity
+        self._next_timeout = time() + self._timeout
+        read_buf = self.read_buf
         try:
-            while True:
-                packet = Packets.parse(self.read_buf, self._parser_state)
-                if packet is None:
-                    break
-                self._queue.append(packet)
+            try:
+                if self.connector.receive(read_buf):
+                    self.em.addWriter(self)
+            finally:
+                # A connector may read some data
+                # before raising ConnectorException
+                while 1:
+                    packet = Packets.parse(read_buf, self._parser_state)
+                    if packet is None:
+                        break
+                    self._queue.append(packet)
+        except ConnectorException:
+            self._closure()
         except PacketMalformedError, e:
             logging.error('malformed packet from %r: %s', self, e)
             self._closure()
+        if self.aborted:
+            self.em.removeReader(self)
+        return not not self._queue
 
     def hasPendingMessages(self):
         """
           Returns True if there are messages queued and awaiting processing.
         """
-        return len(self._queue) != 0
+        return not not self._queue
 
     def process(self):
         """
           Process a pending packet.
         """
         # check out packet and process it with current handler
-        packet = self._queue.pop(0)
-        self._handlers.handle(self, packet)
+        self._handlers.handle(self, self._queue.pop(0))
         self.updateTimeout()
 
     def pending(self):
-        return self.connector is not None and self.write_buf
+        connector = self.connector
+        return connector is not None and connector.queued
+
+    @property
+    def setReconnectionNoDelay(self):
+        return self.connector.setReconnectionNoDelay
 
     def close(self):
         if self.connector is None:
             assert self._on_close is None
             assert not self.read_buf
-            assert not self.write_buf
             assert not self.isPending()
             return
         # process the network events with the last registered handler to
@@ -531,7 +522,6 @@ class Connection(BaseConnection):
         if self._on_close is not None:
             self._on_close()
             self._on_close = None
-        del self.write_buf[:]
         self.read_buf.clear()
         try:
             if self.connecting:
@@ -540,7 +530,7 @@ class Connection(BaseConnection):
             else:
                 handler.connectionClosed(self)
         finally:
-            self._handlers.clear()
+            self._handlers.close()
 
     def _closure(self):
         assert self.connector is not None, self.whoSetConnector()
@@ -548,100 +538,45 @@ class Connection(BaseConnection):
             self._handlers.handle(self, self._queue.pop(0))
         self.close()
 
-    def _recv(self):
-        """Receive data from a connector."""
-        try:
-            data = self.connector.receive()
-        except ConnectorTryAgainException:
-            pass
-        except ConnectorConnectionRefusedException:
-            assert self.connecting
-            self._closure()
-        except ConnectorConnectionClosedException:
-            # connection resetted by peer, according to the man, this error
-            # should not occurs but it seems it's false
-            logging.debug('Connection reset by peer: %r', self.connector)
-            self._closure()
-        except:
-            logging.debug('Unknown connection error: %r', self.connector)
-            self._closure()
-            # unhandled connector exception
-            raise
-        else:
-            if not data:
-                logging.debug('Connection %r closed in recv', self.connector)
-                self._closure()
-                return
-            self._base_timeout = time() # last known remote activity
-            self.read_buf.append(data)
-
-    def _send(self):
-        """Send data to a connector."""
-        if not self.write_buf:
-            return
-        msg = ''.join(self.write_buf)
-        try:
-            n = self.connector.send(msg)
-        except ConnectorTryAgainException:
-            pass
-        except ConnectorConnectionClosedException:
-            # connection resetted by peer
-            logging.debug('Connection reset by peer: %r', self.connector)
-            self._closure()
-        except:
-            logging.debug('Unknown connection error: %r', self.connector)
-            # unhandled connector exception
-            self._closure()
-            raise
-        else:
-            if not n:
-                logging.debug('Connection %r closed in send', self.connector)
-                self._closure()
-                return
-            if n == len(msg):
-                del self.write_buf[:]
-            else:
-                self.write_buf = [msg[n:]]
-
     def _addPacket(self, packet):
         """Add a packet into the write buffer."""
-        if self.connector is None:
-            return
-
-        was_empty = not self.write_buf
-
-        self.write_buf.extend(packet.encode())
-
-        if was_empty:
+        if self.connector.queue(packet.encode()):
             # enable polling for writing.
             self.em.addWriter(self)
         logging.packet(self, packet, True)
 
-    @not_closed
     def notify(self, packet):
         """ Then a packet with a new ID """
+        if self.isClosed():
+            raise ConnectionClosed
         msg_id = self._getNextId()
         packet.setId(msg_id)
         self._addPacket(packet)
         return msg_id
 
-    @not_closed
     def ask(self, packet, timeout=CRITICAL_TIMEOUT, on_timeout=None, **kw):
         """
         Send a packet with a new ID and register the expectation of an answer
         """
+        if self.isClosed():
+            raise ConnectionClosed
         msg_id = self._getNextId()
         packet.setId(msg_id)
         self._addPacket(packet)
         handlers = self._handlers
-        t = not handlers.isPending() and time() or None
+        t = None if handlers.isPending() else time()
         handlers.emit(packet, timeout, on_timeout, kw)
-        self.updateTimeout(t)
+        if not self._queue:
+            next_timeout = self._next_timeout
+            self.updateTimeout(t)
+            if self._next_timeout < next_timeout:
+                self.em.wakeup()
         return msg_id
 
-    @not_closed
     def answer(self, packet, msg_id=None):
         """ Answer to a packet by re-using its ID for the packet answer """
+        if self.isClosed():
+            raise ConnectionClosed
         if msg_id is None:
             msg_id = self.getPeerId()
         packet.setId(msg_id)
@@ -651,45 +586,64 @@ class Connection(BaseConnection):
     def idle(self):
         self.ask(Packets.Ping())
 
+    def _connected(self):
+        self.connecting = False
+        self.getHandler().connectionCompleted(self)
+
 
 class ClientConnection(Connection):
     """A connection from this node to a remote node."""
 
-    connecting = True
     client = True
 
-    def __init__(self, event_manager, handler, node, connector):
+    def __init__(self, app, handler, node):
+        self._ssl = app.ssl
         addr = node.getAddress()
-        Connection.__init__(self, event_manager, handler, connector, addr)
+        connector = self.ConnectorClass(addr)
+        Connection.__init__(self, app.em, handler, connector, addr)
         node.setConnection(self)
         handler.connectionStarted(self)
+        self._connect()
+
+    def _connect(self):
         try:
-            try:
-                self.connector.makeClientConnection(addr)
-            except ConnectorInProgressException:
-                event_manager.addWriter(self)
-            else:
-                self._connectionCompleted()
-        except ConnectorConnectionRefusedException:
-            self._closure()
+            connected = self.connector.makeClientConnection()
+        except ConnectorDelayedConnection, c:
+            connect_limit, = c.args
+            self.getTimeout = lambda: connect_limit
+            self.onTimeout = self._delayedConnect
+            self.em.register(self, timeout_only=True)
         except ConnectorException:
-            # unhandled connector exception
             self._closure()
-            raise
+        else:
+            self.em.register(self)
+            if connected:
+                self._maybeConnected()
+                # A client connection usually has a pending packet to send
+                # from the beginning. It would be too smart to detect when
+                # it's not required to poll for writing.
+            self.em.addWriter(self)
+
+    def _delayedConnect(self):
+        del self.getTimeout, self.onTimeout
+        self._connect()
 
     def writable(self):
         """Called when self is writable."""
         if self.connector.getError():
             self._closure()
         else:
-            self._connectionCompleted()
+            self._maybeConnected()
             self.writable()
 
-    def _connectionCompleted(self):
-        self.writable = super(ClientConnection, self).writable
-        self.connecting = False
+    def _maybeConnected(self):
+        self.writable = self.lockWrapper(super(ClientConnection, self).writable)
         self.updateTimeout(time())
-        self.getHandler().connectionCompleted(self)
+        if self._ssl:
+            self.connector.ssl(self._ssl, self._connected)
+        else:
+            self._connected()
+
 
 class ServerConnection(Connection):
     """A connection from a remote node to this node."""
@@ -705,39 +659,56 @@ class ServerConnection(Connection):
 
     def __init__(self, *args, **kw):
         Connection.__init__(self, *args, **kw)
+        self.em.register(self)
         self.updateTimeout(time())
 
 
+class MTConnectionType(type):
+
+    def __init__(cls, *args):
+        if __debug__:
+            for name in 'answer',:
+                setattr(cls, name, cls.lockCheckWrapper(name))
+        for name in ('_delayedConnect', 'close', 'notify', 'onTimeout',
+                     'process', 'readable', 'writable'):
+            setattr(cls, name, cls.__class__.lockWrapper(cls, name))
+
+    def lockCheckWrapper(cls, name):
+        def wrapper(self, *args, **kw):
+            # XXX: Unfortunately, RLock does not has any public method
+            #      to test whether we own the lock or not.
+            assert self.lock._is_owned(), (self, args, kw)
+            return getattr(super(cls, self), name)(*args, **kw)
+        return wraps(getattr(cls, name).im_func)(wrapper)
+
+    def lockWrapper(cls, name):
+        def wrapper(self, *args, **kw):
+            with self.lock:
+                return getattr(super(cls, self), name)(*args, **kw)
+        return wraps(getattr(cls, name).im_func)(wrapper)
+
+
 class MTClientConnection(ClientConnection):
     """A Multithread-safe version of ClientConnection."""
 
-    def __metaclass__(name, base, d):
-        for k in ('analyse', 'answer', 'checkTimeout',
-                  'process', 'readable', 'writable'):
-            d[k] = lockCheckWrapper(getattr(base[0], k).im_func)
-        return type(name, base, d)
+    __metaclass__ = MTConnectionType
+
+    def lockWrapper(self, func):
+        lock = self.lock
+        def wrapper(*args, **kw):
+            with lock:
+                return func(*args, **kw)
+        return wrapper
 
     def __init__(self, *args, **kwargs):
-        # _lock is only here for lock debugging purposes. Do not use.
-        self._lock = lock = RLock()
-        self.lock = lock.acquire
-        self.unlock = lock.release
+        self.lock = lock = RLock()
         self.dispatcher = kwargs.pop('dispatcher')
-        self.dispatcher.needPollThread()
         with lock:
             super(MTClientConnection, self).__init__(*args, **kwargs)
 
-    def notify(self, *args, **kw):
-        self.lock()
-        try:
-            return super(MTClientConnection, self).notify(*args, **kw)
-        finally:
-            self.unlock()
-
     def ask(self, packet, timeout=CRITICAL_TIMEOUT, on_timeout=None,
             queue=None, **kw):
-        self.lock()
-        try:
+        with self.lock:
             if self.isClosed():
                 raise ConnectionClosed
             # XXX: Here, we duplicate Connection.ask because we need to call
@@ -753,16 +724,11 @@ class MTClientConnection(ClientConnection):
                 self.dispatcher.register(self, msg_id, queue)
             self._addPacket(packet)
             handlers = self._handlers
-            t = not handlers.isPending() and time() or None
+            t = None if handlers.isPending() else time()
             handlers.emit(packet, timeout, on_timeout, kw)
-            self.updateTimeout(t)
+            if not self._queue:
+                next_timeout = self._next_timeout
+                self.updateTimeout(t)
+                if self._next_timeout < next_timeout:
+                    self.em.wakeup()
             return msg_id
-        finally:
-            self.unlock()
-
-    def close(self):
-        self.lock()
-        try:
-            super(MTClientConnection, self).close()
-        finally:
-            self.unlock()
