@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2010-2014  Nexedi SA
+# Copyright (C) 2010-2016  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -44,17 +44,14 @@ def debugHandler(sig, frame):
         neo.__path__)
     imp.load_module('neo.debug', file, filename, (suffix, mode, type))
 
-def getPdb():
+def getPdb(**kw):
     try: # try ipython if available
         import IPython
-        try:
-            shell = get_ipython()
-        except NameError:
-            shell = IPython.frontend.terminal.embed.InteractiveShellEmbed()
-        return IPython.core.debugger.Pdb(shell.colors)
-    except ImportError:
+        shell = IPython.terminal.embed.InteractiveShellEmbed()
+        return IPython.core.debugger.Pdb(shell.colors, **kw)
+    except (AttributeError, ImportError):
         import pdb
-        return pdb.Pdb()
+        return pdb.Pdb(**kw)
 
 _debugger = None
 
