@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2014  Nexedi SA
+# Copyright (C) 2006-2016  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -14,22 +14,34 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import os, thread
 from time import time
 from select import epoll, EPOLLIN, EPOLLOUT, EPOLLERR, EPOLLHUP
-from errno import EINTR, EAGAIN
+from errno import EAGAIN, EEXIST, EINTR, ENOENT
 from . import logging
+from .locking import Lock
 
 class EpollEventManager(object):
     """This class manages connections and events based on epoll(5)."""
 
+    _trigger_exit = False
+
     def __init__(self):
         self.connection_dict = {}
+        # Initialize a dummy 'unregistered' for the very rare case a registered
+        # connection is closed before the first call to poll. We don't care
+        # leaking a few integers for connections closed between 2 polls.
+        self.unregistered = []
         self.reader_set = set()
         self.writer_set = set()
         self.epoll = epoll()
         self._pending_processing = []
+        self._trigger_fd, w = os.pipe()
+        os.close(w)
+        self._trigger_lock = Lock()
 
     def close(self):
+        os.close(self._trigger_fd)
         for c in self.connection_dict.values():
             c.close()
         del self.__dict__
@@ -61,10 +73,17 @@ class EpollEventManager(object):
                 append(conn)
         return result
 
-    def register(self, conn):
+    # epoll_wait always waits for EPOLLERR & EPOLLHUP so we're forced
+    # to unregister when we want to ignore all events for a connection.
+
+    def register(self, conn, timeout_only=False):
         fd = conn.getConnector().getDescriptor()
         self.connection_dict[fd] = conn
-        self.epoll.register(fd)
+        if timeout_only:
+            self.wakeup()
+        else:
+            self.epoll.register(fd)
+            self.addReader(conn)
 
     def unregister(self, conn):
         new_pending_processing = [x for x in self._pending_processing
@@ -74,8 +93,18 @@ class EpollEventManager(object):
         assert len(new_pending_processing) > len(self._pending_processing) - 2
         self._pending_processing = new_pending_processing
         fd = conn.getConnector().getDescriptor()
-        self.epoll.unregister(fd)
-        del self.connection_dict[fd]
+        try:
+            del self.connection_dict[fd]
+            self.unregistered.append(fd)
+            self.epoll.unregister(fd)
+        except KeyError:
+            pass
+        except IOError, e:
+            if e.errno != ENOENT:
+                raise
+        else:
+            self.reader_set.discard(fd)
+            self.writer_set.discard(fd)
 
     def isIdle(self):
         return not (self._pending_processing or self.writer_set)
@@ -85,89 +114,89 @@ class EpollEventManager(object):
         if conn not in pending_processing:
             pending_processing.append(conn)
 
-    def poll(self, timeout=1):
+    def poll(self, blocking=1):
         if not self._pending_processing:
             # Fetch messages from polled file descriptors
-            self._poll(timeout=timeout)
+            self._poll(blocking)
             if not self._pending_processing:
                 return
         to_process = self._pending_processing.pop(0)
-        to_process.lock()
         try:
-            try:
-                to_process.process()
-            finally:
-                # ...and requeue if there are pending messages
-                if to_process.hasPendingMessages():
-                    self._addPendingConnection(to_process)
+            to_process.process()
         finally:
-            to_process.unlock()
+            # ...and requeue if there are pending messages
+            if to_process.hasPendingMessages():
+                self._addPendingConnection(to_process)
         # Non-blocking call: as we handled a packet, we should just offer
         # poll a chance to fetch & send already-available data, but it must
         # not delay us.
-        self._poll(timeout=0)
+        self._poll(0)
 
-    def _poll(self, timeout=1):
+    def _poll(self, blocking):
+        if blocking:
+            timeout = None
+            for conn in self.connection_dict.itervalues():
+                t = conn.getTimeout()
+                if t and (timeout is None or t < timeout):
+                    timeout = t
+                    timeout_conn = conn
+            # Make sure epoll_wait does not return too early, because it has a
+            # granularity of 1ms and Python 2.7 rounds the timeout towards zero.
+            # See also https://bugs.python.org/issue20452 (fixed in Python 3).
+            blocking = .001 + max(0, timeout - time()) if timeout else -1
         try:
-            event_list = self.epoll.poll(timeout)
+            event_list = self.epoll.poll(blocking)
         except IOError, exc:
             if exc.errno in (0, EAGAIN):
                 logging.info('epoll.poll triggered undocumented error %r',
                     exc.errno)
             elif exc.errno != EINTR:
                 raise
-            event_list = ()
-        wlist = []
-        elist = []
-        for fd, event in event_list:
-            if event & EPOLLIN:
-                conn = self.connection_dict[fd]
-                conn.lock()
+            return
+        if event_list:
+            self.unregistered = unregistered = []
+            wlist = []
+            elist = []
+            for fd, event in event_list:
+                if event & EPOLLIN:
+                    conn = self.connection_dict[fd]
+                    if conn.readable():
+                        self._addPendingConnection(conn)
+                if event & EPOLLOUT:
+                    wlist.append(fd)
+                if event & (EPOLLERR | EPOLLHUP):
+                    elist.append(fd)
+            for fd in wlist:
+                if fd not in unregistered:
+                    self.connection_dict[fd].writable()
+            for fd in elist:
+                if fd in unregistered:
+                    continue
                 try:
-                    conn.readable()
-                finally:
-                    conn.unlock()
-                if conn.hasPendingMessages():
+                    conn = self.connection_dict[fd]
+                except KeyError:
+                    assert fd == self._trigger_fd, fd
+                    with self._trigger_lock:
+                        self.epoll.unregister(fd)
+                        if self._trigger_exit:
+                            del self._trigger_exit
+                            thread.exit()
+                    continue
+                if conn.readable():
                     self._addPendingConnection(conn)
-            if event & EPOLLOUT:
-                wlist.append(fd)
-            if event & (EPOLLERR | EPOLLHUP):
-                elist.append(fd)
+        elif blocking > 0:
+            logging.debug('timeout triggered for %r', timeout_conn)
+            timeout_conn.onTimeout()
 
-        for fd in wlist:
-            # This can fail, if a connection is closed in readable().
+    def wakeup(self, exit=False):
+        with self._trigger_lock:
+            self._trigger_exit |= exit
             try:
-                conn = self.connection_dict[fd]
-            except KeyError:
-                continue
-            conn.lock()
-            try:
-                conn.writable()
-            finally:
-                conn.unlock()
-
-        for fd in elist:
-            # This can fail, if a connection is closed in previous calls to
-            # readable() or writable().
-            try:
-                conn = self.connection_dict[fd]
-            except KeyError:
-                continue
-            conn.lock()
-            try:
-                conn.readable()
-            finally:
-                conn.unlock()
-            if conn.hasPendingMessages():
-                self._addPendingConnection(conn)
-
-        t = time()
-        for conn in self.connection_dict.values():
-            conn.lock()
-            try:
-                conn.checkTimeout(t)
-            finally:
-                conn.unlock()
+                self.epoll.register(self._trigger_fd)
+            except IOError, e:
+                # Ignore if 'wakeup' is called several times in a row.
+                if e.errno != EEXIST:
+                    raise
 
     def addReader(self, conn):
         connector = conn.getConnector()
