@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2014  Nexedi SA
+# Copyright (C) 2006-2016  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -124,21 +124,23 @@ class NEOLogger(Logger):
     def commit(self):
         try:
             self._db.commit()
-        except sqlite3.OperationalError, e:
+        except sqlite3.OperationalError as e:
             x = e.args[0]
-            if x == 'database is locked':
-                sys.stderr.write('%s: retrying to emit log...' % x)
-                while e.args[0] == x:
-                    try:
-                        self._db.commit()
-                    except sqlite3.OperationalError, e:
-                        continue
-                    sys.stderr.write(' ok\n')
-                    return
-            raise
+            if x != 'database is locked':
+                raise
+            sys.stderr.write('%s: retrying to emit log...' % x)
+            while 1:
+                try:
+                    self._db.commit()
+                    break
+                except sqlite3.OperationalError as e:
+                    if e.args[0] != x:
+                        raise
+            sys.stderr.write(' ok\n')
 
-    def backlog(self, max_size=1<<24):
+    def backlog(self, max_size=1<<24, max_packet=None):
         with self:
+            self._max_packet = max_packet
             self._max_size = max_size
             if max_size is None:
                 self.flush()
@@ -219,8 +221,11 @@ class NEOLogger(Logger):
             ip, port = r.addr
             peer = '%s %s (%s:%u)' % ('>' if r.outgoing else '<',
                                       uuid_str(r.uuid), ip, port)
+            msg = r.msg
+            if msg is not None:
+                msg = buffer(msg)
             self._db.execute("INSERT INTO packet VALUES (NULL,?,?,?,?,?,?)",
-                (r.created, r._name, r.msg_id, r.code, peer, buffer(r.msg)))
+                (r.created, r._name, r.msg_id, r.code, peer, msg))
         else:
             pathname = os.path.relpath(r.pathname, *neo.__path__)
             self._db.execute("INSERT INTO log VALUES (NULL,?,?,?,?,?,?)",
@@ -250,7 +255,7 @@ class NEOLogger(Logger):
             record.msg = record.getMessage()
             record.args = None
             if record.exc_info:
-                record.msg += '\n' + ''.join(
+                record.msg = (record.msg and record.msg + '\n') + ''.join(
                     format_exception(*record.exc_info)).strip()
                 record.exc_info = None
             self._queue(record)
@@ -260,6 +265,9 @@ class NEOLogger(Logger):
 
     def packet(self, connection, packet, outgoing):
         if self._db is not None:
+            body = packet._body
+            if self._max_packet and self._max_packet < len(body):
+                body = None
             self._queue(PacketRecord(
                 created=time(),
                 msg_id=packet._id,
@@ -267,7 +275,7 @@ class NEOLogger(Logger):
                 outgoing=outgoing,
                 uuid=connection.getUUID(),
                 addr=connection.getAddress(),
-                msg=packet._body))
+                msg=body))
 
 
 logging = NEOLogger()
