@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2014  Nexedi SA
+# Copyright (C) 2006-2016  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -409,6 +409,7 @@ class NodeManager(object):
             # XXX: It's probably a bug to include connecting nodes but there's
             #      no API yet to update manager when connection is established.
             if node.isConnected(connecting=True):
+                assert node in self._node_set, node
                 self._identified_dict[uuid] = node
             else:
                 self._identified_dict.pop(uuid, None)
@@ -555,13 +556,6 @@ class NodeManager(object):
     def createFromNodeType(self, node_type, **kw):
         return self._createNode(self._getClassFromNodeType(node_type), **kw)
 
-    def init(self):
-        self._node_set.clear()
-        self._type_dict.clear()
-        self._state_dict.clear()
-        self._uuid_dict.clear()
-        self._address_dict.clear()
-
     def update(self, node_list):
         for node_type, addr, uuid, state in node_list:
             # This should be done here (although klass might not be used in this
