@@ -1,5 +1,5 @@
 
-# Copyright (C) 2006-2014  Nexedi SA
+# Copyright (C) 2006-2016  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -19,12 +19,8 @@ import sys
 import traceback
 from cStringIO import StringIO
 from struct import Struct
-try:
-    from .util import getAddressType
-except ImportError:
-    pass
 
-PROTOCOL_VERSION = 2
+PROTOCOL_VERSION = 5
 
 # Size restrictions.
 MIN_PACKET_SIZE = 10
@@ -449,65 +445,6 @@ class PEnum(PStructItem):
             enum = self._enum.__class__.__name__
             raise ValueError, 'Invalid code for %s enum: %r' % (enum, code)
 
-class PAddressIPGeneric(PStructItem):
-
-    def __init__(self, name, format):
-        PStructItem.__init__(self, name, format)
-
-    def encode(self, writer, address):
-        host, port = address
-        host = socket.inet_pton(self.af_type, host)
-        writer(self.pack(host, port))
-
-    def decode(self, reader):
-        data = reader(self.size)
-        address = self.unpack(data)
-        host, port = address
-        host =  socket.inet_ntop(self.af_type, host)
-        return (host, port)
-
-class PAddressIPv4(PAddressIPGeneric):
-    af_type = socket.AF_INET
-
-    def __init__(self, name):
-        PAddressIPGeneric.__init__(self, name, '!4sH')
-
-class PAddressIPv6(PAddressIPGeneric):
-    af_type = socket.AF_INET6
-
-    def __init__(self, name):
-        PAddressIPGeneric.__init__(self, name, '!16sH')
-
-class PAddress(PStructItem):
-    """
-        An host address (IPv4/IPv6)
-    """
-
-    address_format_dict = {
-        socket.AF_INET: PAddressIPv4('ipv4'),
-        socket.AF_INET6: PAddressIPv6('ipv6'),
-    }
-
-    def __init__(self, name):
-        PStructItem.__init__(self, name, '!L')
-
-    def _encode(self, writer, address):
-        if address is None:
-            writer(self.pack(INVALID_ADDRESS_TYPE))
-            return
-        af_type = getAddressType(address)
-        writer(self.pack(af_type))
-        encoder = self.address_format_dict[af_type]
-        encoder.encode(writer, address)
-
-    def _decode(self, reader):
-        af_type  = self.unpack(reader(self.size))[0]
-        if af_type == INVALID_ADDRESS_TYPE:
-            return None
-        decoder = self.address_format_dict[af_type]
-        host, port = decoder.decode(reader)
-        return (host, port)
-
 class PString(PStructItem):
     """
         A variable-length string
@@ -523,6 +460,29 @@ class PString(PStructItem):
         length = self.unpack(reader(self.size))[0]
         return reader(length)
 
+class PAddress(PString):
+    """
+        An host address (IPv4/IPv6)
+    """
+
+    def __init__(self, name):
+        PString.__init__(self, name)
+        self._port = Struct('!H')
+
+    def _encode(self, writer, address):
+        if address:
+            host, port = address
+            PString._encode(self, writer, host)
+            writer(self._port.pack(port))
+        else:
+            PString._encode(self, writer, '')
+
+    def _decode(self, reader):
+        host = PString._decode(self, reader)
+        if host:
+            p = self._port
+            return host, p.unpack(reader(p.size))[0]
+
 class PBoolean(PStructItem):
     """
         A boolean value, encoded as a single byte
@@ -762,16 +722,24 @@ class ReelectPrimary(Packet):
     Force a re-election of a primary master node. M -> M.
     """
 
+class Recovery(Packet):
+    """
+    Ask all data needed by master to recover. PM -> S, S -> PM.
+    """
+    _answer = PStruct('answer_recovery',
+        PPTID('ptid'),
+        PTID('backup_tid'),
+        PTID('truncate_tid'),
+    )
+
 class LastIDs(Packet):
     """
-    Ask the last OID, the last TID and the last Partition Table ID so that
-    a master recover. PM -> S, S -> PM.
+    Ask the last OID/TID so that a master can initialize its TransactionManager.
+    PM -> S, S -> PM.
     """
     _answer = PStruct('answer_last_ids',
         POID('last_oid'),
         PTID('last_tid'),
-        PPTID('last_ptid'),
-        PTID('backup_tid'),
     )
 
 class PartitionTable(Packet):
@@ -815,6 +783,8 @@ class StartOperation(Packet):
     this message, it must not serve client nodes. PM -> S.
     """
     _fmt = PStruct('start_operation',
+        # XXX: Is this boolean needed ? Maybe this
+        #      can be deduced from cluster state.
         PBoolean('backup'),
     )
 
@@ -826,8 +796,8 @@ class StopOperation(Packet):
 
 class UnfinishedTransactions(Packet):
     """
-    Ask unfinished transactions  PM -> S.
-    Answer unfinished transactions  S -> PM.
+    Ask unfinished transactions  S -> PM.
+    Answer unfinished transactions  PM -> S.
     """
     _answer = PStruct('answer_unfinished_transactions',
         PTID('max_tid'),
@@ -836,36 +806,36 @@ class UnfinishedTransactions(Packet):
         ),
     )
 
-class ObjectPresent(Packet):
+class LockedTransactions(Packet):
     """
-    Ask if an object is present. If not present, OID_NOT_FOUND should be
-    returned. PM -> S.
-    Answer that an object is present. PM -> S.
+    Ask locked transactions  PM -> S.
+    Answer locked transactions  S -> PM.
     """
-    _fmt = PStruct('object_present',
-        POID('oid'),
+    _answer = PStruct('answer_locked_transactions',
+        PDict('tid_dict',
+            PTID('ttid'),
+            PTID('tid'),
+        ),
+    )
+
+class FinalTID(Packet):
+    """
+    Return final tid if ttid has been committed. * -> S.
+    """
+    _fmt = PStruct('final_tid',
+        PTID('ttid'),
+    )
+
+    _answer = PStruct('final_tid',
         PTID('tid'),
     )
 
-    _answer = PStruct('object_present',
-        POID('oid'),
-        PTID('tid'),
-    )
-
-class DeleteTransaction(Packet):
-    """
-    Delete a transaction. PM -> S.
-    """
-    _fmt = PStruct('delete_transaction',
-        PTID('tid'),
-        PFOidList,
-    )
-
-class CommitTransaction(Packet):
+class ValidateTransaction(Packet):
     """
     Commit a transaction. PM -> S.
     """
-    _fmt = PStruct('commit_transaction',
+    _fmt = PStruct('validate_transaction',
+        PTID('ttid'),
         PTID('tid'),
     )
 
@@ -890,6 +860,9 @@ class FinishTransaction(Packet):
     _fmt = PStruct('ask_finish_transaction',
         PTID('tid'),
         PFOidList,
+        PList('checked_list',
+            POID('oid'),
+        ),
     )
 
     _answer = PStruct('answer_information_locked',
@@ -915,11 +888,10 @@ class LockInformation(Packet):
     _fmt = PStruct('ask_lock_informations',
         PTID('ttid'),
         PTID('tid'),
-        PFOidList,
     )
 
     _answer = PStruct('answer_information_locked',
-        PTID('tid'),
+        PTID('ttid'),
     )
 
 class InvalidateObjects(Packet):
@@ -936,7 +908,7 @@ class UnlockInformation(Packet):
     Unlock information on a transaction. PM -> S.
     """
     _fmt = PStruct('notify_unlock_information',
-        PTID('tid'),
+        PTID('ttid'),
     )
 
 class GenerateOIDs(Packet):
@@ -998,10 +970,17 @@ class StoreTransaction(Packet):
         PString('extension'),
         PFOidList,
     )
+    _answer = PFEmpty
 
-    _answer = PStruct('answer_store_transaction',
+class VoteTransaction(Packet):
+    """
+    Ask to store a transaction. C -> S.
+    Answer if transaction has been stored. S -> C.
+    """
+    _fmt = PStruct('ask_vote_transaction',
         PTID('tid'),
     )
+    _answer = PFEmpty
 
 class GetObject(Packet):
     """
@@ -1469,7 +1448,13 @@ class AddObject(Packet):
 
 class Replicate(Packet):
     """
+    Notify a storage node to replicate partitions up to given 'tid'
+    and from given sources.
     M -> S
+
+    - upstream_name: replicate from an upstream cluster
+    - address: address of the source storage node, or None if there's no new
+               data up to 'tid' for the given partition
     """
     _fmt = PStruct('replicate',
         PTID('tid'),
@@ -1493,13 +1478,14 @@ class ReplicationDone(Packet):
 
 class Truncate(Packet):
     """
-    XXX: Used for both make storage consistent and leave backup mode
-    M -> S
+    Request DB to be truncated. Also used to leave backup mode.
     """
     _fmt = PStruct('truncate',
         PTID('tid'),
     )
 
+    _answer = Error
+
 
 StaticRegistry = {}
 def register(request, ignore_when_closed=None):
@@ -1617,6 +1603,8 @@ class Packets(dict):
                     ReelectPrimary)
     NotifyNodeInformation = register(
                     NotifyNodeInformation)
+    AskRecovery, AnswerRecovery = register(
+                    Recovery)
     AskLastIDs, AnswerLastIDs = register(
                     LastIDs)
     AskPartitionTable, AnswerPartitionTable = register(
@@ -1631,12 +1619,12 @@ class Packets(dict):
                     StopOperation)
     AskUnfinishedTransactions, AnswerUnfinishedTransactions = register(
                     UnfinishedTransactions)
-    AskObjectPresent, AnswerObjectPresent = register(
-                    ObjectPresent)
-    DeleteTransaction = register(
-                    DeleteTransaction)
-    CommitTransaction = register(
-                    CommitTransaction)
+    AskLockedTransactions, AnswerLockedTransactions = register(
+                    LockedTransactions)
+    AskFinalTID, AnswerFinalTID = register(
+                    FinalTID)
+    ValidateTransaction = register(
+                    ValidateTransaction)
     AskBeginTransaction, AnswerBeginTransaction = register(
                     BeginTransaction)
     AskFinishTransaction, AnswerTransactionFinished = register(
@@ -1655,6 +1643,8 @@ class Packets(dict):
                     AbortTransaction)
     AskStoreTransaction, AnswerStoreTransaction = register(
                     StoreTransaction)
+    AskVoteTransaction, AnswerVoteTransaction = register(
+                    VoteTransaction)
     AskObject, AnswerObject = register(
                     GetObject)
     AskTIDs, AnswerTIDs = register(
