@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2014  Nexedi SA
+# Copyright (C) 2006-2016  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,15 +18,13 @@ import sys, weakref
 from time import time
 
 from neo.lib import logging
-from neo.lib.connector import getConnectorHandler
+from neo.lib.app import BaseApplication
 from neo.lib.debug import register as registerLiveDebugger
 from neo.lib.protocol import uuid_str, UUID_NAMESPACES, ZERO_TID
 from neo.lib.protocol import ClusterStates, NodeStates, NodeTypes, Packets
-from neo.lib.node import NodeManager
-from neo.lib.event import EventManager
 from neo.lib.handler import EventHandler
 from neo.lib.connection import ListeningConnection, ClientConnection
-from neo.lib.exception import ElectionFailure, PrimaryFailure, OperationFailure
+from neo.lib.exception import ElectionFailure, PrimaryFailure, StoppedOperation
 
 class StateChangedException(Exception): pass
 
@@ -39,7 +37,7 @@ from .transactions import TransactionManager
 from .verification import VerificationManager
 
 
-class Application(object):
+class Application(BaseApplication):
     """The master node application."""
     packing = None
     # Latest completely commited TID
@@ -47,20 +45,19 @@ class Application(object):
     backup_tid = None
     backup_app = None
     uuid = None
+    truncate_tid = None
 
     def __init__(self, config):
-        # Internal attributes.
-        self.em = EventManager()
-        self.nm = NodeManager(config.getDynamicMasterList())
+        super(Application, self).__init__(
+            config.getSSL(), config.getDynamicMasterList())
         self.tm = TransactionManager(self.onTransactionCommitted)
 
         self.name = config.getCluster()
         self.server = config.getBind()
+        self.autostart = config.getAutostart()
 
         self.storage_readiness = set()
-        master_addresses, connector_name = config.getMasters()
-        self.connector_handler = getConnectorHandler(connector_name)
-        for master_address in master_addresses:
+        for master_address in config.getMasters():
             self.nm.createMaster(address=master_address)
 
         logging.debug('IP address is %s, port is %d', *self.server)
@@ -81,7 +78,6 @@ class Application(object):
         self.primary = None
         self.primary_master_node = None
         self.cluster_state = None
-        self._startup_allowed = False
 
         uuid = config.getUUID()
         if uuid:
@@ -101,7 +97,7 @@ class Application(object):
                 raise ValueError("upstream cluster name must be"
                                  " different from cluster name")
             self.backup_app = BackupApplication(self, upstream_cluster,
-                                                *config.getUpstreamMasters())
+                                                config.getUpstreamMasters())
 
         self.administration_handler = administration.AdministrationHandler(
             self)
@@ -115,9 +111,7 @@ class Application(object):
         self.listening_conn = None
         if self.backup_app is not None:
             self.backup_app.close()
-        self.nm.close()
-        self.em.close()
-        del self.__dict__
+        super(Application, self).close()
 
     def log(self):
         self.em.log()
@@ -134,13 +128,13 @@ class Application(object):
         except Exception:
             logging.exception('Pre-mortem data:')
             self.log()
+            logging.flush()
             raise
 
     def _run(self):
         """Make sure that the status is sane and start a loop."""
         # Make a listening port.
-        self.listening_conn = ListeningConnection(self.em, None,
-            addr=self.server, connector=self.connector_handler())
+        self.listening_conn = ListeningConnection(self, None, self.server)
 
         # Start a normal operation.
         while self.cluster_state != ClusterStates.STOPPING:
@@ -190,12 +184,11 @@ class Application(object):
                 while (self.unconnected_master_node_set or
                         self.negotiating_master_node_set):
                     for addr in self.unconnected_master_node_set:
-                        ClientConnection(self.em, client_handler,
+                        self.negotiating_master_node_set.add(addr)
+                        ClientConnection(self, client_handler,
                             # XXX: Ugly, but the whole election code will be
                             # replaced soon
-                            node=getByAddress(addr),
-                            connector=self.connector_handler())
-                        self.negotiating_master_node_set.add(addr)
+                            getByAddress(addr))
                     self.unconnected_master_node_set.clear()
                     self.em.poll(1)
             except ElectionFailure, m:
@@ -210,6 +203,9 @@ class Application(object):
                 # Wait until the connections are closed.
                 self.primary = None
                 self.primary_master_node = None
+                # XXX: Since poll does not wake up anymore every second,
+                #      the following time condition should be reviewed.
+                #      See also playSecondaryRole.
                 t = time() + 10
                 while self.em.getClientList() and time() < t:
                     try:
@@ -225,7 +221,7 @@ class Application(object):
                 self.primary = self.primary is None
                 break
 
-    def broadcastNodesInformation(self, node_list):
+    def broadcastNodesInformation(self, node_list, exclude=None):
         """
           Broadcast changes for a set a nodes
           Send only one packet per connection to reduce bandwidth
@@ -247,7 +243,7 @@ class Application(object):
         # send at most one non-empty notification packet per node
         for node in self.nm.getIdentifiedList():
             node_list = node_dict.get(node.getType())
-            if node_list and node.isRunning():
+            if node_list and node.isRunning() and node is not exclude:
                 node.notify(Packets.NotifyNodeInformation(node_list))
 
     def broadcastPartitionChanges(self, cell_list):
@@ -258,7 +254,6 @@ class Application(object):
             ptid = self.pt.setNextID()
             packet = Packets.NotifyPartitionChanges(ptid, cell_list)
             for node in self.nm.getIdentifiedList():
-                # TODO: notify masters
                 if node.isRunning() and not node.isMaster():
                     node.notify(packet)
 
@@ -270,8 +265,6 @@ class Application(object):
         """
         logging.info('provide service')
         poll = self.em.poll
-        self.tm.reset()
-
         self.changeClusterState(ClusterStates.RUNNING)
 
         # Now everything is passive.
@@ -282,8 +275,13 @@ class Application(object):
             if e.args[0] != ClusterStates.STARTING_BACKUP:
                 raise
             self.backup_tid = tid = self.getLastTransaction()
-            self.pt.setBackupTidDict({node.getUUID(): tid
-                for node in self.nm.getStorageList(only_identified=True)})
+            packet = Packets.StartOperation(True)
+            tid_dict = {}
+            for node in self.nm.getStorageList(only_identified=True):
+                tid_dict[node.getUUID()] = tid
+                if node.isRunning():
+                    node.notify(packet)
+            self.pt.setBackupTidDict(tid_dict)
 
     def playPrimaryRole(self):
         logging.info('play the primary role with %r', self.listening_conn)
@@ -327,30 +325,46 @@ class Application(object):
                     in_conflict)
                 in_conflict.setUUID(None)
 
-        # recover the cluster status at startup
+        # Do not restart automatically if ElectionFailure is raised, in order
+        # to avoid a split of the database. For example, with 2 machines with
+        # a master and a storage on each one and replicas=1, the secondary
+        # master becomes primary in case of network failure between the 2
+        # machines but must not start automatically: otherwise, each storage
+        # node would diverge.
+        self._startup_allowed = False
         try:
-            self.runManager(RecoveryManager)
             while True:
-                self.runManager(VerificationManager)
+                self.runManager(RecoveryManager)
                 try:
-                    if self.backup_tid:
-                        if self.backup_app is None:
-                            raise RuntimeError("No upstream cluster to backup"
-                                               " defined in configuration")
-                        self.backup_app.provideService()
-                        # Reset connection with storages (and go through a
-                        # recovery phase) when leaving backup mode in order
-                        # to get correct last oid/tid.
-                        self.runManager(RecoveryManager)
-                        continue
-                    self.provideService()
-                except OperationFailure:
+                    self.runManager(VerificationManager)
+                    if not self.backup_tid:
+                        self.provideService()
+                        # self.provideService only returns without raising
+                        # when switching to backup mode.
+                    if self.backup_app is None:
+                        raise RuntimeError("No upstream cluster to backup"
+                                           " defined in configuration")
+                    truncate = Packets.Truncate(
+                        self.backup_app.provideService())
+                except StoppedOperation, e:
                     logging.critical('No longer operational')
+                    truncate = Packets.Truncate(*e.args) if e.args else None
+                    # Automatic restart except if we truncate or retry to.
+                    self._startup_allowed = not (self.truncate_tid or truncate)
+                node_list = []
                 for node in self.nm.getIdentifiedList():
                     if node.isStorage() or node.isClient():
-                        node.notify(Packets.StopOperation())
+                        conn = node.getConnection()
+                        conn.notify(Packets.StopOperation())
                         if node.isClient():
-                            node.getConnection().abort()
+                            conn.abort()
+                            continue
+                        if truncate:
+                            conn.notify(truncate)
+                        if node.isRunning():
+                            node.setPending()
+                            node_list.append(node)
+                self.broadcastNodesInformation(node_list)
         except StateChangedException, e:
             assert e.args[0] == ClusterStates.STOPPING
             self.shutdown()
@@ -363,6 +377,7 @@ class Application(object):
 
         # Wait for an announcement. If this is too long, probably
         # the primary master is down.
+        # XXX: Same remark as in electPrimary.
         t = time() + 10
         while self.primary_master_node is None:
             self.em.poll(1)
@@ -379,9 +394,7 @@ class Application(object):
 
         # Reconnect to primary master node.
         primary_handler = secondary.PrimaryHandler(self)
-        ClientConnection(self.em, primary_handler,
-            node=self.primary_master_node,
-            connector=self.connector_handler())
+        ClientConnection(self, primary_handler, self.primary_master_node)
 
         # and another for the future incoming connections
         self.listening_conn.setHandler(
@@ -392,8 +405,10 @@ class Application(object):
 
     def runManager(self, manager_klass):
         self._current_manager = manager_klass(self)
-        self._current_manager.run()
-        self._current_manager = None
+        try:
+            self._current_manager.run()
+        finally:
+            self._current_manager = None
 
     def changeClusterState(self, state):
         """
@@ -430,7 +445,7 @@ class Application(object):
                 continue # keep handler
             if type(handler) is not type(conn.getLastHandler()):
                 conn.setHandler(handler)
-                handler.connectionCompleted(conn)
+                handler.connectionCompleted(conn, new=False)
         self.cluster_state = state
 
     def getNewUUID(self, uuid, address, node_type):
@@ -464,7 +479,7 @@ class Application(object):
             # wait for all transaction to be finished
             while self.tm.hasPending():
                 self.em.poll(1)
-        except OperationFailure:
+        except StoppedOperation:
             logging.critical('No longer operational')
 
         logging.info("asking remaining nodes to shutdown")
