@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2014  Nexedi SA
+# Copyright (C) 2006-2016  Nexedi SA
 
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -20,9 +20,6 @@ from . import MasterHandler
 class ClientServiceHandler(MasterHandler):
     """ Handler dedicated to client during service state """
 
-    def connectionCompleted(self, conn):
-        pass
-
     def connectionLost(self, conn, new_state):
         # cancel its transactions and forgot the node
         app = self.app
@@ -54,13 +51,15 @@ class ClientServiceHandler(MasterHandler):
     def askNewOIDs(self, conn, num_oids):
         conn.answer(Packets.AnswerNewOIDs(self.app.tm.getNextOIDList(num_oids)))
 
-    def askFinishTransaction(self, conn, ttid, oid_list):
+    def askFinishTransaction(self, conn, ttid, oid_list, checked_list):
         app = self.app
         pt = app.pt
 
         # Collect partitions related to this transaction.
-        partition_set = set(map(pt.getPartition, oid_list))
-        partition_set.add(pt.getPartition(ttid))
+        getPartition = pt.getPartition
+        partition_set = set(map(getPartition, oid_list))
+        partition_set.update(map(getPartition, checked_list))
+        partition_set.add(getPartition(ttid))
 
         # Collect the UUIDs of nodes related to this transaction.
         uuid_list = filter(app.isStorageReady, {cell.getUUID()
@@ -84,7 +83,6 @@ class ClientServiceHandler(MasterHandler):
                 {x.getUUID() for x in identified_node_list},
                 conn.getPeerId(),
             ),
-            oid_list,
         )
         for node in identified_node_list:
             node.ask(p, timeout=60)
