@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2014  Nexedi SA
+# Copyright (C) 2006-2016  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,7 +17,7 @@
 from time import time
 from struct import pack, unpack
 from neo.lib import logging
-from neo.lib.protocol import ProtocolError, uuid_str, ZERO_TID
+from neo.lib.protocol import ProtocolError, uuid_str, ZERO_OID, ZERO_TID
 from neo.lib.util import dump, u64, addTID, tidFromTime
 
 class DelayedError(Exception):
@@ -155,15 +155,18 @@ class TransactionManager(object):
     """
         Manage current transactions
     """
-    _last_tid = ZERO_TID
 
     def __init__(self, on_commit):
+        self._on_commit = on_commit
+        self.reset()
+
+    def reset(self):
         # ttid -> transaction
         self._ttid_dict = {}
         # node -> transactions mapping
         self._node_dict = {}
-        self._last_oid = None
-        self._on_commit = on_commit
+        self._last_oid = ZERO_OID
+        self._last_tid = ZERO_TID
         # queue filled with ttids pointing to transactions with increasing tids
         self._queue = []
 
@@ -182,15 +185,14 @@ class TransactionManager(object):
 
     def getNextOIDList(self, num_oids):
         """ Generate a new OID list """
-        if self._last_oid is None:
-            raise RuntimeError, 'I do not know the last OID'
         oid = unpack('!Q', self._last_oid)[0] + 1
         oid_list = [pack('!Q', oid + i) for i in xrange(num_oids)]
         self._last_oid = oid_list[-1]
         return oid_list
 
     def setLastOID(self, oid):
-        self._last_oid = max(oid, self._last_oid)
+        if self._last_oid < oid:
+            self._last_oid = oid
 
     def getLastOID(self):
         return self._last_oid
@@ -245,15 +247,8 @@ class TransactionManager(object):
         """
             Set the last TID, keep the previous if lower
         """
-        self._last_tid = max(self._last_tid, tid)
-
-    def reset(self):
-        """
-            Discard all manager content
-            This doesn't reset the last TID.
-        """
-        self._ttid_dict = {}
-        self._node_dict = {}
+        if self._last_tid < tid:
+            self._last_tid = tid
 
     def hasPending(self):
         """
@@ -340,9 +335,7 @@ class TransactionManager(object):
             instanciation time.
         """
         logging.debug('Lock TXN %s for %s', dump(ttid), uuid_str(uuid))
-        assert ttid in self._ttid_dict, "Transaction not started"
-        txn = self._ttid_dict[ttid]
-        if txn.lock(uuid) and self._queue[0][1] == ttid:
+        if self._ttid_dict[ttid].lock(uuid) and self._queue[0][1] == ttid:
             # all storage are locked and we unlock the commit queue
             self._unlockPending()
 
@@ -352,25 +345,28 @@ class TransactionManager(object):
             current transactions
         """
         unlock = False
-        # iterate over a copy because _unlockPending may alter the dict
-        for ttid, txn in self._ttid_dict.items():
+        for ttid, txn in self._ttid_dict.iteritems():
             if txn.forget(uuid) and self._queue[0][1] == ttid:
                 unlock = True
         if unlock:
             self._unlockPending()
 
     def _unlockPending(self):
-        # unlock pending transactions
+        """Serialize transaction unlocks
+
+        This should rarely delay unlocks since the time needed to lock a
+        transaction is roughly constant. The most common case where reordering
+        is required is when some storages are already busy by other tasks.
+        """
         queue = self._queue
         pop = queue.pop
         insert = queue.insert
         on_commit = self._on_commit
-        get = self._ttid_dict.get
+        ttid_dict = self._ttid_dict
         while queue:
             uuid, ttid = pop(0)
-            txn = get(ttid, None)
-            # _queue can contain un-prepared transactions
-            if txn is not None and txn.locked():
+            txn = ttid_dict[ttid]
+            if txn.locked():
                 on_commit(txn)
             else:
                 insert(0, (uuid, ttid))
@@ -380,6 +376,10 @@ class TransactionManager(object):
         """
             Abort pending transactions initiated by a node
         """
+        # BUG: As soon as we have started to lock a transaction,
+        #      we should complete it even if the client is lost.
+        #      Of course, we won't reply to the FinishTransaction
+        #      finish but we'll send invalidations to all clients.
         logging.debug('Abort TXN for %s', node)
         uuid = node.getUUID()
         # XXX: this loop is usefull only during an import
