@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2014  Nexedi SA
+# Copyright (C) 2006-2016  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -37,6 +37,7 @@ action_dict = {
     'tweak': 'tweakPartitionTable',
     'drop': 'dropNode',
     'kill': 'killNode',
+    'truncate': 'truncate',
 }
 
 uuid_int = (lambda ns: lambda uuid:
@@ -44,8 +45,8 @@ uuid_int = (lambda ns: lambda uuid:
     )({str(k)[0]: v for k, v in UUID_NAMESPACES.iteritems()})
 
 class TerminalNeoCTL(object):
-    def __init__(self, address):
-        self.neoctl = NeoCTL(address)
+    def __init__(self, *args, **kw):
+        self.neoctl = NeoCTL(*args, **kw)
 
     def __del__(self):
         self.neoctl.close()
@@ -85,11 +86,14 @@ class TerminalNeoCTL(object):
           Get last ids.
         """
         assert not params
-        r = self.neoctl.getLastIds()
-        if r[3]:
-            return "last_tid = 0x%x" % u64(self.neoctl.getLastTransaction())
-        return "last_oid = 0x%x\nlast_tid = 0x%x\nlast_ptid = %u" % (
-            u64(r[0]), u64(r[1]), r[2])
+        ptid, backup_tid, truncate_tid = self.neoctl.getRecovery()
+        if backup_tid:
+            ltid = self.neoctl.getLastTransaction()
+            r = "backup_tid = 0x%x" % u64(backup_tid)
+        else:
+            loid, ltid = self.neoctl.getLastIds()
+            r = "last_oid = 0x%x" % u64(loid)
+        return r + "\nlast_tid = 0x%x\nlast_ptid = %u" % (u64(ltid), ptid)
 
     def getPartitionRowList(self, params):
         """
@@ -193,6 +197,19 @@ class TerminalNeoCTL(object):
         """
         return uuid_str(self.neoctl.getPrimary())
 
+    def truncate(self, params):
+        """
+          Truncate the database at the given tid.
+
+          The cluster must be in RUNNING state, without any pending transaction.
+          This causes the cluster to go back in RECOVERING state, waiting all
+          nodes to be pending (do not use 'start' command unless you're sure
+          the missing nodes don't need to be truncated).
+
+          Parameters: tid
+        """
+        self.neoctl.truncate(self.asTID(*params))
+
     def checkReplicas(self, params):
         """
           Test whether partitions have corrupted metadata
@@ -231,8 +248,8 @@ class TerminalNeoCTL(object):
 class Application(object):
     """The storage node application."""
 
-    def __init__(self, address):
-        self.neoctl = TerminalNeoCTL(address)
+    def __init__(self, *args, **kw):
+        self.neoctl = TerminalNeoCTL(*args, **kw)
 
     def execute(self, args):
         """Execute the command given."""
