@@ -2,7 +2,7 @@
 #
 # neoadmin - run an administrator node of NEO
 #
-# Copyright (C) 2009-2014  Nexedi SA
+# Copyright (C) 2009-2016  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,15 +17,14 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from optparse import OptionParser
 from neo.lib import logging
+from neo.lib.config import getOptionParser
 from neo.lib.util import parseNodeAddress
 
-parser = OptionParser()
+parser = getOptionParser()
 parser.add_option('-a', '--address', help = 'specify the address (ip:port) ' \
     'of an admin node', default = '127.0.0.1:9999')
 parser.add_option('--handler', help = 'specify the connection handler')
-parser.add_option('-l', '--logfile', help = 'specify a logging file')
 
 def main(args=None):
     (options, args) = parser.parse_args(args=args)
@@ -34,8 +33,16 @@ def main(args=None):
     else:
         address = ('127.0.0.1', 9999)
 
-    logging.setup(options.logfile)
+    if options.logfile:
+        # Contrary to daemons, we log everything to disk automatically
+        # because a user using -l option here:
+        # - is certainly debugging an issue and wants everything,
+        # - would not have to time to send SIGRTMIN before neoctl exits.
+        logging.backlog(None)
+        logging.setup(options.logfile)
     from neo.neoctl.app import Application
 
-    print Application(address).execute(args)
-
+    ssl = options.ca, options.cert, options.key
+    r = Application(address, ssl=ssl if any(ssl) else None).execute(args)
+    if r is not None:
+        print r
