@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #
-# Copyright (C) 2009-2014  Nexedi SA
+# Copyright (C) 2009-2016  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -34,7 +34,6 @@ UNIT_TEST_MODULES = [
     # generic parts
     'neo.tests.testBootstrap',
     'neo.tests.testConnection',
-    'neo.tests.testEvent',
     'neo.tests.testHandler',
     'neo.tests.testNodes',
     'neo.tests.testDispatcher',
@@ -47,7 +46,6 @@ UNIT_TEST_MODULES = [
     'neo.tests.master.testMasterPT',
     'neo.tests.master.testRecovery',
     'neo.tests.master.testStorageHandler',
-    'neo.tests.master.testVerification',
     'neo.tests.master.testTransactions',
     # storage application
     'neo.tests.storage.testClientHandler',
@@ -55,7 +53,6 @@ UNIT_TEST_MODULES = [
     'neo.tests.storage.testMasterHandler',
     'neo.tests.storage.testStorageApp',
     'neo.tests.storage.testStorage' + os.getenv('NEO_TESTS_ADAPTER', 'SQLite'),
-    'neo.tests.storage.testVerificationHandler',
     'neo.tests.storage.testIdentificationHandler',
     'neo.tests.storage.testTransactions',
     # client application
@@ -67,6 +64,7 @@ UNIT_TEST_MODULES = [
     'neo.tests.threaded.test',
     'neo.tests.threaded.testImporter',
     'neo.tests.threaded.testReplication',
+    'neo.tests.threaded.testSSL',
 ]
 
 FUNC_TEST_MODULES = [
