@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2014  Nexedi SA
+# Copyright (C) 2006-2016  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,45 +18,41 @@ import sys
 from collections import deque
 
 from neo.lib import logging
+from neo.lib.app import BaseApplication
 from neo.lib.protocol import uuid_str, \
     CellStates, ClusterStates, NodeTypes, Packets
 from neo.lib.node import NodeManager
-from neo.lib.event import EventManager
 from neo.lib.connection import ListeningConnection
-from neo.lib.exception import OperationFailure, PrimaryFailure
-from neo.lib.connector import getConnectorHandler
+from neo.lib.exception import StoppedOperation, PrimaryFailure
 from neo.lib.pt import PartitionTable
 from neo.lib.util import dump
 from neo.lib.bootstrap import BootstrapManager
 from .checker import Checker
 from .database import buildDatabaseManager
 from .exception import AlreadyPendingError
-from .handlers import identification, verification, initialization
+from .handlers import identification, initialization
 from .handlers import master, hidden
 from .replicator import Replicator
 from .transactions import TransactionManager
 
 from neo.lib.debug import register as registerLiveDebugger
 
-class Application(object):
+class Application(BaseApplication):
     """The storage node application."""
 
     def __init__(self, config):
+        super(Application, self).__init__(
+            config.getSSL(), config.getDynamicMasterList())
         # set the cluster name
         self.name = config.getCluster()
 
-        # Internal attributes.
-        self.em = EventManager()
-        self.nm = NodeManager(config.getDynamicMasterList())
         self.tm = TransactionManager(self)
         self.dm = buildDatabaseManager(config.getAdapter(),
-            (config.getDatabase(), config.getWait())
+            (config.getDatabase(), config.getEngine(), config.getWait()),
         )
 
         # load master nodes
-        master_addresses, connector_name = config.getMasters()
-        self.connector_handler = getConnectorHandler(connector_name)
-        for master_address in master_addresses :
+        for master_address in config.getMasters():
             self.nm.createMaster(address=master_address)
 
         # set the bind address
@@ -92,13 +88,8 @@ class Application(object):
 
     def close(self):
         self.listening_conn = None
-        self.nm.close()
-        self.em.close()
-        try:
-            self.dm.close()
-        except AttributeError:
-            pass
-        del self.__dict__
+        self.dm.close()
+        super(Application, self).close()
 
     def _poll(self):
         self.em.poll(1)
@@ -167,6 +158,7 @@ class Application(object):
         except Exception:
             logging.exception('Pre-mortem data:')
             self.log()
+            logging.flush()
             raise
 
     def _run(self):
@@ -176,8 +168,7 @@ class Application(object):
 
         # Make a listening port
         handler = identification.IdentificationHandler(self)
-        self.listening_conn = ListeningConnection(self.em, handler,
-            addr=self.server, connector=self.connector_handler())
+        self.listening_conn = ListeningConnection(self, handler, self.server)
         self.server = self.listening_conn.getAddress()
 
         # Connect to a primary master node, verify data, and
@@ -202,14 +193,11 @@ class Application(object):
             self.event_queue = deque()
             self.event_queue_dict = {}
             try:
-                self.verifyData()
                 self.initialize()
                 self.doOperation()
                 raise RuntimeError, 'should not reach here'
-            except OperationFailure, msg:
+            except StoppedOperation, msg:
                 logging.error('operation stopped: %s', msg)
-                if self.cluster_state == ClusterStates.STOPPING_BACKUP:
-                    self.dm.setBackupTID(None)
             except PrimaryFailure, msg:
                 logging.error('primary master is down: %s', msg)
             finally:
@@ -233,7 +221,7 @@ class Application(object):
         # search, find, connect and identify to the primary master
         bootstrap = BootstrapManager(self, self.name,
                 NodeTypes.STORAGE, self.uuid, self.server)
-        data = bootstrap.getPrimaryConnection(self.connector_handler)
+        data = bootstrap.getPrimaryConnection()
         (node, conn, uuid, num_partitions, num_replicas) = data
         self.master_node = node
         self.master_conn = conn
@@ -256,30 +244,11 @@ class Application(object):
             self.pt = PartitionTable(num_partitions, num_replicas)
             self.loadPartitionTable()
 
-    def verifyData(self):
-        """Verify data under the control by a primary master node.
-        Connections from client nodes may not be accepted at this stage."""
-        logging.info('verifying data')
-
-        handler = verification.VerificationHandler(self)
-        self.master_conn.setHandler(handler)
-        _poll = self._poll
-
-        while not self.operational:
-            _poll()
-
     def initialize(self):
-        """ Retreive partition table and node informations from the primary """
         logging.debug('initializing...')
         _poll = self._poll
-        handler = initialization.InitializationHandler(self)
-        self.master_conn.setHandler(handler)
-
-        # ask node list and partition table
-        self.pt.clear()
-        self.master_conn.ask(Packets.AskNodeInformation())
-        self.master_conn.ask(Packets.AskPartitionTable())
-        while self.master_conn.isPending():
+        self.master_conn.setHandler(initialization.InitializationHandler(self))
+        while not self.operational:
             _poll()
         self.ready = True
         self.replicator.populate()
@@ -345,27 +314,25 @@ class Application(object):
 
     def queueEvent(self, some_callable, conn=None, args=(), key=None,
             raise_on_duplicate=True):
-        msg_id = None if conn is None else conn.getPeerId()
         event_queue_dict = self.event_queue_dict
-        if raise_on_duplicate and key in event_queue_dict:
+        n = event_queue_dict.get(key)
+        if n and raise_on_duplicate:
             raise AlreadyPendingError()
-        else:
-            self.event_queue.append((key, some_callable, msg_id, conn, args))
-            if key is not None:
-                try:
-                    event_queue_dict[key] += 1
-                except KeyError:
-                    event_queue_dict[key] = 1
+        msg_id = None if conn is None else conn.getPeerId()
+        self.event_queue.append((key, some_callable, msg_id, conn, args))
+        if key is not None:
+            event_queue_dict[key] = n + 1 if n else 1
 
     def executeQueuedEvents(self):
-        l = len(self.event_queue)
         p = self.event_queue.popleft
         event_queue_dict = self.event_queue_dict
-        for _ in xrange(l):
+        for _ in xrange(len(self.event_queue)):
             key, some_callable, msg_id, conn, args = p()
             if key is not None:
-                event_queue_dict[key] -= 1
-                if event_queue_dict[key] == 0:
+                n = event_queue_dict[key] - 1
+                if n:
+                    event_queue_dict[key] = n
+                else:
                     del event_queue_dict[key]
             if conn is None:
                 some_callable(*args)
