@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2014  Nexedi SA
+# Copyright (C) 2014-2016  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -14,6 +14,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import os
 import cPickle, pickle, time
 from bisect import bisect, insort
 from collections import deque
@@ -22,11 +23,15 @@ from ConfigParser import SafeConfigParser
 from ZODB.config import storageFromString
 from ZODB.POSException import POSKeyError
 
-from . import buildDatabaseManager, DatabaseManager
-from neo.lib import logging, util
+from . import buildDatabaseManager
+from .manager import DatabaseManager
+from neo.lib import logging, patch, util
 from neo.lib.exception import DatabaseFailure
-from neo.lib.protocol import CellStates, ZERO_OID, ZERO_TID, ZERO_HASH, MAX_TID
+from neo.lib.interfaces import implements
+from neo.lib.protocol import BackendNotImplemented, CellStates, \
+    MAX_TID, ZERO_HASH, ZERO_OID, ZERO_TID
 
+patch.speedupFileStorageTxnLookup()
 
 class Reference(object):
 
@@ -239,7 +244,7 @@ class ZODB(object):
 
     def getDataTid(self, oid, tid):
         try:
-            return self.data_tid[tid][oid]
+            return self.data_tid[tid].get(oid)
         except KeyError:
             assert tid not in self.data_tid, (oid, tid)
             p_tid = util.p64(tid)
@@ -247,8 +252,10 @@ class ZODB(object):
             if txn.tid != p_tid:
                 raise
         u64 = util.u64
-        txn = self.data_tid[tid] = {u64(x.oid): x.data_txn for x in txn}
-        return txn[oid]
+        txn = self.data_tid[tid] = {
+            u64(x.oid): x.data_txn
+            for x in txn if x.data_txn}
+        return txn.get(oid)
 
 
 class ZODBIterator(object):
@@ -276,6 +283,9 @@ class ImporterDatabaseManager(DatabaseManager):
     def __init__(self, *args, **kw):
         super(ImporterDatabaseManager, self).__init__(*args, **kw)
         self.db._connect()
+        implements(self, """_getNextTID checkSerialRange checkTIDRange
+            deleteObject deleteTransaction dropPartitions getLastTID
+            getReplicationObjectList getTIDList nonempty""".split())
 
     _uncommitted_data = property(
         lambda self: self.db._uncommitted_data,
@@ -283,7 +293,7 @@ class ImporterDatabaseManager(DatabaseManager):
 
     def _parse(self, database):
         config = SafeConfigParser()
-        config.read(database)
+        config.read(os.path.expanduser(database))
         sections = config.sections()
         # XXX: defaults copy & pasted from elsewhere - refactoring needed
         main = {'adapter': 'MySQL', 'wait': 0}
@@ -291,11 +301,12 @@ class ImporterDatabaseManager(DatabaseManager):
         self.zodb = ((x, dict(config.items(x))) for x in sections)
         self.compress = main.get('compress', 1)
         self.db = buildDatabaseManager(main['adapter'],
-            (main['database'], main['wait']))
+            (main['database'], main.get('engine'), main['wait']))
         for x in """query erase getConfiguration _setConfiguration
-                    getPartitionTable changePartitionTable getUnfinishedTIDList
-                    dropUnfinishedData storeTransaction finishTransaction
-                    storeData
+                    getPartitionTable changePartitionTable
+                    getUnfinishedTIDDict dropUnfinishedData abortTransaction
+                    storeTransaction lockTransaction unlockTransaction
+                    storeData _pruneData
                  """.split():
             setattr(self, x, getattr(self.db, x))
 
@@ -355,11 +366,14 @@ class ImporterDatabaseManager(DatabaseManager):
         tid = None
         def finish():
             if tid:
-                self.storeTransaction(tid, (), (oid_list,
+                self.storeTransaction(tid, object_list, (
+                    (x[0] for x in object_list),
                     str(txn.user), str(txn.description),
                     cPickle.dumps(txn.extension), False, tid), False)
+                self.releaseData(data_id_list)
                 logging.debug("TXN %s imported (user=%r, desc=%r, len(oid)=%s)",
-                    util.dump(tid), txn.user, txn.description, len(oid_list))
+                    util.dump(tid), txn.user, txn.description, len(object_list))
+                del object_list[:], data_id_list[:]
                 if self._last_commit + 1 < time.time():
                     self.commit()
                 self.zodb_tid = u64(tid)
@@ -368,6 +382,8 @@ class ImporterDatabaseManager(DatabaseManager):
         else:
             compress = None
             compression = 0
+        object_list = []
+        data_id_list = []
         while zodb_list:
             zodb_list.sort()
             z = zodb_list[0]
@@ -375,7 +391,6 @@ class ImporterDatabaseManager(DatabaseManager):
             # user/desc/ext from first ZODB are kept.
             if tid != z.tid:
                 finish()
-                oid_list = []
                 txn = z.transaction
                 tid = txn.tid
                 yield 1
@@ -393,14 +408,16 @@ class ImporterDatabaseManager(DatabaseManager):
                         if compression:
                             data = compressed_data
                     checksum = util.makeChecksum(data)
-                    data_id = self.storeData(util.makeChecksum(data), data,
-                                             compression)
-                # Write metadata before next yield. This may not be efficient
-                # but if they were written at the same time as the transaction,
-                # _pruneData could delete imported but not yet referenced data.
-                self.storeTransaction(tid, ((oid, data_id, data_tid),), (),
-                                      False)
-                oid_list.append(oid)
+                    data_id = self.holdData(util.makeChecksum(data), data,
+                                            compression)
+                    data_id_list.append(data_id)
+                object_list.append((oid, data_id, data_tid))
+                # Give the main loop the opportunity to process requests
+                # from other nodes. In particular, clients may commit. If the
+                # storage node exits after such commit, and before we actually
+                # update 'obj' with 'object_list', some rows in 'data' may be
+                # unreferenced. This is not a problem because the leak is
+                # solved when resuming the migration.
                 yield 1
             try:
                 z.next()
@@ -411,7 +428,7 @@ class ImporterDatabaseManager(DatabaseManager):
         logging.warning("All data are imported. You should change"
             " your configuration to use the native backend and restart.")
         self._import = None
-        for x in """getObject objectPresent getReplicationTIDList
+        for x in """getObject getReplicationTIDList
                  """.split():
             setattr(self, x, getattr(self.db, x))
 
@@ -424,23 +441,11 @@ class ImporterDatabaseManager(DatabaseManager):
         zodb = self.zodb[bisect(self.zodb_index, oid) - 1]
         return zodb, oid - zodb.shift_oid
 
-    def getLastIDs(self, all=True):
-        tid, _, _, oid = self.db.getLastIDs(all)
+    def getLastIDs(self):
+        tid, _, _, oid = self.db.getLastIDs()
         return (max(tid, util.p64(self.zodb_ltid)), None, None,
                 max(oid, util.p64(self.zodb_loid)))
 
-    def objectPresent(self, oid, tid, all=True):
-        r = self.db.objectPresent(oid, tid, all)
-        if not r:
-            u_oid = util.u64(oid)
-            u_tid = util.u64(tid)
-            if self.inZodb(u_oid, u_tid):
-                zodb, oid = self.zodbFromOid(u_oid)
-                try:
-                    return zodb.loadSerial(util.p64(oid), tid)
-                except POSKeyError:
-                    pass
-
     def getObject(self, oid, tid=None, before_tid=None):
         u64 = util.u64
         u_oid = u64(oid)
@@ -481,8 +486,11 @@ class ImporterDatabaseManager(DatabaseManager):
             #         except clause because it would be crazy to import a
             #         NEO DB using this backend.
             checksum = None
-        return (serial, next_serial or
-            db._getNextTID(db._getPartition(u_oid), u_oid, u_tid),
+        if not next_serial:
+            next_serial = db._getNextTID(db._getPartition(u_oid), u_oid, u_tid)
+            if next_serial:
+                next_serial = p64(next_serial)
+        return (serial, next_serial,
             0, checksum, value, zodb.getDataTid(z_oid, u_tid))
 
     def getTransaction(self, tid, all=False):
@@ -498,6 +506,18 @@ class ImporterDatabaseManager(DatabaseManager):
         else:
             return self.db.getTransaction(tid, all)
 
+    def getFinalTID(self, ttid):
+        if u64(ttid) <= self.zodb_ltid and self._import:
+            raise NotImplementedError
+        return self.db.getFinalTID(ttid)
+
+    def _deleteRange(self, partition, min_tid=None, max_tid=None):
+        # Even if everything is imported, we can't truncate below
+        # because it would import again if we restart with this backend.
+        if u64(min_tid) < self.zodb_ltid:
+            raise NotImplementedError
+        self.db._deleteRange(partition, min_tid, max_tid)
+
     def getReplicationTIDList(self, min_tid, max_tid, length, partition):
         p64 = util.p64
         tid = p64(self.zodb_tid)
@@ -535,3 +555,8 @@ class ImporterDatabaseManager(DatabaseManager):
                                                    length, partition)
         return r
 
+    def getObjectHistory(self, *args, **kw):
+        raise BackendNotImplemented(self.getObjectHistory)
+
+    def pack(self, *args, **kw):
+        raise BackendNotImplemented(self.pack)
