@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2014  Nexedi SA
+# Copyright (C) 2006-2016  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,7 +17,9 @@
 from collections import defaultdict
 from functools import wraps
 from neo.lib import logging, util
-from neo.lib.protocol import ZERO_TID, BackendNotImplemented
+from neo.lib.exception import DatabaseFailure
+from neo.lib.interfaces import abstract, requires
+from neo.lib.protocol import ZERO_TID
 
 def lazymethod(func):
     def getter(self):
@@ -36,16 +38,29 @@ def fallback(func):
         return func
     return lazymethod(wraps(func)(warn))
 
+def splitOIDField(tid, oids):
+    if len(oids) % 8:
+        raise DatabaseFailure('invalid oids length for tid %s: %s'
+            % (tid, len(oids)))
+    return [oids[i:i+8] for i in xrange(0, len(oids), 8)]
+
 class CreationUndone(Exception):
     pass
 
 class DatabaseManager(object):
     """This class only describes an interface for database managers."""
 
-    def __init__(self, database, wait=0):
+    ENGINES = ()
+
+    def __init__(self, database, engine=None, wait=0):
         """
             Initialize the object.
         """
+        if engine:
+            if engine not in self.ENGINES:
+                raise ValueError("Unsupported engine: %r not in %r"
+                                 % (engine, self.ENGINES))
+            self._engine = engine
         self._wait = wait
         self._parse(database)
 
@@ -58,9 +73,9 @@ class DatabaseManager(object):
         setattr(self, attr, value)
         return value
 
+    @abstract
     def _parse(self, database):
         """Called during instanciation, to process database parameter."""
-        pass
 
     def setup(self, reset=0):
         """Set up a database, discarding existing data first if reset is True
@@ -70,6 +85,11 @@ class DatabaseManager(object):
         self._uncommitted_data = defaultdict(int)
         self._setup()
 
+    @abstract
+    def erase(self):
+        """"""
+
+    @abstract
     def _setup(self):
         """To be overriden by the backend to set up a database
 
@@ -79,7 +99,22 @@ class DatabaseManager(object):
         where the refcount is increased later, when the object is read-locked.
         Keys are data ids and values are number of references.
         """
-        raise NotImplementedError
+
+    @abstract
+    def nonempty(self, table):
+        """Check whether table is empty or return None if it does not exist"""
+
+    def _checkNoUnfinishedTransactions(self, *hint):
+        if self.nonempty('ttrans') or self.nonempty('tobj'):
+            raise DatabaseFailure(
+                "The database can not be upgraded because you have unfinished"
+                " transactions. Use an older version of NEO to verify them.")
+
+    def _getVersion(self):
+        version = int(self.getConfiguration("version") or 0)
+        if self.VERSION < version:
+            raise DatabaseFailure("The database can not be downgraded.")
+        return version
 
     def doOperation(self, app):
         pass
@@ -87,11 +122,11 @@ class DatabaseManager(object):
     def commit(self):
         pass
 
+    @abstract
     def getConfiguration(self, key):
         """
             Return a configuration value, returns None if not found or not set
         """
-        raise NotImplementedError
 
     def setConfiguration(self, key, value):
         """
@@ -100,8 +135,9 @@ class DatabaseManager(object):
         self._setConfiguration(key, value)
         self.commit()
 
+    @abstract
     def _setConfiguration(self, key, value):
-        raise NotImplementedError
+        """"""
 
     def getUUID(self):
         """
@@ -167,7 +203,7 @@ class DatabaseManager(object):
         """
         ptid = self.getConfiguration('ptid')
         if ptid is not None:
-            return long(ptid)
+            return int(ptid)
 
     def setPTID(self, ptid):
         """
@@ -181,10 +217,18 @@ class DatabaseManager(object):
     def getBackupTID(self):
         return util.bin(self.getConfiguration('backup_tid'))
 
-    def setBackupTID(self, backup_tid):
-        tid = util.dump(backup_tid)
+    def _setBackupTID(self, tid):
+        tid = util.dump(tid)
         logging.debug('backup_tid = %s', tid)
-        return self.setConfiguration('backup_tid', tid)
+        return self._setConfiguration('backup_tid', tid)
+
+    def getTruncateTID(self):
+        return util.bin(self.getConfiguration('truncate_tid'))
+
+    def _setTruncateTID(self, tid):
+        tid = util.dump(tid)
+        logging.debug('truncate_tid = %s', tid)
+        return self._setConfiguration('truncate_tid', tid)
 
     def _setPackTID(self, tid):
         self._setConfiguration('_pack_tid', tid)
@@ -195,24 +239,25 @@ class DatabaseManager(object):
         except TypeError:
             return -1
 
+    @abstract
     def getPartitionTable(self):
         """Return a whole partition table as a sequence of rows. Each row
         is again a tuple of an offset (row ID), the NID of a storage
         node, and a cell state."""
-        raise NotImplementedError
 
+    @abstract
     def getLastTID(self, max_tid):
         """Return greatest tid in trans table that is <= given 'max_tid'
 
         Required only to import a DB using Importer backend.
         max_tid must be in unpacked format.
         """
-        raise NotImplementedError
 
-    def _getLastIDs(self, all=True):
-        raise NotImplementedError
+    def _getLastIDs(self):
+        """"""
 
-    def getLastIDs(self, all=True):
+    @requires(_getLastIDs)
+    def getLastIDs(self):
         trans, obj, oid = self._getLastIDs()
         if trans:
             tid = max(trans.itervalues())
@@ -220,18 +265,25 @@ class DatabaseManager(object):
                 tid = max(tid, max(obj.itervalues()))
         else:
             tid = max(obj.itervalues()) if obj else None
+        # TODO: Replication can't be resumed from the tids in 'trans' and 'obj'
+        #       because outdated cells are writable and may contain recently
+        #       committed data. We must save somewhere where replication was
+        #       interrupted and return this information. For the moment, we
+        #       tell the replicator to resume from the beginning.
+        trans = obj = {}
         return tid, trans, obj, oid
 
-    def getUnfinishedTIDList(self):
-        """Return a list of unfinished transaction's IDs."""
-        raise NotImplementedError
+    def _getUnfinishedTIDDict(self):
+        """"""
 
-    def objectPresent(self, oid, tid, all = True):
-        """Return true iff an object specified by a given pair of an
-        object ID and a transaction ID is present in a database.
-        Otherwise, return false. If all is true, the object must be
-        searched from unfinished transactions as well."""
-        raise NotImplementedError
+    @requires(_getUnfinishedTIDDict)
+    def getUnfinishedTIDDict(self):
+        trans, obj = self._getUnfinishedTIDDict()
+        obj = dict.fromkeys(obj)
+        obj.update(trans)
+        p64 = util.p64
+        return {p64(ttid): None if tid is None else p64(tid)
+                for ttid, tid in obj.iteritems()}
 
     @fallback
     def getLastObjectTID(self, oid):
@@ -239,6 +291,7 @@ class DatabaseManager(object):
         r = self.getObject(oid)
         return r and r[0]
 
+    @abstract
     def _getNextTID(self, partition, oid, tid):
         """
         partition (int)
@@ -250,7 +303,6 @@ class DatabaseManager(object):
 
         If tid is the last revision of oid, None is returned.
         """
-        raise NotImplementedError
 
     def _getObject(self, oid, tid=None, before_tid=None):
         """
@@ -262,8 +314,8 @@ class DatabaseManager(object):
             Serial to retrieve is the highest existing one strictly below this
             value.
         """
-        raise NotImplementedError
 
+    @requires(_getObject)
     def getObject(self, oid, tid=None, before_tid=None):
         """
         oid (packed)
@@ -298,22 +350,23 @@ class DatabaseManager(object):
                 compression, checksum, data,
                 None if data_serial is None else util.p64(data_serial))
 
+    @abstract
     def changePartitionTable(self, ptid, cell_list, reset=False):
         """Change a part of a partition table. The list of cells is
         a tuple of tuples, each of which consists of an offset (row ID),
         the NID of a storage node, and a cell state. The Partition
         Table ID must be stored as well. If reset is True, existing data
         is first thrown away."""
-        raise NotImplementedError
 
+    @abstract
     def dropPartitions(self, offset_list):
         """Delete all data for specified partitions"""
-        raise NotImplementedError
 
+    @abstract
     def dropUnfinishedData(self):
         """Drop any unfinished data from a database."""
-        raise NotImplementedError
 
+    @abstract
     def storeTransaction(self, tid, object_list, transaction, temporary = True):
         """Store a transaction temporarily, if temporary is true. Note
         that this transaction is not finished yet. The list of objects
@@ -322,8 +375,8 @@ class DatabaseManager(object):
         The transaction is either None or a tuple of the list of OIDs,
         user information, a description, extension information and transaction
         pack state (True for packed)."""
-        raise NotImplementedError
 
+    @abstract
     def _pruneData(self, data_id_list):
         """To be overriden by the backend to delete any unreferenced data
 
@@ -332,15 +385,14 @@ class DatabaseManager(object):
         - and not referenced by a fully-committed object (storage should have
           an index or a refcount of all data ids of all objects)
         """
-        raise NotImplementedError
 
+    @abstract
     def storeData(self, checksum, data, compression):
         """To be overriden by the backend to store object raw data
 
         If same data was already stored, the storage only has to check there's
         no hash collision.
         """
-        raise NotImplementedError
 
     def holdData(self, checksum_or_id, *args):
         """Store raw data of temporary object
@@ -459,68 +511,76 @@ class DatabaseManager(object):
             data_tid = p64(data_tid)
         return p64(current_tid), data_tid, is_current
 
-    def finishTransaction(self, tid):
-        """Finish a transaction specified by a given ID, by moving
-        temporarily data to a finished area."""
-        raise NotImplementedError
+    @abstract
+    def lockTransaction(self, tid, ttid):
+        """Mark voted transaction 'ttid' as committed with given 'tid'"""
 
-    def deleteTransaction(self, tid, oid_list=()):
-        """Delete a transaction and its content specified by a given ID and
-        an oid list"""
-        raise NotImplementedError
+    @abstract
+    def unlockTransaction(self, tid, ttid):
+        """Finalize a transaction by moving data to a finished area."""
 
+    @abstract
+    def abortTransaction(self, ttid):
+        """"""
+
+    @abstract
+    def deleteTransaction(self, tid):
+        """"""
+
+    @abstract
     def deleteObject(self, oid, serial=None):
         """Delete given object. If serial is given, only delete that serial for
         given oid."""
-        raise NotImplementedError
 
+    @abstract
     def _deleteRange(self, partition, min_tid=None, max_tid=None):
         """Delete all objects and transactions between given min_tid (excluded)
         and max_tid (included)"""
-        raise NotImplementedError
 
-    def truncate(self, tid):
-        assert tid not in (None, ZERO_TID), tid
-        assert self.getBackupTID()
-        self.setBackupTID(None) # XXX
-        for partition in xrange(self.getNumPartitions()):
-            self._deleteRange(partition, tid)
-        self.commit()
+    def truncate(self):
+        tid = self.getTruncateTID()
+        if tid:
+            assert tid != ZERO_TID, tid
+            for partition in xrange(self.getNumPartitions()):
+                self._deleteRange(partition, tid)
+            self._setTruncateTID(None)
+            self.commit()
 
+    @abstract
     def getTransaction(self, tid, all = False):
         """Return a tuple of the list of OIDs, user information,
         a description, and extension information, for a given transaction
         ID. If there is no such transaction ID in a database, return None.
         If all is true, the transaction must be searched from a temporary
         area as well."""
-        raise NotImplementedError
 
+    @abstract
     def getObjectHistory(self, oid, offset, length):
         """Return a list of serials and sizes for a given object ID.
         The length specifies the maximum size of such a list. Result starts
         with latest serial, and the list must be sorted in descending order.
         If there is no such object ID in a database, return None."""
-        raise BackendNotImplemented(self.getObjectHistory)
 
+    @abstract
     def getReplicationObjectList(self, min_tid, max_tid, length, partition,
             min_oid):
         """Return a dict of length oids grouped by serial at (or above)
         min_tid and min_oid and below max_tid, for given partition,
         sorted in ascending order."""
-        raise NotImplementedError
 
+    @abstract
     def getTIDList(self, offset, length, partition_list):
         """Return a list of TIDs in ascending order from an offset,
         at most the specified length. The list of partitions are passed
         to filter out non-applicable TIDs."""
-        raise BackendNotImplemented(self.getTIDList)
 
+    @abstract
     def getReplicationTIDList(self, min_tid, max_tid, length, partition):
         """Return a list of TIDs in ascending order from an initial tid value,
         at most the specified length up to max_tid. The partition number is
         passed to filter out non-applicable TIDs."""
-        raise NotImplementedError
 
+    @abstract
     def pack(self, tid, updateObjectDataForPack):
         """Prune all non-current object revisions at given tid.
         updateObjectDataForPack is a function called for each deleted object
@@ -536,8 +596,8 @@ class DatabaseManager(object):
             Takes no parameter, returns a 3-tuple: compression, data_id,
             value
         """
-        raise NotImplementedError
 
+    @abstract
     def checkTIDRange(self, partition, length, min_tid, max_tid):
         """
         Generate a diggest from transaction list.
@@ -553,8 +613,8 @@ class DatabaseManager(object):
             - biggest TID found (ie, TID of last record read)
               ZERO_TID if not record found
         """
-        raise NotImplementedError
 
+    @abstract
     def checkSerialRange(self, partition, length, min_tid, max_tid, min_oid):
         """
         Generate a diggest from object list.
@@ -577,5 +637,3 @@ class DatabaseManager(object):
               record read)
               ZERO_TID if no record found
         """
-        raise NotImplementedError
-
