@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2014  Nexedi SA
+# Copyright (C) 2006-2016  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -16,42 +16,46 @@
 
 from binascii import a2b_hex
 import MySQLdb
-from MySQLdb import DataError, IntegrityError, OperationalError
+from MySQLdb import DataError, IntegrityError, \
+    OperationalError, ProgrammingError
 from MySQLdb.constants.CR import SERVER_GONE_ERROR, SERVER_LOST
-from MySQLdb.constants.ER import DATA_TOO_LONG, DUP_ENTRY
+from MySQLdb.constants.ER import DATA_TOO_LONG, DUP_ENTRY, NO_SUCH_TABLE
 from array import array
 from hashlib import sha1
+import os
 import re
 import string
+import struct
 import time
 
-from . import DatabaseManager, LOG_QUERIES
-from .manager import CreationUndone
+from . import LOG_QUERIES
+from .manager import CreationUndone, DatabaseManager, splitOIDField
 from neo.lib import logging, util
 from neo.lib.exception import DatabaseFailure
+from neo.lib.interfaces import implements
 from neo.lib.protocol import CellStates, ZERO_OID, ZERO_TID, ZERO_HASH
 
-def splitOIDField(tid, oids):
-    if (len(oids) % 8) != 0 or len(oids) == 0:
-        raise DatabaseFailure('invalid oids length for tid %d: %d' % (tid,
-            len(oids)))
-    oid_list = []
-    append = oid_list.append
-    for i in xrange(0, len(oids), 8):
-        append(oids[i:i+8])
-    return oid_list
 
+def getPrintableQuery(query, max=70):
+    return ''.join(c if c in string.printable and c not in '\t\x0b\x0c\r'
+        else '\\x%02x' % ord(c) for c in query)
+
+
+@implements
 class MySQLDatabaseManager(DatabaseManager):
     """This class manages a database on MySQL."""
 
-    # WARNING: some parts are not concurrent safe (ex: holdData)
-    # (there must be only 1 writable connection per DB)
+    VERSION = 1
+    ENGINES = "InnoDB", "TokuDB"
+    _engine = ENGINES[0] # default engine
 
     # Disabled even on MySQL 5.1-5.5 and MariaDB 5.2-5.3 because
     # 'select count(*) from obj' sometimes returns incorrect values
     # (tested with testOudatedCellsOnDownStorage).
     _use_partition = False
 
+    _max_allowed_packet = 32769 * 1024
+
     def __init__(self, *args, **kw):
         super(MySQLDatabaseManager, self).__init__(*args, **kw)
         self.conn = None
@@ -60,9 +64,9 @@ class MySQLDatabaseManager(DatabaseManager):
 
     def _parse(self, database):
         """ Get the database credentials (username, password, database) """
-        # expected pattern : [user[:password]@]database[(.|/)unix_socket]
+        # expected pattern : [user[:password]@]database[(~|.|/)unix_socket]
         self.user, self.passwd, self.db, self.socket = re.match(
-            '(?:([^:]+)(?::(.*))?@)?([^./]+)(.+)?$', database).groups()
+            '(?:([^:]+)(?::(.*))?@)?([^~./]+)(.+)?$', database).groups()
 
     def close(self):
         self.conn.close()
@@ -72,7 +76,7 @@ class MySQLDatabaseManager(DatabaseManager):
         if self.passwd is not None:
             kwd['passwd'] = self.passwd
         if self.socket:
-            kwd['unix_socket'] = self.socket
+            kwd['unix_socket'] = os.path.expanduser(self.socket)
         logging.info('connecting to MySQL on the database %s with user %s',
                      self.db, self.user)
         if self._wait < 0:
@@ -89,9 +93,18 @@ class MySQLDatabaseManager(DatabaseManager):
                 logging.exception('Connection to MySQL failed, retrying.')
                 time.sleep(1)
         self._active = 0
-        self.conn.autocommit(False)
-        self.conn.query("SET SESSION group_concat_max_len = %u" % (2**32-1))
-        self.conn.set_sql_mode("TRADITIONAL,NO_ENGINE_SUBSTITUTION")
+        conn = self.conn
+        conn.autocommit(False)
+        conn.query("SET SESSION group_concat_max_len = %u" % (2**32-1))
+        conn.set_sql_mode("TRADITIONAL,NO_ENGINE_SUBSTITUTION")
+        conn.query("SHOW VARIABLES WHERE variable_name='max_allowed_packet'")
+        r = conn.store_result()
+        (name, value), = r.fetch_row(r.num_rows())
+        if int(value) < self._max_allowed_packet:
+            raise DatabaseFailure("Global variable %r is too small."
+                " Minimal value must be %uk."
+                % (name, self._max_allowed_packet // 1024))
+        self._max_allowed_packet = int(value)
 
     def commit(self):
         logging.debug('committing...')
@@ -100,35 +113,31 @@ class MySQLDatabaseManager(DatabaseManager):
 
     def query(self, query):
         """Query data from a database."""
-        conn = self.conn
-        try:
-            if LOG_QUERIES:
-                printable_char_list = []
-                for c in query.split('\n', 1)[0][:70]:
-                    if c not in string.printable or c in '\t\x0b\x0c\r':
-                        c = '\\x%02x' % ord(c)
-                    printable_char_list.append(c)
-                query_part = ''.join(printable_char_list)
-                logging.debug('querying %s...', query_part)
-
-            conn.query(query)
-            if query.startswith("SELECT "):
-                r = conn.store_result()
-                return tuple([
-                    tuple([d.tostring() if isinstance(d, array) else d
-                           for d in row])
-                    for row in r.fetch_row(r.num_rows())])
-            r = query.split(None, 1)[0]
-            if r in ("INSERT", "REPLACE", "DELETE", "UPDATE"):
-                self._active = 1
-            else:
-                assert r in ("ALTER", "CREATE", "DROP", "TRUNCATE"), query
-        except OperationalError, m:
-            if m[0] in (SERVER_GONE_ERROR, SERVER_LOST) and not self._active:
+        if LOG_QUERIES:
+            logging.debug('querying %s...',
+                getPrintableQuery(query.split('\n', 1)[0][:70]))
+        while 1:
+            conn = self.conn
+            try:
+                conn.query(query)
+                if query.startswith("SELECT "):
+                    r = conn.store_result()
+                    return tuple([
+                        tuple([d.tostring() if isinstance(d, array) else d
+                              for d in row])
+                        for row in r.fetch_row(r.num_rows())])
+                break
+            except OperationalError, m:
+                if self._active or m[0] not in (SERVER_GONE_ERROR, SERVER_LOST):
+                    raise DatabaseFailure('MySQL error %d: %s\nQuery: %s'
+                        % (m[0], m[1], getPrintableQuery(query[:1000])))
                 logging.info('the MySQL server is gone; reconnecting')
                 self._connect()
-                return self.query(query)
-            raise DatabaseFailure('MySQL error %d: %s' % (m[0], m[1]))
+        r = query.split(None, 1)[0]
+        if r in ("INSERT", "REPLACE", "DELETE", "UPDATE"):
+            self._active = 1
+        else:
+            assert r in ("ALTER", "CREATE", "DROP", "TRUNCATE"), query
 
     @property
     def escape(self):
@@ -136,18 +145,36 @@ class MySQLDatabaseManager(DatabaseManager):
         return self.conn.escape_string
 
     def erase(self):
-        self.query(
-            "DROP TABLE IF EXISTS config, pt, trans, obj, data, ttrans, tobj")
+        self.query("DROP TABLE IF EXISTS"
+            " config, pt, trans, obj, data, bigdata, ttrans, tobj")
+
+    def nonempty(self, table):
+        try:
+            return bool(self.query("SELECT 1 FROM %s LIMIT 1" % table))
+        except ProgrammingError, (code, _):
+            if code != NO_SUCH_TABLE:
+                raise
 
     def _setup(self):
         self._config.clear()
         q = self.query
-        # The table "config" stores configuration parameters which affect the
-        # persistent data.
-        q("""CREATE TABLE IF NOT EXISTS config (
-                 name VARBINARY(255) NOT NULL PRIMARY KEY,
-                 value VARBINARY(255) NULL
-             ) ENGINE = InnoDB""")
+        p = engine = self._engine
+
+        if self.nonempty("config") is None:
+            # The table "config" stores configuration
+            # parameters which affect the persistent data.
+            q("""CREATE TABLE config (
+                     name VARBINARY(255) NOT NULL PRIMARY KEY,
+                     value VARBINARY(255) NULL
+                 ) ENGINE=""" + engine)
+        else:
+            # Automatic migration.
+            version = self._getVersion()
+            if version < 1:
+                self._checkNoUnfinishedTransactions()
+                q("DROP TABLE IF EXISTS ttrans")
+
+        self._setConfiguration("version", self.VERSION)
 
         # The table "pt" stores a partition table.
         q("""CREATE TABLE IF NOT EXISTS pt (
@@ -155,10 +182,11 @@ class MySQLDatabaseManager(DatabaseManager):
                  nid INT NOT NULL,
                  state TINYINT UNSIGNED NOT NULL,
                  PRIMARY KEY (rid, nid)
-             ) ENGINE = InnoDB""")
+             ) ENGINE=""" + engine)
 
-        p = self._use_partition and """ PARTITION BY LIST (`partition`) (
-            PARTITION dummy VALUES IN (NULL))""" or ''
+        if self._use_partition:
+            p += """ PARTITION BY LIST (`partition`) (
+                PARTITION dummy VALUES IN (NULL))"""
 
         # The table "trans" stores information on committed transactions.
         q("""CREATE TABLE IF NOT EXISTS trans (
@@ -171,7 +199,7 @@ class MySQLDatabaseManager(DatabaseManager):
                  ext BLOB NOT NULL,
                  ttid BIGINT UNSIGNED NOT NULL,
                  PRIMARY KEY (`partition`, tid)
-             ) ENGINE = InnoDB""" + p)
+             ) ENGINE=""" + p)
 
         # The table "obj" stores committed object metadata.
         q("""CREATE TABLE IF NOT EXISTS obj (
@@ -183,29 +211,38 @@ class MySQLDatabaseManager(DatabaseManager):
                  PRIMARY KEY (`partition`, tid, oid),
                  KEY (`partition`, oid, tid),
                  KEY (data_id)
-             ) ENGINE = InnoDB""" + p)
+             ) ENGINE=""" + p)
+
+        if engine == "TokuDB":
+            engine += " compression='tokudb_uncompressed'"
 
         # The table "data" stores object data.
         # We'd like to have partial index on 'hash' colum (e.g. hash(4))
         # but 'UNIQUE' constraint would not work as expected.
         q("""CREATE TABLE IF NOT EXISTS data (
                  id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
-                 hash BINARY(20) NOT NULL UNIQUE,
+                 hash BINARY(20) NOT NULL,
                  compression TINYINT UNSIGNED NULL,
-                 value LONGBLOB NULL
-             ) ENGINE = InnoDB""")
+                 value MEDIUMBLOB NOT NULL,
+                 UNIQUE (hash, compression)
+             ) ENGINE=""" + engine)
+
+        q("""CREATE TABLE IF NOT EXISTS bigdata (
+                 id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
+                 value MEDIUMBLOB NOT NULL
+             ) ENGINE=""" + engine)
 
         # The table "ttrans" stores information on uncommitted transactions.
         q("""CREATE TABLE IF NOT EXISTS ttrans (
                  `partition` SMALLINT UNSIGNED NOT NULL,
-                 tid BIGINT UNSIGNED NOT NULL,
+                 tid BIGINT UNSIGNED,
                  packed BOOLEAN NOT NULL,
                  oids MEDIUMBLOB NOT NULL,
                  user BLOB NOT NULL,
                  description BLOB NOT NULL,
                  ext BLOB NOT NULL,
                  ttid BIGINT UNSIGNED NOT NULL
-             ) ENGINE = InnoDB""")
+             ) ENGINE=""" + engine)
 
         # The table "tobj" stores uncommitted object metadata.
         q("""CREATE TABLE IF NOT EXISTS tobj (
@@ -215,7 +252,7 @@ class MySQLDatabaseManager(DatabaseManager):
                  data_id BIGINT UNSIGNED NULL,
                  value_tid BIGINT UNSIGNED NULL,
                  PRIMARY KEY (tid, oid)
-             ) ENGINE = InnoDB""")
+             ) ENGINE=""" + engine)
 
         self._uncommitted_data.update(q("SELECT data_id, count(*)"
             " FROM tobj WHERE data_id IS NOT NULL GROUP BY data_id"))
@@ -255,10 +292,10 @@ class MySQLDatabaseManager(DatabaseManager):
         return self.query("SELECT * FROM pt")
 
     def getLastTID(self, max_tid):
-        return self.query("SELECT MAX(tid) FROM trans WHERE tid<=%s"
-                          % max_tid)[0][0]
+        return self.query("SELECT MAX(t) FROM (SELECT MAX(tid) as t FROM trans"
+            " WHERE tid<=%s GROUP BY `partition`) as t" % max_tid)[0][0]
 
-    def _getLastIDs(self, all=True):
+    def _getLastIDs(self):
         p64 = util.p64
         q = self.query
         trans = {partition: p64(tid)
@@ -269,29 +306,21 @@ class MySQLDatabaseManager(DatabaseManager):
                                     " FROM obj GROUP BY `partition`")}
         oid = q("SELECT MAX(oid) FROM (SELECT MAX(oid) AS oid FROM obj"
                                       " GROUP BY `partition`) as t")[0][0]
-        if all:
-            tid = q("SELECT MAX(tid) FROM ttrans")[0][0]
-            if tid is not None:
-                trans[None] = p64(tid)
-            tid, toid = q("SELECT MAX(tid), MAX(oid) FROM tobj")[0]
-            if tid is not None:
-                obj[None] = p64(tid)
-            if toid is not None and (oid < toid or oid is None):
-                oid = toid
         return trans, obj, None if oid is None else p64(oid)
 
-    def getUnfinishedTIDList(self):
-        p64 = util.p64
-        return [p64(t[0]) for t in self.query("SELECT tid FROM ttrans"
-                                       " UNION SELECT tid FROM tobj")]
-
-    def objectPresent(self, oid, tid, all = True):
-        oid = util.u64(oid)
-        tid = util.u64(tid)
+    def _getUnfinishedTIDDict(self):
         q = self.query
-        return q("SELECT 1 FROM obj WHERE `partition`=%d AND oid=%d AND tid=%d"
-                 % (self._getPartition(oid), oid, tid)) or all and \
-               q("SELECT 1 FROM tobj WHERE tid=%d AND oid=%d" % (tid, oid))
+        return q("SELECT ttid, tid FROM ttrans"), (ttid
+            for ttid, in q("SELECT DISTINCT tid FROM tobj"))
+
+    def getFinalTID(self, ttid):
+        ttid = util.u64(ttid)
+        # MariaDB is smart enough to realize that 'ttid' is constant.
+        r = self.query("SELECT tid FROM trans"
+            " WHERE `partition`=%s AND tid>=ttid AND ttid=%s LIMIT 1"
+            % (self._getPartition(ttid), ttid))
+        if r:
+            return util.p64(r[0][0])
 
     def getLastObjectTID(self, oid):
         oid = util.u64(oid)
@@ -326,6 +355,9 @@ class MySQLDatabaseManager(DatabaseManager):
             serial, compression, checksum, data, value_serial = r[0]
         except IndexError:
             return None
+        if compression and compression & 0x80:
+            compression &= 0x7f
+            data = ''.join(self._bigData(data))
         return (serial, self._getNextTID(partition, oid, serial),
                 compression, checksum, data, value_serial)
 
@@ -398,6 +430,9 @@ class MySQLDatabaseManager(DatabaseManager):
             obj_table = 'obj'
             trans_table = 'trans'
         q = self.query
+        sql = ["REPLACE INTO %s VALUES " % obj_table]
+        values_max = self._max_allowed_packet - len(sql[0])
+        values_size = 0
         for oid, data_id, value_serial in object_list:
             oid = u64(oid)
             partition = self._getPartition(oid)
@@ -410,41 +445,103 @@ class MySQLDatabaseManager(DatabaseManager):
                     self.holdData(data_id)
             else:
                 value_serial = 'NULL'
-            q("REPLACE INTO %s VALUES (%d, %d, %d, %s, %s)" % (obj_table,
-                partition, oid, tid, data_id or 'NULL', value_serial))
+            value = "(%s,%s,%s,%s,%s)," % (
+                partition, oid, tid, data_id or 'NULL', value_serial)
+            values_size += len(value)
+            # actually: max_values < values_size + EXTRA - len(final comma)
+            # (test_max_allowed_packet checks that EXTRA == 2)
+            if values_max <= values_size:
+                sql[-1] = sql[-1][:-1] # remove final comma
+                q(''.join(sql))
+                del sql[1:]
+                values_size = len(value)
+            sql.append(value)
+        if values_size:
+            sql[-1] = value[:-1] # remove final comma
+            q(''.join(sql))
         if transaction:
             oid_list, user, desc, ext, packed, ttid = transaction
             partition = self._getPartition(tid)
             assert packed in (0, 1)
-            q("REPLACE INTO %s VALUES (%d,%d,%i,'%s','%s','%s','%s',%d)" % (
-                trans_table, partition, tid, packed, e(''.join(oid_list)),
-                e(user), e(desc), e(ext), u64(ttid)))
+            q("REPLACE INTO %s VALUES (%s,%s,%s,'%s','%s','%s','%s',%s)" % (
+                trans_table, partition, 'NULL' if temporary else tid, packed,
+                e(''.join(oid_list)), e(user), e(desc), e(ext), u64(ttid)))
         if temporary:
             self.commit()
 
+    _structLL = struct.Struct(">LL")
+    _unpackLL = _structLL.unpack
+
     def _pruneData(self, data_id_list):
         data_id_list = set(data_id_list).difference(self._uncommitted_data)
         if data_id_list:
-            self.query("DELETE data FROM data"
-                " LEFT JOIN obj ON (id = data_id)"
-                " WHERE id IN (%s) AND data_id IS NULL"
-                % ",".join(map(str, data_id_list)))
+            q = self.query
+            id_list = []
+            bigid_list = []
+            for id, value in q("SELECT id, IF(compression < 128, NULL, value)"
+                               " FROM data LEFT JOIN obj ON (id = data_id)"
+                               " WHERE id IN (%s) AND data_id IS NULL"
+                               % ",".join(map(str, data_id_list))):
+                id_list.append(str(id))
+                if value:
+                    bigdata_id, length = self._unpackLL(value)
+                    bigid_list += xrange(bigdata_id,
+                                         bigdata_id + (length + 0x7fffff >> 23))
+            if id_list:
+                q("DELETE FROM data WHERE id IN (%s)" % ",".join(id_list))
+                if bigid_list:
+                    q("DELETE FROM bigdata WHERE id IN (%s)"
+                      % ",".join(map(str, bigid_list)))
 
-    def storeData(self, checksum, data, compression):
+    def _bigData(self, value):
+        bigdata_id, length = self._unpackLL(value)
+        q = self.query
+        return (q("SELECT value FROM bigdata WHERE id=%s" % i)[0][0]
+            for i in xrange(bigdata_id,
+                            bigdata_id + (length + 0x7fffff >> 23)))
+
+    def storeData(self, checksum, data, compression, _pack=_structLL.pack):
         e = self.escape
         checksum = e(checksum)
+        if 0x1000000 <= len(data): # 16M (MEDIUMBLOB limit)
+            compression |= 0x80
+            q = self.query
+            for r, d in q("SELECT id, value FROM data"
+                          " WHERE hash='%s' AND compression=%s"
+                          % (checksum, compression)):
+                i = 0
+                for d in self._bigData(d):
+                    j = i + len(d)
+                    if data[i:j] != d:
+                        raise IntegrityError(DUP_ENTRY)
+                    i = j
+                if j != len(data):
+                    raise IntegrityError(DUP_ENTRY)
+                return r
+            i = 'NULL'
+            length = len(data)
+            for j in xrange(0, length, 0x800000): # 8M
+                q("INSERT INTO bigdata VALUES (%s, '%s')"
+                  % (i, e(data[j:j+0x800000])))
+                if not j:
+                    i = bigdata_id = self.conn.insert_id()
+                i += 1
+            data = _pack(bigdata_id, length)
         try:
             self.query("INSERT INTO data VALUES (NULL, '%s', %d, '%s')" %
                        (checksum, compression,  e(data)))
         except IntegrityError, (code, _):
             if code == DUP_ENTRY:
-                (r, c, d), = self.query("SELECT id, compression, value"
-                                        " FROM data WHERE hash='%s'" % checksum)
-                if c == compression and d == data:
+                (r, d), = self.query("SELECT id, value FROM data"
+                                     " WHERE hash='%s' AND compression=%s"
+                                     % (checksum, compression))
+                if d == data:
                     return r
             raise
         return self.conn.insert_id()
 
+    del _structLL
+
     def _getDataTID(self, oid, tid=None, before_tid=None):
         sql = ('SELECT tid, value_tid FROM obj'
                ' WHERE `partition` = %d AND oid = %d'
@@ -460,40 +557,40 @@ class MySQLDatabaseManager(DatabaseManager):
         r = self.query(sql)
         return r[0] if r else (None, None)
 
-    def finishTransaction(self, tid):
+    def lockTransaction(self, tid, ttid):
+        u64 = util.u64
+        self.query("UPDATE ttrans SET tid=%d WHERE ttid=%d LIMIT 1"
+                   % (u64(tid), u64(ttid)))
+        self.commit()
+
+    def unlockTransaction(self, tid, ttid):
         q = self.query
-        tid = util.u64(tid)
-        sql = " FROM tobj WHERE tid=%d" % tid
+        u64 = util.u64
+        tid = u64(tid)
+        sql = " FROM tobj WHERE tid=%d" % u64(ttid)
         data_id_list = [x for x, in q("SELECT data_id" + sql) if x]
-        q("INSERT INTO obj SELECT *" + sql)
-        q("DELETE FROM tobj WHERE tid=%d" % tid)
+        q("INSERT INTO obj SELECT `partition`, oid, %d, data_id, value_tid %s"
+          % (tid, sql))
+        q("DELETE" + sql)
         q("INSERT INTO trans SELECT * FROM ttrans WHERE tid=%d" % tid)
         q("DELETE FROM ttrans WHERE tid=%d" % tid)
         self.releaseData(data_id_list)
         self.commit()
 
-    def deleteTransaction(self, tid, oid_list=()):
-        u64 = util.u64
-        tid = u64(tid)
-        getPartition = self._getPartition
+    def abortTransaction(self, ttid):
+        ttid = util.u64(ttid)
         q = self.query
-        sql = " FROM tobj WHERE tid=%d" % tid
+        sql = " FROM tobj WHERE tid=%s" % ttid
         data_id_list = [x for x, in q("SELECT data_id" + sql) if x]
-        self.releaseData(data_id_list)
         q("DELETE" + sql)
-        q("""DELETE FROM ttrans WHERE tid = %d""" % tid)
-        q("""DELETE FROM trans WHERE `partition` = %d AND tid = %d""" %
-            (getPartition(tid), tid))
-        # delete from obj using indexes
-        data_id_set = set()
-        for oid in oid_list:
-            oid = u64(oid)
-            sql = " FROM obj WHERE `partition`=%d AND oid=%d AND tid=%d" \
-               % (getPartition(oid), oid, tid)
-            data_id_set.update(*q("SELECT data_id" + sql))
-            q("DELETE" + sql)
-        data_id_set.discard(None)
-        self._pruneData(data_id_set)
+        q("DELETE FROM ttrans WHERE ttid=%s" % ttid)
+        self.releaseData(data_id_list, True)
+
+    def deleteTransaction(self, tid):
+        tid = util.u64(tid)
+        getPartition = self._getPartition
+        self.query("DELETE FROM trans WHERE `partition`=%s AND tid=%s" %
+            (self._getPartition(tid), tid))
 
     def deleteObject(self, oid, serial=None):
         u64 = util.u64
@@ -540,10 +637,11 @@ class MySQLDatabaseManager(DatabaseManager):
         # client's transaction.
         oid = util.u64(oid)
         p64 = util.p64
-        r = self.query("""SELECT tid, LENGTH(value)
-                    FROM obj LEFT JOIN data ON (obj.data_id = data.id)
-                    WHERE `partition` = %d AND oid = %d AND tid >= %d
-                    ORDER BY tid DESC LIMIT %d, %d""" %
+        r = self.query("SELECT tid, IF(compression < 128, LENGTH(value),"
+            "  CAST(CONV(HEX(SUBSTR(value, 5, 4)), 16, 10) AS INT))"
+            " FROM obj LEFT JOIN data ON (obj.data_id = data.id)"
+            " WHERE `partition` = %d AND oid = %d AND tid >= %d"
+            " ORDER BY tid DESC LIMIT %d, %d" %
             (self._getPartition(oid), oid, self._getPackTID(), offset, length))
         if r:
             return [(p64(tid), length or 0) for tid, length in r]
