@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2012  Nexedi SA
+# Copyright (C) 2012-2016  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -14,53 +14,51 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import os
 import sqlite3
 from hashlib import sha1
 import string
 import traceback
 
-from . import DatabaseManager, LOG_QUERIES
-from .manager import CreationUndone
+from . import LOG_QUERIES
+from .manager import CreationUndone, DatabaseManager, splitOIDField
 from neo.lib import logging, util
 from neo.lib.exception import DatabaseFailure
+from neo.lib.interfaces import implements
 from neo.lib.protocol import CellStates, ZERO_OID, ZERO_TID, ZERO_HASH
 
-def unique_constraint_message(table, column):
+def unique_constraint_message(table, *columns):
     c = sqlite3.connect(":memory:")
-    c.execute("CREATE TABLE %s (%s UNIQUE)" % (table, column))
+    values = '?' * len(columns)
+    insert = "INSERT INTO %s VALUES(%s)" % (table, ', '.join(values))
+    x = "%s (%s)" % (table, ', '.join(columns))
+    c.execute("CREATE TABLE " + x)
+    c.execute("CREATE UNIQUE INDEX i ON " + x)
     try:
-        c.executemany("INSERT INTO %s VALUES(?)" % table, 'xx')
+        c.executemany(insert, (values, values))
     except sqlite3.IntegrityError, e:
         return e.args[0]
     assert False
 
-def splitOIDField(tid, oids):
-    if (len(oids) % 8) != 0 or len(oids) == 0:
-        raise DatabaseFailure('invalid oids length for tid %d: %d' % (tid,
-            len(oids)))
-    oid_list = []
-    append = oid_list.append
-    for i in xrange(0, len(oids), 8):
-        append(oids[i:i+8])
-    return oid_list
-
 def retry_if_locked(f, *args):
     try:
         return f(*args)
-    except sqlite3.OperationalError, e:
+    except sqlite3.OperationalError as e:
         x = e.args[0]
-        if x == 'database is locked':
-            msg = traceback.format_exception_only(type(e), e)
-            msg += traceback.format_stack()
-            logging.warning(''.join(msg))
-            while e.args[0] == x:
-                try:
-                    return f(*args)
-                except sqlite3.OperationalError, e:
-                    pass
-        raise
+        if x != 'database is locked':
+            raise
+        msg = traceback.format_exception_only(type(e), e)
+        msg += traceback.format_stack()
+        logging.warning(''.join(msg))
+        while 1:
+            try:
+                return f(*args)
+            except sqlite3.OperationalError as e:
+                if e.args[0] != x:
+                    raise
 
 
+@implements
 class SQLiteDatabaseManager(DatabaseManager):
     """This class manages a database on SQLite.
 
@@ -70,13 +68,15 @@ class SQLiteDatabaseManager(DatabaseManager):
              never be used for small requests.
     """
 
+    VERSION = 1
+
     def __init__(self, *args, **kw):
         super(SQLiteDatabaseManager, self).__init__(*args, **kw)
         self._config = {}
         self._connect()
 
     def _parse(self, database):
-        self.db = database
+        self.db = os.path.expanduser(database)
 
     def close(self):
         self.conn.close()
@@ -105,15 +105,32 @@ class SQLiteDatabaseManager(DatabaseManager):
         for t in 'config', 'pt', 'trans', 'obj', 'data', 'ttrans', 'tobj':
             self.query('DROP TABLE IF EXISTS ' + t)
 
+    def nonempty(self, table):
+        try:
+            return bool(self.query(
+                "SELECT 1 FROM %s LIMIT 1" % table).fetchone())
+        except sqlite3.OperationalError as e:
+            if not e.args[0].startswith("no such table:"):
+                raise
+
     def _setup(self):
         self._config.clear()
         q = self.query
-        # The table "config" stores configuration parameters which affect the
-        # persistent data.
-        q("""CREATE TABLE IF NOT EXISTS config (
-                 name TEXT NOT NULL PRIMARY KEY,
-                 value TEXT)
-          """)
+
+        if self.nonempty("config") is None:
+            # The table "config" stores configuration
+            # parameters which affect the persistent data.
+            q("CREATE TABLE IF NOT EXISTS config ("
+              "  name TEXT NOT NULL PRIMARY KEY,"
+              "  value TEXT)")
+        else:
+            # Automatic migration.
+            version = self._getVersion()
+            if version < 1:
+                self._checkNoUnfinishedTransactions()
+                q("DROP TABLE IF EXISTS ttrans")
+
+        self._setConfiguration("version", self.VERSION)
 
         # The table "pt" stores a partition table.
         q("""CREATE TABLE IF NOT EXISTS pt (
@@ -155,15 +172,18 @@ class SQLiteDatabaseManager(DatabaseManager):
         # The table "data" stores object data.
         q("""CREATE TABLE IF NOT EXISTS data (
                  id INTEGER PRIMARY KEY AUTOINCREMENT,
-                 hash BLOB NOT NULL UNIQUE,
-                 compression INTEGER,
-                 value BLOB)
+                 hash BLOB NOT NULL,
+                 compression INTEGER NOT NULL,
+                 value BLOB NOT NULL)
+          """)
+        q("""CREATE UNIQUE INDEX IF NOT EXISTS _data_i1 ON
+                 data(hash, compression)
           """)
 
         # The table "ttrans" stores information on uncommitted transactions.
         q("""CREATE TABLE IF NOT EXISTS ttrans (
                  partition INTEGER NOT NULL,
-                 tid INTEGER NOT NULL,
+                 tid INTEGER,
                  packed BOOLEAN NOT NULL,
                  oids BLOB NOT NULL,
                  user BLOB NOT NULL,
@@ -208,11 +228,21 @@ class SQLiteDatabaseManager(DatabaseManager):
     def getPartitionTable(self):
         return self.query("SELECT * FROM pt")
 
+    # A test with a table of 20 million lines and SQLite 3.8.7.1 shows that
+    # it's not worth changing getLastTID:
+    # - It already returns the result in less than 2 seconds, without reading
+    #   the whole table (this is 4-7 times faster than MySQL).
+    # - Strangely, a "GROUP BY partition" clause makes SQLite almost twice
+    #   slower.
+    # - Getting MAX(tid) is immediate with a "AND partition=?" condition so one
+    #   way to speed up the following 2 methods is to repeat the queries for
+    #   each partition (and finish in Python with max() for getLastTID).
+
     def getLastTID(self, max_tid):
         return self.query("SELECT MAX(tid) FROM trans WHERE tid<=?",
                           (max_tid,)).next()[0]
 
-    def _getLastIDs(self, all=True):
+    def _getLastIDs(self):
         p64 = util.p64
         q = self.query
         trans = {partition: p64(tid)
@@ -223,30 +253,21 @@ class SQLiteDatabaseManager(DatabaseManager):
                                     " FROM obj GROUP BY partition")}
         oid = q("SELECT MAX(oid) FROM (SELECT MAX(oid) AS oid FROM obj"
                                       " GROUP BY partition) as t").next()[0]
-        if all:
-            tid = q("SELECT MAX(tid) FROM ttrans").next()[0]
-            if tid is not None:
-                trans[None] = p64(tid)
-            tid, toid = q("SELECT MAX(tid), MAX(oid) FROM tobj").next()
-            if tid is not None:
-                obj[None] = p64(tid)
-            if toid is not None and (oid < toid or oid is None):
-                oid = toid
         return trans, obj, None if oid is None else p64(oid)
 
-    def getUnfinishedTIDList(self):
-        p64 = util.p64
-        return [p64(t[0]) for t in self.query("SELECT tid FROM ttrans"
-                                       " UNION SELECT tid FROM tobj")]
-
-    def objectPresent(self, oid, tid, all=True):
-        oid = util.u64(oid)
-        tid = util.u64(tid)
+    def _getUnfinishedTIDDict(self):
         q = self.query
-        return q("SELECT 1 FROM obj WHERE partition=? AND oid=? AND tid=?",
-                 (self._getPartition(oid), oid, tid)).fetchone() or all and \
-               q("SELECT 1 FROM tobj WHERE tid=? AND oid=?",
-                 (tid, oid)).fetchone()
+        return q("SELECT ttid, tid FROM ttrans"), (ttid
+            for ttid, in q("SELECT DISTINCT tid FROM tobj"))
+
+    def getFinalTID(self, ttid):
+        ttid = util.u64(ttid)
+        # As of SQLite 3.8.7.1, 'tid>=ttid' would ignore the index on tid,
+        # even though ttid is a constant.
+        for tid, in self.query("SELECT tid FROM trans"
+                " WHERE partition=? AND tid>=? AND ttid=? LIMIT 1",
+                (self._getPartition(ttid), ttid, ttid)):
+            return util.p64(tid)
 
     def getLastObjectTID(self, oid):
         oid = util.u64(oid)
@@ -353,7 +374,8 @@ class SQLiteDatabaseManager(DatabaseManager):
             partition = self._getPartition(tid)
             assert packed in (0, 1)
             q("INSERT OR FAIL INTO %strans VALUES (?,?,?,?,?,?,?,?)" % T,
-                (partition, tid, packed, buffer(''.join(oid_list)),
+                (partition, None if temporary else tid,
+                 packed, buffer(''.join(oid_list)),
                  buffer(user), buffer(desc), buffer(ext), u64(ttid)))
         if temporary:
             self.commit()
@@ -369,16 +391,17 @@ class SQLiteDatabaseManager(DatabaseManager):
               % ",".join(map(str, data_id_list)))
 
     def storeData(self, checksum, data, compression,
-                   _dup_hash=unique_constraint_message("data", "hash")):
+            _dup=unique_constraint_message("data", "hash", "compression")):
         H = buffer(checksum)
         try:
             return self.query("INSERT INTO data VALUES (NULL,?,?,?)",
                 (H, compression,  buffer(data))).lastrowid
         except sqlite3.IntegrityError, e:
-            if e.args[0] == _dup_hash:
-                (r, c, d), = self.query("SELECT id, compression, value"
-                              " FROM data WHERE hash=?",  (H,))
-                if c == compression and str(d) == data:
+            if e.args[0] == _dup:
+                (r, d), = self.query("SELECT id, value FROM data"
+                                     " WHERE hash=? AND compression=?",
+                                     (H, compression))
+                if str(d) == data:
                     return r
             raise
 
@@ -397,40 +420,41 @@ class SQLiteDatabaseManager(DatabaseManager):
         r = r.fetchone()
         return r or (None, None)
 
-    def finishTransaction(self, tid):
-        args = util.u64(tid),
+    def lockTransaction(self, tid, ttid):
+        u64 = util.u64
+        self.query("UPDATE ttrans SET tid=? WHERE ttid=?",
+                   (u64(tid), u64(ttid)))
+        self.commit()
+
+    def unlockTransaction(self, tid, ttid):
         q = self.query
+        u64 = util.u64
+        tid = u64(tid)
+        ttid = u64(ttid)
         sql = " FROM tobj WHERE tid=?"
-        data_id_list = [x for x, in q("SELECT data_id" + sql, args) if x]
-        q("INSERT OR FAIL INTO obj SELECT *" + sql, args)
-        q("DELETE FROM tobj WHERE tid=?", args)
-        q("INSERT OR FAIL INTO trans SELECT * FROM ttrans WHERE tid=?", args)
-        q("DELETE FROM ttrans WHERE tid=?", args)
+        data_id_list = [x for x, in q("SELECT data_id" + sql, (ttid,)) if x]
+        q("INSERT INTO obj SELECT partition, oid, ?, data_id, value_tid" + sql,
+          (tid, ttid))
+        q("DELETE" + sql, (ttid,))
+        q("INSERT INTO trans SELECT * FROM ttrans WHERE tid=?", (tid,))
+        q("DELETE FROM ttrans WHERE tid=?", (tid,))
         self.releaseData(data_id_list)
         self.commit()
 
-    def deleteTransaction(self, tid, oid_list=()):
-        u64 = util.u64
-        tid = u64(tid)
-        getPartition = self._getPartition
+    def abortTransaction(self, ttid):
+        args = util.u64(ttid),
         q = self.query
         sql = " FROM tobj WHERE tid=?"
-        data_id_list = [x for x, in q("SELECT data_id" + sql, (tid,)) if x]
-        self.releaseData(data_id_list)
-        q("DELETE" + sql, (tid,))
-        q("DELETE FROM ttrans WHERE tid=?", (tid,))
-        q("DELETE FROM trans WHERE partition=? AND tid=?",
-            (getPartition(tid), tid))
-        # delete from obj using indexes
-        data_id_set = set()
-        for oid in oid_list:
-            oid = u64(oid)
-            sql = " FROM obj WHERE partition=? AND oid=? AND tid=?"
-            args = getPartition(oid), oid, tid
-            data_id_set.update(*q("SELECT data_id" + sql, args))
-            q("DELETE" + sql, args)
-        data_id_set.discard(None)
-        self._pruneData(data_id_set)
+        data_id_list = [x for x, in q("SELECT data_id" + sql, args) if x]
+        q("DELETE" + sql, args)
+        q("DELETE FROM ttrans WHERE ttid=?", args)
+        self.releaseData(data_id_list, True)
+
+    def deleteTransaction(self, tid):
+        tid = util.u64(tid)
+        getPartition = self._getPartition
+        self.query("DELETE FROM trans WHERE partition=? AND tid=?",
+            (self._getPartition(tid), tid))
 
     def deleteObject(self, oid, serial=None):
         oid = util.u64(oid)
