@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2014  Nexedi SA
+# Copyright (C) 2006-2016  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -16,15 +16,15 @@
 
 import weakref
 from functools import wraps
-from neo.lib.connector import ConnectorConnectionClosedException
+from neo.lib.connection import ConnectionClosed
 from neo.lib.handler import EventHandler
 from neo.lib.protocol import Errors, NodeStates, Packets, ProtocolError, \
     ZERO_HASH
 
 def checkConnectionIsReplicatorConnection(func):
     def wrapper(self, conn, *args, **kw):
-        assert self.app.replicator.getCurrentConnection() is conn
-        return func(self, conn, *args, **kw)
+        if self.app.replicator.getCurrentConnection() is conn:
+            return func(self, conn, *args, **kw)
     return wraps(func)(wrapper)
 
 def checkFeedingConnection(check):
@@ -154,7 +154,7 @@ class StorageOperationHandler(EventHandler):
             r = app.dm.checkTIDRange(*args)
             try:
                 conn.answer(Packets.AnswerCheckTIDRange(*r), msg_id)
-            except (weakref.ReferenceError, ConnectorConnectionClosedException):
+            except (weakref.ReferenceError, ConnectionClosed):
                 pass
             yield
         app.newTask(check())
@@ -170,7 +170,7 @@ class StorageOperationHandler(EventHandler):
             r = app.dm.checkSerialRange(*args)
             try:
                 conn.answer(Packets.AnswerCheckSerialRange(*r), msg_id)
-            except (weakref.ReferenceError, ConnectorConnectionClosedException):
+            except (weakref.ReferenceError, ConnectionClosed):
                 pass
             yield
         app.newTask(check())
@@ -211,7 +211,7 @@ class StorageOperationHandler(EventHandler):
                 conn.answer(Packets.AnswerFetchTransactions(
                     pack_tid, next_tid, peer_tid_set), msg_id)
                 yield
-            except (weakref.ReferenceError, ConnectorConnectionClosedException):
+            except (weakref.ReferenceError, ConnectionClosed):
                 pass
         app.newTask(push())
 
@@ -253,6 +253,6 @@ class StorageOperationHandler(EventHandler):
                 conn.answer(Packets.AnswerFetchObjects(
                     pack_tid, next_tid, next_oid, object_dict), msg_id)
                 yield
-            except (weakref.ReferenceError, ConnectorConnectionClosedException):
+            except (weakref.ReferenceError, ConnectionClosed):
                 pass
         app.newTask(push())
