@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2009-2014  Nexedi SA
+# Copyright (C) 2009-2016  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,19 +17,21 @@
 import __builtin__
 import errno
 import functools
+import gc
 import os
 import random
 import socket
 import sys
 import tempfile
 import unittest
+import weakref
 import MySQLdb
 import transaction
 
+from functools import wraps
 from mock import Mock
 from neo.lib import debug, logging, protocol
 from neo.lib.protocol import NodeTypes, Packets, UUID_NAMESPACES
-from neo.lib.util import getAddressType
 from time import time
 from struct import pack, unpack
 from unittest.case import _ExpectedFailure, _UnexpectedSuccess
@@ -47,7 +49,7 @@ def expectedFailure(exception=AssertionError):
                 # XXX: passing sys.exc_info() causes deadlocks
                 raise _ExpectedFailure((type(e), None, None))
             raise _UnexpectedSuccess
-        return functools.wraps(func)(wrapper)
+        return wraps(func)(wrapper)
     if callable(exception) and not isinstance(exception, type):
         func = exception
         exception = Exception
@@ -66,8 +68,11 @@ IP_VERSION_FORMAT_DICT = {
 
 ADDRESS_TYPE = socket.AF_INET
 
+SSL = os.path.dirname(__file__) + os.sep
+SSL = SSL + "ca.crt", SSL + "node.crt", SSL + "node.key"
+
 logging.default_root_handler.handle = lambda record: None
-logging.backlog(None)
+logging.backlog(None, 1<<20)
 
 debug.register()
 # prevent "signal only works in main thread" errors in subprocesses
@@ -101,13 +106,21 @@ def getTempDirectory():
     except KeyError:
         neo_dir = os.path.join(tempfile.gettempdir(), 'neo_tests')
         while True:
-            temp_dir = os.path.join(neo_dir, repr(time()))
+            temp_name = repr(time())
+            temp_dir = os.path.join(neo_dir, temp_name)
             try:
                 os.makedirs(temp_dir)
                 break
             except OSError, e:
                 if e.errno != errno.EEXIST:
                     raise
+        last = os.path.join(neo_dir, "last")
+        try:
+            os.remove(last)
+        except OSError, e:
+            if e.errno != errno.ENOENT:
+                raise
+        os.symlink(temp_name, last)
         os.environ['TEMP'] = temp_dir
         print 'Using temp directory %r.' % temp_dir
     return temp_dir
@@ -145,6 +158,7 @@ class NeoTestBase(unittest.TestCase):
     def tearDown(self):
         assert self.tearDown.im_func is NeoTestBase.tearDown.im_func
         self._tearDown(sys._getframe(1).f_locals['success'])
+        assert not gc.garbage, gc.garbage
 
     def _tearDown(self, success):
         # Kill all unfinished transactions for next test.
@@ -202,8 +216,7 @@ class NeoUnitTestBase(NeoTestBase):
         return Mock({
                 'getCluster': cluster,
                 'getBind': masters[0],
-                'getMasters': (masters, getAddressType((
-                        self.local_ip, 0))),
+                'getMasters': masters,
                 'getReplicas': replicas,
                 'getPartitions': partitions,
                 'getUUID': uuid,
@@ -225,8 +238,7 @@ class NeoUnitTestBase(NeoTestBase):
         return Mock({
                 'getCluster': cluster,
                 'getBind': (masters[0], 10020 + index),
-                'getMasters': (masters, getAddressType((
-                        self.local_ip, 0))),
+                'getMasters': masters,
                 'getDatabase': db,
                 'getUUID': uuid,
                 'getReset': False,
@@ -451,9 +463,6 @@ class NeoUnitTestBase(NeoTestBase):
     def checkAskTransactionInformation(self, conn, **kw):
         return self.checkAskPacket(conn, Packets.AskTransactionInformation, **kw)
 
-    def checkAskObjectPresent(self, conn, **kw):
-        return self.checkAskPacket(conn, Packets.AskObjectPresent, **kw)
-
     def checkAskObject(self, conn, **kw):
         return self.checkAskPacket(conn, Packets.AskObject, **kw)
 
@@ -502,40 +511,75 @@ class NeoUnitTestBase(NeoTestBase):
     def checkAnswerObjectHistory(self, conn, **kw):
         return self.checkAnswerPacket(conn, Packets.AnswerObjectHistory, **kw)
 
-    def checkAnswerStoreTransaction(self, conn, **kw):
-        return self.checkAnswerPacket(conn, Packets.AnswerStoreTransaction, **kw)
-
     def checkAnswerStoreObject(self, conn, **kw):
         return self.checkAnswerPacket(conn, Packets.AnswerStoreObject, **kw)
 
     def checkAnswerPartitionTable(self, conn, **kw):
         return self.checkAnswerPacket(conn, Packets.AnswerPartitionTable, **kw)
 
-    def checkAnswerObjectPresent(self, conn, **kw):
-        return self.checkAnswerPacket(conn, Packets.AnswerObjectPresent, **kw)
 
-connector_cpt = 0
+class Patch(object):
+    """
+    Patch attributes and revert later automatically.
 
-class DoNothingConnector(Mock):
-    def __init__(self, s=None):
-        logging.info("initializing connector")
-        global connector_cpt
-        self.desc = connector_cpt
-        connector_cpt += 1
-        self.packet_cpt = 0
-        Mock.__init__(self)
+    Usage:
 
-    def getAddress(self):
-        return self.addr
+      with Patch(someObject, attrToPatch=newValue, [otherAttr=...]) as patch:
+        [... code that runs with patches ...]
+      [... code that runs without patch ...]
 
-    def makeClientConnection(self, addr):
-        self.addr = addr
+      ' as patch' is optional: 'patch.revert()' can be used to revert patches
+      in the middle of the 'with' clause.
 
-    def makeListeningConnection(self, addr):
-        self.addr = addr
+    Or:
 
-    def getDescriptor(self):
-        return self.desc
+      patch = Patch(...)
+      patch.apply()
+
+      In this case, patches are automatically reverted when 'patch' is deleted.
+
+    For patched callables, the new one receives the original value as first
+    argument.
+    """
+
+    applied = False
+
+    def __init__(self, patched, **patch):
+        (name, patch), = patch.iteritems()
+        self._patched = patched
+        self._name = name
+        if callable(patch):
+            wrapped = getattr(patched, name, None)
+            func = patch
+            patch = lambda *args, **kw: func(wrapped, *args, **kw)
+            if callable(wrapped):
+                patch = wraps(wrapped)(patch)
+        self._patch = patch
+        try:
+            orig = patched.__dict__[name]
+            self._revert = lambda: setattr(patched, name, orig)
+        except KeyError:
+            self._revert = lambda: delattr(patched, name)
+
+    def apply(self):
+        assert not self.applied
+        setattr(self._patched, self._name, self._patch)
+        self.applied = True
+
+    def revert(self):
+        del self.applied
+        self._revert()
+
+    def __del__(self):
+        if self.applied:
+            self.revert()
+
+    def __enter__(self):
+        self.apply()
+        return weakref.proxy(self)
+
+    def __exit__(self, t, v, tb):
+        self.__del__()
 
 
 __builtin__.pdb = lambda depth=0: \
