@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2009-2014  Nexedi SA
+# Copyright (C) 2009-2016  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -22,7 +22,6 @@ import ZODB
 import socket
 import signal
 import random
-import weakref
 import MySQLdb
 import sqlite3
 import unittest
@@ -38,9 +37,8 @@ from neo.lib import logging
 from neo.lib.protocol import ClusterStates, NodeTypes, CellStates, NodeStates, \
     UUID_NAMESPACES
 from neo.lib.util import dump
-from .. import DB_USER, setupMySQLdb, NeoTestBase, buildUrlFromString, \
-        ADDRESS_TYPE, IP_VERSION_FORMAT_DICT, getTempDirectory
-from ..cluster import SocketLock
+from .. import cluster, DB_USER, setupMySQLdb, NeoTestBase, buildUrlFromString, \
+        ADDRESS_TYPE, IP_VERSION_FORMAT_DICT, getTempDirectory, SSL
 from neo.client.Storage import Storage
 from neo.storage.database import buildDatabaseManager
 
@@ -67,55 +65,49 @@ class NotFound(Exception):
 
 class PortAllocator(object):
 
-    lock = SocketLock('neo.PortAllocator')
-    allocator_set = weakref.WeakSet()
-
     def __init__(self):
         self.socket_list = []
-        # WKRD: this is a workaround for a weird bug allowing the same port to
-        # be bound more than once, causing later failure in tests, when
-        # different processes try to bind to the same port.
-        self.sock_port_set = set()
+        self.tried_port_set = set()
 
     def allocate(self, address_type, local_ip):
-        s = socket.socket(address_type, socket.SOCK_STREAM)
-        s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-        if not self.lock.locked():
-            self.lock.acquire()
-        self.allocator_set.add(self)
-        self.socket_list.append(s)
-        sock_port_set = self.sock_port_set
+        min_port = n = 16384
+        max_port = min_port + n
+        tried_port_set = self.tried_port_set
         while True:
-            # Do not let the system choose the port to avoid conflicts
-            # with other software. IOW, use a range different than:
-            # - /proc/sys/net/ipv4/ip_local_port_range on Linux
-            # - what IANA recommends (49152 to 65535)
-            port = random.randint(16384, 32767)
+            s = socket.socket(address_type, socket.SOCK_STREAM)
+            s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+            # Find an unreserved port.
+            while True:
+                # Do not let the system choose the port to avoid conflicts
+                # with other software. IOW, use a range different than:
+                # - /proc/sys/net/ipv4/ip_local_port_range on Linux
+                # - what IANA recommends (49152 to 65535)
+                port = random.randrange(min_port, max_port)
+                if port not in tried_port_set:
+                    tried_port_set.add(port)
+                    try:
+                        s.bind((local_ip, port))
+                        break
+                    except socket.error, e:
+                        if e.errno != errno.EADDRINUSE:
+                            raise
+                elif len(tried_port_set) >= n:
+                    raise RuntimeError("No free port")
+            # Reserve port.
             try:
-                s.bind((local_ip, port))
+                s.listen(1)
+                self.socket_list.append(s)
+                return port
             except socket.error, e:
                 if e.errno != errno.EADDRINUSE:
                     raise
-            else:
-                if port not in sock_port_set:
-                    sock_port_set.add(port)
-                    return port
-                logging.warning('Same port allocated twice: %s in %s',
-                    port, sock_port_set)
 
     def release(self):
         for s in self.socket_list:
             s.close()
         self.__init__()
 
-    def reset(self):
-        if self.lock.locked():
-            self.allocator_set.discard(self)
-            if not self.allocator_set:
-                self.lock.release()
-            self.release()
-
-    __del__ = reset
+    __del__ = release
 
 
 class NEOProcess(object):
@@ -150,9 +142,6 @@ class NEOProcess(object):
             try:
                 # release SQLite debug log
                 logging.setup()
-                # release system-wide lock
-                for allocator in PortAllocator.allocator_set.copy():
-                    allocator.reset()
                 sys.argv = [command] + args
                 getattr(neo.scripts,  command).main()
                 status = 0
@@ -234,6 +223,8 @@ class NEOProcess(object):
 
 class NEOCluster(object):
 
+    SSL = None
+
     def __init__(self, db_list, master_count=1, partitions=1, replicas=0,
                  db_user=DB_USER, db_password='', name=None,
                  cleanup_on_delete=False, temp_dir=None, clear_databases=True,
@@ -300,7 +291,7 @@ class NEOCluster(object):
             self._newProcess(NodeTypes.STORAGE, logger and 'storage_%u' % i,
                              0, adapter=adapter, database=self.db_template(db))
         # create neoctl
-        self.neoctl = NeoCTL((self.local_ip, admin_port))
+        self.neoctl = NeoCTL((self.local_ip, admin_port), ssl=self.SSL)
 
     def _newProcess(self, node_type, logfile=None, port=None, **kw):
         self.uuid_dict[node_type] = uuid = 1 + self.uuid_dict.get(node_type, 0)
@@ -312,6 +303,8 @@ class NEOCluster(object):
             kw['logfile'] = os.path.join(self.temp_dir, logfile + '.log')
         if port is not None:
             kw['bind'] = '%s:%u' % (buildUrlFromString(self.local_ip), port)
+        if self.SSL:
+            kw['ca'], kw['cert'], kw['key'] = self.SSL
         self.process_dict.setdefault(node_type, []).append(
             NEOProcess(command_dict[node_type], uuid, kw))
 
@@ -350,7 +343,6 @@ class NEOCluster(object):
             return True
         if not pdb.wait(test, MAX_START_TIME):
             raise AssertionError('Timeout when starting cluster')
-        self.port_allocator.reset()
 
     def start(self, except_storages=()):
         """ Do a complete start of a cluster """
@@ -412,11 +404,10 @@ class NEOCluster(object):
                 except (AlreadyStopped, NodeProcessError):
                     pass
 
-    def getNEOCTL(self):
-        return self.neoctl
-
     def getZODBStorage(self, **kw):
         master_nodes = self.master_nodes.replace('/', ' ')
+        if self.SSL:
+            kw['ca'], kw['cert'], kw['key'] = self.SSL
         result = Storage(
             master_nodes=master_nodes,
             name=self.cluster_name,
@@ -641,12 +632,22 @@ class NEOCluster(object):
         self.expectCondition(expected_storage_not_known, *args, **kw)
 
     def __del__(self):
+        self.neoctl.close()
         if self.cleanup_on_delete:
             os.removedirs(self.temp_dir)
 
 
 class NEOFunctionalTest(NeoTestBase):
 
+    def setUp(self):
+        if random.randint(0, 1):
+            NEOCluster.SSL = SSL
+        super(NEOFunctionalTest, self).setUp()
+
+    def _tearDown(self, success):
+        super(NEOFunctionalTest, self)._tearDown(success)
+        NEOCluster.SSL = None
+
     def setupLog(self):
         logging.setup(os.path.join(self.getTempDirectory(), 'test.log'))
 
@@ -671,7 +672,7 @@ class NEOFunctionalTest(NeoTestBase):
         thread.daemon = True
         thread.start()
         thread.join(timeout)
-        self.assertFalse(thread.isAlive(), 'Run timeout')
+        self.assertFalse(thread.is_alive(), 'Run timeout')
         if exc_list:
             assert len(exc_list) == 1, exc_list
             exc = exc_list[0]
