@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2009-2014  Nexedi SA
+# Copyright (C) 2009-2016  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -29,8 +29,6 @@ class MasterTests(NEOFunctionalTest):
                 temp_dir=self.getTempDirectory())
         self.neo.stop()
         self.neo.run()
-        self.storage = self.neo.getZODBStorage()
-        self.neoctl = self.neo.getNEOCTL()
 
     def _tearDown(self, success):
         self.neo.stop()
@@ -38,17 +36,18 @@ class MasterTests(NEOFunctionalTest):
 
     def testStoppingSecondaryMaster(self):
         # Wait for masters to stabilize
-        self.neo.expectAllMasters(MASTER_NODE_COUNT)
+        self.neo.expectAllMasters(MASTER_NODE_COUNT, NodeStates.RUNNING)
 
         # Kill
-        primary_uuid = self.neoctl.getPrimary()
+        neoctl = self.neo.neoctl
+        primary_uuid = neoctl.getPrimary()
         for master in self.neo.getMasterProcessList():
             uuid = master.getUUID()
             if uuid != primary_uuid:
                 break
-        self.neo.neoctl.killNode(uuid)
+        neoctl.killNode(uuid)
         self.neo.expectDead(master)
-        self.assertRaises(RuntimeError, self.neo.neoctl.killNode, primary_uuid)
+        self.assertRaises(RuntimeError, neoctl.killNode, primary_uuid)
 
     def testStoppingPrimaryWithTwoSecondaries(self):
         # Wait for masters to stabilize
@@ -61,6 +60,8 @@ class MasterTests(NEOFunctionalTest):
         uuid = killed_uuid_list[0]
         # Check the state of the primary we just killed
         self.neo.expectMasterState(uuid, (None, NodeStates.UNKNOWN))
+        # BUG: The following check expects neoctl to reconnect before
+        #      the election finishes.
         self.assertEqual(self.neo.getPrimary(), None)
         # Check that a primary master arised.
         self.neo.expectPrimary(timeout=10)
@@ -95,6 +96,12 @@ class MasterTests(NEOFunctionalTest):
         # Stop the cluster (so we can start processes manually)
         self.neo.killMasters()
 
+        # Restart admin to make sure it knows all masters.
+        admin, = self.neo.getAdminProcessList()
+        admin.kill()
+        admin.wait()
+        admin.start()
+
         # Start the first master.
         first_master = master_list[0]
         first_master.start()
