@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2009-2014  Nexedi SA
+# Copyright (C) 2009-2016  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -21,7 +21,7 @@ from neo.storage.database.sqlite import SQLiteDatabaseManager
 class StorageSQLiteTests(StorageDBTests):
 
     def getDB(self, reset=0):
-        db = SQLiteDatabaseManager(':memory:', 0)
+        db = SQLiteDatabaseManager(':memory:')
         db.setup(reset)
         return db
 
