@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2011-2014  Nexedi SA
+# Copyright (C) 2011-2016  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
 
 # XXX: Consider using ClusterStates.STOPPING to stop clusters
 
-import os, random, socket, sys, tempfile, threading, time, types, weakref
+import os, random, select, socket, sys, tempfile, threading, time, weakref
 import traceback
 from collections import deque
 from ConfigParser import SafeConfigParser
@@ -29,144 +29,211 @@ import transaction, ZODB
 import neo.admin.app, neo.master.app, neo.storage.app
 import neo.client.app, neo.neoctl.app
 from neo.client import Storage
-from neo.lib import bootstrap, logging
-from neo.lib.connection import BaseConnection, Connection
-from neo.lib.connector import SocketConnector, \
-    ConnectorConnectionRefusedException, ConnectorTryAgainException
-from neo.lib.event import EventManager
-from neo.lib.protocol import CellStates, ClusterStates, NodeStates, NodeTypes
-from neo.lib.util import SOCKET_CONNECTORS_DICT, parseMasterList, p64
-from .. import NeoTestBase, getTempDirectory, setupMySQLdb, \
+from neo.lib import logging
+from neo.lib.connection import BaseConnection, \
+    ClientConnection, Connection, ListeningConnection
+from neo.lib.connector import SocketConnector, ConnectorException
+from neo.lib.handler import EventHandler
+from neo.lib.locking import SimpleQueue
+from neo.lib.protocol import ClusterStates, NodeStates, NodeTypes
+from neo.lib.util import cached_property, parseMasterList, p64
+from .. import NeoTestBase, Patch, getTempDirectory, setupMySQLdb, \
     ADDRESS_TYPE, IP_VERSION_FORMAT_DICT, DB_PREFIX, DB_USER
 
 BIND = IP_VERSION_FORMAT_DICT[ADDRESS_TYPE], 0
 LOCAL_IP = socket.inet_pton(ADDRESS_TYPE, IP_VERSION_FORMAT_DICT[ADDRESS_TYPE])
 
 
+class FairLock(deque):
+    """Same as a threading.Lock except that waiting threads are queued, so that
+    the first one waiting for the lock is the first to get it. This is useful
+    when several concurrent threads fight for the same resource in loop:
+    the owner could give too little time for other to get a chance to acquire,
+    blocking them for a long time with bad luck.
+    """
+
+    def __enter__(self, _allocate_lock=threading.Lock):
+        me = _allocate_lock()
+        me.acquire()
+        self.append(me)
+        other = self[0]
+        while me is not other:
+            with other:
+                other = self[0]
+
+    def __exit__(self, t, v, tb):
+        self.popleft().release()
+
+
 class Serialized(object):
+    """
+    "Threaded" tests run all nodes in the same process as the test itself,
+    and threads are scheduled by this class, which mainly provides 2 features:
+    - more determinism, by minimizing the number of active threads, and
+      switching them in a round-robin;
+    - tic() method to wait only the necessary time for the cluster to be idle.
+
+    The basic concept is that each thread has a lock that always gets acquired
+    by itself. The following pattern is used to yield the processor to the next
+    thread:
+        release(); acquire()
+    It should be noted that this is not atomic, i.e. all other threads
+    sometimes complete before a thread tries to acquire its lock: in order that
+    the previous thread does not fail by releasing an un-acquired lock,
+    we actually use Semaphores instead of Locks.
+
+    The epoll object of each node is hooked so that thread switching happens
+    before polling for network activity. An extra epoll object is used to
+    detect which node has a readable epoll object.
+
+    XXX: It seems wrong to rely only on epoll as way to know if there are
+         pending network messages. I had rare random failures due to tic()
+         returning prematurely.
+    """
+    check_timeout = False
 
     @classmethod
     def init(cls):
-        cls._global_lock = threading.Lock()
-        cls._global_lock.acquire()
-        cls._lock_list = deque()
-        cls._lock_lock = threading.Lock()
-        cls._pdb = False
-        cls.pending = 0
+        cls._busy = set()
+        cls._busy_cond = threading.Condition(threading.Lock())
+        cls._epoll = select.epoll()
+        cls._pdb = None
+        cls._sched_lock = threading.Semaphore(0)
+        cls._tic_lock = FairLock()
+        cls._fd_dict = {}
 
     @classmethod
-    def release(cls, lock=None, wake_other=True, stop=None):
-        """Suspend lock owner and resume first suspended thread"""
-        if lock is None:
-            lock = cls._global_lock
-            if stop:
-                cls.pending = frozenset(stop)
-            else:
-                cls.pending = 0
+    def idle(cls, owner):
+        with cls._busy_cond:
+            cls._busy.discard(owner)
+            cls._busy_cond.notify_all()
+
+    @classmethod
+    def stop(cls):
+        assert not cls._fd_dict, cls._fd_dict
+        del(cls._busy, cls._busy_cond, cls._epoll, cls._fd_dict,
+            cls._pdb, cls._sched_lock, cls._tic_lock)
+
+    @classmethod
+    def _sort_key(cls, fd_event):
+        return -cls._fd_dict[fd_event[0]]._last
+
+    @classmethod
+    @contextmanager
+    def pdb(cls):
         try:
-            sys._getframe(1).f_trace.im_self.set_continue()
-            cls._pdb = True
+            cls._pdb = sys._getframe(2).f_trace.im_self
+            cls._pdb.set_continue()
         except AttributeError:
             pass
-        q = cls._lock_list
-        l = cls._lock_lock
-        l.acquire()
-        try:
-            q.append(lock)
-            if wake_other:
-                q.popleft().release()
-        finally:
-            l.release()
+        yield
+        p = cls._pdb
+        if p is not None:
+            cls._pdb = None
+            t = threading.currentThread()
+            p.stdout.write(getattr(t, 'node_name', t.name))
+            p.set_trace(sys._getframe(3))
 
     @classmethod
-    def acquire(cls, lock=None):
-        """Suspend all threads except lock owner"""
-        if lock is None:
-            lock = cls._global_lock
-        lock.acquire()
-        pending = cls.pending # XXX: getattr once to avoid race conditions
-        if type(pending) is frozenset:
-            if lock is cls._global_lock:
-                cls.pending = 0
-            elif threading.currentThread() in pending:
-                sys.exit()
-        if cls._pdb:
-            cls._pdb = False
-            try:
-                sys.stdout.write(threading.currentThread().node_name)
-            except AttributeError:
-                pass
-            pdb(1)
+    def tic(cls, step=-1, check_timeout=(), quiet=False):
+        # If you're in a pdb here, 'n' switches to another thread
+        # (the following lines are not supposed to be debugged into)
+        with cls._tic_lock, cls.pdb():
+            if not quiet:
+                f = sys._getframe(1)
+                try:
+                    logging.info('tic (%s:%u) ...',
+                        f.f_code.co_filename, f.f_lineno)
+                finally:
+                    del f
+            if cls._busy:
+                with cls._busy_cond:
+                    while cls._busy:
+                        cls._busy_cond.wait()
+            for app in check_timeout:
+                app.em.epoll.check_timeout = True
+                app.em.wakeup()
+                del app
+            while step:
+                event_list = cls._epoll.poll(0)
+                if not event_list:
+                    break
+                step -= 1
+                event_list.sort(key=cls._sort_key)
+                next_lock = cls._sched_lock
+                for fd, event in event_list:
+                    self = cls._fd_dict[fd]
+                    self._release_next = next_lock.release
+                    next_lock = self._lock
+                del self
+                next_lock.release()
+                cls._sched_lock.acquire()
 
-    @classmethod
-    def tic(cls, lock=None):
-        # switch to another thread
-        # (the following calls are not supposed to be debugged into)
-        cls.release(lock); cls.acquire(lock)
+    def __init__(self, app, busy=True):
+        self._epoll = app.em.epoll
+        app.em.epoll = self
+        # XXX: It may have been initialized before the SimpleQueue is patched.
+        thread_container = getattr(app, '_thread_container', None)
+        thread_container is None or thread_container.__init__()
+        if busy:
+            self._busy.add(self) # block tic until app waits for polling
 
-    @classmethod
-    def background(cls):
-        with cls._lock_lock:
-            if cls._lock_list:
-                cls._lock_list.popleft().release()
+    def __getattr__(self, attr):
+        if attr in ('close', 'modify', 'register', 'unregister'):
+            return getattr(self._epoll, attr)
+        return self.__getattribute__(attr)
 
-class SerializedEventManager(EventManager):
+    def poll(self, timeout):
+        if self.check_timeout:
+            assert timeout >= 0, (self, timeout)
+            del self.check_timeout
+        elif timeout:
+            with self.pdb(): # same as in tic()
+                release = self._release_next
+                self._release_next = None
+                release()
+                self._lock.acquire()
+                self._last = time.time()
+        return self._epoll.poll(timeout)
 
-    _lock = None
-    _timeout = 0
-
-    @classmethod
-    def decorate(cls, func):
-        def decorator(*args, **kw):
-            try:
-                EventManager.__init__ = types.MethodType(
-                    cls.__init__.im_func, None, EventManager)
-                return func(*args, **kw)
-            finally:
-                EventManager.__init__ = types.MethodType(
-                    cls._super__init__.im_func, None, EventManager)
-        return wraps(func)(decorator)
-
-    _super__init__ = EventManager.__init__.im_func
-
-    def __init__(self):
+    def _release_next(self):
+        self._last = time.time()
+        self._lock = threading.Semaphore(0)
+        fd = self._epoll.fileno()
         cls = self.__class__
-        assert cls is EventManager
-        self.__class__ = SerializedEventManager
-        self._super__init__()
+        cls._fd_dict[fd] = self
+        cls._epoll.register(fd)
+        cls.idle(self)
 
-    def _poll(self, timeout=1):
-        if self._pending_processing:
-            assert timeout <= 0
-        elif 0 == self._timeout == timeout == Serialized.pending == len(
-            self.writer_set):
-            return
+    def exit(self):
+        fd = self._epoll.fileno()
+        cls = self.__class__
+        if cls._fd_dict.pop(fd, None) is None:
+            cls.idle(self)
         else:
-            if self.writer_set and Serialized.pending == 0:
-                Serialized.pending = 1
-            # Jump to another thread before polling, so that when a message is
-            # sent on the network, one can debug immediately the receiving part.
-            # XXX: Unfortunately, this means we have a useless full-cycle
-            #      before the first message is sent.
-            # TODO: Detect where a message is sent to jump immediately to nodes
-            #       that will do something.
-            Serialized.tic(self._lock)
-            if timeout != 0:
-                timeout = self._timeout
-                if timeout != 0 and Serialized.pending == 1:
-                    Serialized.pending = timeout = 0
-        EventManager._poll(self, timeout)
+            cls._epoll.unregister(fd)
+            self._release_next()
 
-    def addReader(self, conn):
-        EventManager.addReader(self, conn)
-        if type(Serialized.pending) is not frozenset:
-            Serialized.pending = 1
+class TestSerialized(Serialized):
+
+    def __init__(*args):
+        Serialized.__init__(busy=False, *args)
+
+    def poll(self, timeout):
+        if timeout:
+            for x in xrange(1000):
+                r = self._epoll.poll(0)
+                if r:
+                    return r
+                Serialized.tic(step=1)
+            raise Exception("tic is looping forever")
+        return self._epoll.poll(timeout)
 
 
 class Node(object):
 
     def getConnectionList(self, *peers):
-        addr = lambda c: c and (c.accepted_from or c.getAddress())
+        addr = lambda c: c and (c.addr if c.is_server else c.getAddress())
         addr_set = {addr(c.connector) for peer in peers
             for c in peer.em.connection_dict.itervalues()
             if isinstance(c, Connection)}
@@ -183,7 +250,6 @@ class ServerNode(Node):
 
     class __metaclass__(type):
         def __init__(cls, name, bases, d):
-            type.__init__(cls, name, bases, d)
             if Node not in bases and threading.Thread not in cls.__mro__:
                 cls.__bases__ = bases + (threading.Thread,)
                 cls.node_type = getattr(NodeTypes, name[:-11].upper())
@@ -211,13 +277,12 @@ class ServerNode(Node):
             return address
         return cls._node_list[address[1]].getListeningAddress()
 
-    @SerializedEventManager.decorate
     def __init__(self, cluster=None, address=None, **kw):
         if not address:
             address = self.newAddress()
         if cluster is None:
-            master_nodes = kw['master_nodes']
-            name = kw['name']
+            master_nodes = kw.get('master_nodes', ())
+            name = kw.get('name', 'test')
         else:
             master_nodes = kw.get('master_nodes', cluster.master_nodes)
             name = kw.get('name', cluster.name)
@@ -230,52 +295,45 @@ class ServerNode(Node):
         self.daemon = True
         self.node_name = '%s_%u' % (self.node_type, port)
         kw.update(getCluster=name, getBind=address,
-                  getMasters=parseMasterList(master_nodes, address))
+            getMasters=master_nodes and parseMasterList(master_nodes, address))
         super(ServerNode, self).__init__(Mock(kw))
 
     def getVirtualAddress(self):
         return self._init_args['address']
 
     def resetNode(self):
-        assert not self.isAlive()
+        assert not self.is_alive()
         kw = self._init_args
-        self.__dict__.clear()
+        self.close()
         self.__init__(**kw)
 
     def start(self):
-        Serialized.pending = 1
-        self.em._lock = l = threading.Lock()
-        l.acquire()
-        Serialized.release(l, wake_other=0)
+        Serialized(self)
         threading.Thread.start(self)
 
     def run(self):
         try:
-            Serialized.acquire(self.em._lock)
             super(ServerNode, self).run()
         finally:
             self._afterRun()
             logging.debug('stopping %r', self)
-            Serialized.background()
+            self.em.epoll.exit()
 
     def _afterRun(self):
         try:
             self.listening_conn.close()
+            self.listening_conn = None
         except AttributeError:
             pass
 
-    def stop(self):
-        try:
-            Serialized.release(stop=(self,))
-            self.join()
-        finally:
-            Serialized.acquire()
-
     def getListeningAddress(self):
         try:
             return self.listening_conn.getAddress()
         except AttributeError:
-            raise ConnectorConnectionRefusedException
+            raise ConnectorException
+
+    def stop(self):
+        self.em.wakeup(True)
 
 class AdminApplication(ServerNode, neo.admin.app.Application):
     pass
@@ -285,58 +343,51 @@ class MasterApplication(ServerNode, neo.master.app.Application):
 
 class StorageApplication(ServerNode, neo.storage.app.Application):
 
+    dm = type('', (), {'close': lambda self: None})()
+
     def resetNode(self, clear_database=False):
         self._init_args['getReset'] = clear_database
-        dm = self.dm
         super(StorageApplication, self).resetNode()
-        if dm and not clear_database:
-            self.dm = dm
 
     def _afterRun(self):
         super(StorageApplication, self)._afterRun()
         try:
             self.dm.close()
-            self.dm = None
+            del self.dm
         except StandardError: # AttributeError & ProgrammingError
             pass
+        if self.master_conn:
+            self.master_conn.close()
 
-    def switchTables(self):
-        q = self.dm.query
-        for table in 'trans', 'obj':
-            q('ALTER TABLE %s RENAME TO tmp' % table)
-            q('ALTER TABLE t%s RENAME TO %s' % (table, table))
-            q('ALTER TABLE tmp RENAME TO t%s' % table)
+    def getAdapter(self):
+        return self._init_args['getAdapter']
 
     def getDataLockInfo(self):
         dm = self.dm
-        checksum_dict = dict(dm.query("SELECT id, hash FROM data"))
-        assert set(dm._uncommitted_data).issubset(checksum_dict)
+        index = tuple(dm.query("SELECT id, hash, compression FROM data"))
+        assert set(dm._uncommitted_data).issubset(x[0] for x in index)
         get = dm._uncommitted_data.get
-        return {str(v): get(k, 0) for k, v in checksum_dict.iteritems()}
+        return {(str(h), c & 0x7f): get(i, 0) for i, h, c in index}
+
+    def sqlCount(self, table):
+        (r,), = self.dm.query("SELECT COUNT(*) FROM " + table)
+        return r
 
 class ClientApplication(Node, neo.client.app.Application):
 
-    @SerializedEventManager.decorate
     def __init__(self, master_nodes, name, **kw):
         super(ClientApplication, self).__init__(master_nodes, name, **kw)
-        self.em._lock = threading.Lock()
+        self.poll_thread.node_name = name
 
-    def setPoll(self, master=False):
-        if master:
-            self.em._timeout = 1
-            if not self.em._lock.acquire(0):
-                Serialized.background()
-        else:
-            Serialized.release(wake_other=0); Serialized.acquire()
-            self.em._timeout = 0
-
-    def __del__(self):
+    def _run(self):
         try:
-            super(ClientApplication, self).__del__()
+            super(ClientApplication, self)._run()
         finally:
-            if self.poll_thread.isAlive():
-                Serialized.background()
-    close = __del__
+            self.em.epoll.exit()
+
+    def start(self):
+        isinstance(self.em.epoll, Serialized) or Serialized(self)
+        super(ClientApplication, self).start()
 
     def getConnectionList(self, *peers):
         for peer in peers:
@@ -349,10 +400,9 @@ class ClientApplication(Node, neo.client.app.Application):
 
 class NeoCTL(neo.neoctl.app.NeoCTL):
 
-    @SerializedEventManager.decorate
     def __init__(self, *args, **kw):
         super(NeoCTL, self).__init__(*args, **kw)
-        self.em._timeout = -1
+        TestSerialized(self)
 
 
 class LoggerThreadName(str):
@@ -373,29 +423,12 @@ class LoggerThreadName(str):
             return str.__str__(self)
 
 
-class Patch(object):
-
-    def __init__(self, patched, **patch):
-        (name, patch), = patch.iteritems()
-        wrapped = getattr(patched, name)
-        wrapper = lambda *args, **kw: patch(wrapped, *args, **kw)
-        orig = patched.__dict__.get(name)
-        setattr(patched, name, wraps(wrapped)(wrapper))
-        if orig is None:
-            self._revert = lambda: delattr(patched, name)
-        else:
-            self._revert = lambda: setattr(patched, name, orig)
-
-    def __del__(self):
-        self._revert()
-
-
 class ConnectionFilter(object):
 
     filtered_count = 0
     filter_list = []
     filter_queue = weakref.WeakKeyDictionary()
-    lock = threading.Lock()
+    lock = threading.RLock()
     _addPacket = Connection._addPacket
 
     @contextmanager
@@ -447,6 +480,8 @@ class ConnectionFilter(object):
                         queue.appendleft(packet)
                         break
                 else:
+                    if conn.isClosed():
+                        return
                     cls._addPacket(conn, packet)
                     continue
                 break
@@ -456,6 +491,8 @@ class ConnectionFilter(object):
     def add(self, filter, *patches):
         with self.lock:
             self.filter_dict[filter] = patches
+            for p in patches:
+                p.apply()
 
     def remove(self, *filters):
         with self.lock:
@@ -463,18 +500,37 @@ class ConnectionFilter(object):
                 del self.filter_dict[filter]
             self._retry()
 
+    def discard(self, *filters):
+        try:
+            self.remove(*filters)
+        except KeyError:
+            pass
+
     def __contains__(self, filter):
         return filter in self.filter_dict
 
 class NEOCluster(object):
 
-    BaseConnection_checkTimeout = staticmethod(BaseConnection.checkTimeout)
-    SocketConnector_makeClientConnection = staticmethod(
-        SocketConnector.makeClientConnection)
-    SocketConnector_makeListeningConnection = staticmethod(
-        SocketConnector.makeListeningConnection)
-    SocketConnector_receive = staticmethod(SocketConnector.receive)
-    SocketConnector_send = staticmethod(SocketConnector.send)
+    SSL = None
+
+    def __init__(orig, self): # temporary definition for SimpleQueue patch
+        orig(self)
+        lock = self._lock
+        def _lock(blocking=True):
+            if blocking:
+                logging.info('<SimpleQueue>._lock.acquire()')
+                while not lock(False):
+                    Serialized.tic(step=1, quiet=True)
+                return True
+            return lock(False)
+        self._lock = _lock
+    _patches = (
+        Patch(BaseConnection, getTimeout=lambda orig, self: None),
+        Patch(SimpleQueue, __init__=__init__),
+        Patch(SocketConnector, CONNECT_LIMIT=0),
+        Patch(SocketConnector, _bind=lambda orig, self, addr: orig(self, BIND)),
+        Patch(SocketConnector, _connect = lambda orig, self, addr:
+            orig(self, ServerNode.resolv(addr))))
     _patch_count = 0
     _resource_dict = weakref.WeakValueDictionary()
 
@@ -491,40 +547,8 @@ class NEOCluster(object):
         cls._patch_count += 1
         if cls._patch_count > 1:
             return
-        def makeClientConnection(self, addr):
-            real_addr = ServerNode.resolv(addr)
-            try:
-                return cls.SocketConnector_makeClientConnection(self, real_addr)
-            finally:
-                self.remote_addr = addr
-        def send(self, msg):
-            result = cls.SocketConnector_send(self, msg)
-            if type(Serialized.pending) is not frozenset:
-                Serialized.pending = 1
-            return result
-        def receive(self):
-            # If the peer sent an entire packet, make sure we read it entirely,
-            # otherwise Serialize.pending would be reset to 0.
-            data = ''
-            try:
-                while True:
-                    d = cls.SocketConnector_receive(self)
-                    if not d:
-                        return data
-                    data += d
-            except ConnectorTryAgainException:
-                if data:
-                    return data
-                raise
-        # TODO: 'sleep' should 'tic' in a smart way, so that storages can be
-        #       safely started even if the cluster isn't.
-        bootstrap.sleep = lambda seconds: None
-        BaseConnection.checkTimeout = lambda self, t: None
-        SocketConnector.makeClientConnection = makeClientConnection
-        SocketConnector.makeListeningConnection = lambda self, addr: \
-            cls.SocketConnector_makeListeningConnection(self, BIND)
-        SocketConnector.receive = receive
-        SocketConnector.send = send
+        for patch in cls._patches:
+            patch.apply()
         Serialized.init()
 
     @staticmethod
@@ -534,33 +558,31 @@ class NEOCluster(object):
         cls._patch_count -= 1
         if cls._patch_count:
             return
-        bootstrap.sleep = time.sleep
-        BaseConnection.checkTimeout = cls.BaseConnection_checkTimeout
-        SocketConnector.makeClientConnection = \
-            cls.SocketConnector_makeClientConnection
-        SocketConnector.makeListeningConnection = \
-            cls.SocketConnector_makeListeningConnection
-        SocketConnector.receive = cls.SocketConnector_receive
-        SocketConnector.send = cls.SocketConnector_send
+        for patch in cls._patches:
+            patch.revert()
+        Serialized.stop()
 
     def __init__(self, master_count=1, partitions=1, replicas=0, upstream=None,
                        adapter=os.getenv('NEO_TESTS_ADAPTER', 'SQLite'),
                        storage_count=None, db_list=None, clear_databases=True,
                        db_user=DB_USER, db_password='', compress=True,
-                       importer=None):
+                       importer=None, autostart=None):
         self.name = 'neo_%s' % self._allocate('name',
             lambda: random.randint(0, 100))
+        self.compress = compress
         master_list = [MasterApplication.newAddress()
                        for _ in xrange(master_count)]
         self.master_nodes = ' '.join('%s:%s' % x for x in master_list)
         weak_self = weakref.proxy(self)
         kw = dict(cluster=weak_self, getReplicas=replicas, getAdapter=adapter,
-                  getPartitions=partitions, getReset=clear_databases)
+                  getPartitions=partitions, getReset=clear_databases,
+                  getSSL=self.SSL)
         if upstream is not None:
             self.upstream = weakref.proxy(upstream)
             kw.update(getUpstreamCluster=upstream.name,
                 getUpstreamMasters=parseMasterList(upstream.master_nodes))
-        self.master_list = [MasterApplication(address=x, **kw)
+        self.master_list = [MasterApplication(getAutostart=autostart,
+                                              address=x, **kw)
                             for x in master_list]
         if db_list is None:
             if storage_count is None:
@@ -591,9 +613,11 @@ class NEOCluster(object):
         self.storage_list = [StorageApplication(getDatabase=db % x, **kw)
                              for x in db_list]
         self.admin_list = [AdminApplication(**kw)]
-        self.client = ClientApplication(name=self.name,
-            master_nodes=self.master_nodes, compress=compress)
-        self.neoctl = NeoCTL(self.admin.getVirtualAddress())
+        self.neoctl = NeoCTL(self.admin.getVirtualAddress(), ssl=self.SSL)
+
+    def __repr__(self):
+        return "<%s(%s) at 0x%x>" % (self.__class__.__name__,
+                                     self.name, id(self))
 
     # A few shortcuts that work when there's only 1 master/storage/admin
     @property
@@ -622,35 +646,53 @@ class NEOCluster(object):
                 kw['clear_database'] = clear_database
             for node in getattr(self, node_type + '_list'):
                 node.resetNode(**kw)
-        self.client = ClientApplication(name=self.name,
-            master_nodes=self.master_nodes)
-        self.neoctl = NeoCTL(self.admin.getVirtualAddress())
+        self.neoctl.close()
+        self.neoctl = NeoCTL(self.admin.getVirtualAddress(), ssl=self.SSL)
 
     def start(self, storage_list=None, fast_startup=False):
         self._patch()
         for node_type in 'master', 'admin':
             for node in getattr(self, node_type + '_list'):
                 node.start()
-        self.tic()
+        Serialized.tic()
         if fast_startup:
-            self._startCluster()
+            self.startCluster()
         if storage_list is None:
             storage_list = self.storage_list
         for node in storage_list:
             node.start()
-        self.tic()
+        Serialized.tic()
         if not fast_startup:
-            self._startCluster()
-            self.tic()
+            self.startCluster()
+            Serialized.tic()
         state = self.neoctl.getClusterState()
         assert state in (ClusterStates.RUNNING, ClusterStates.BACKINGUP), state
         self.enableStorageList(storage_list)
 
-    def _startCluster(self):
+    def newClient(self):
+        return ClientApplication(name=self.name, master_nodes=self.master_nodes,
+                                 compress=self.compress, ssl=self.SSL)
+
+    @cached_property
+    def client(self):
+        client = self.newClient()
+        # Make sure client won't be reused after it was closed.
+        def close():
+            client = self.client
+            del self.client, client.close
+            client.close()
+        client.close = close
+        return client
+
+    @cached_property
+    def db(self):
+        return ZODB.DB(storage=self.getZODBStorage())
+
+    def startCluster(self):
         try:
             self.neoctl.startCluster()
         except RuntimeError:
-            self.tic()
+            Serialized.tic()
             if self.neoctl.getClusterState() not in (
                       ClusterStates.BACKINGUP,
                       ClusterStates.RUNNING,
@@ -660,64 +702,49 @@ class NEOCluster(object):
 
     def enableStorageList(self, storage_list):
         self.neoctl.enableStorageList([x.uuid for x in storage_list])
-        self.tic()
+        Serialized.tic()
         for node in storage_list:
             assert self.getNodeState(node) == NodeStates.RUNNING
 
-    @property
-    def db(self):
-        try:
-            return self._db
-        except AttributeError:
-            self._db = db = ZODB.DB(storage=self.getZODBStorage())
-            return db
+    def join(self, thread_list, timeout=5):
+        timeout += time.time()
+        while thread_list:
+            assert time.time() < timeout, thread_list
+            Serialized.tic()
+            thread_list = [t for t in thread_list if t.is_alive()]
 
     def stop(self):
-        if hasattr(self, '_db') and self.client.em._timeout == 0:
-            self.client.setPoll(True)
-        self.__dict__.pop('_db', self.client).close()
+        logging.debug("stopping %s", self)
+        client = self.__dict__.get("client")
+        client is None or self.__dict__.pop("db", client).close()
+        node_list = self.admin_list + self.storage_list + self.master_list
+        for node in node_list:
+            node.stop()
         try:
-            Serialized.release(stop=
-                self.admin_list + self.storage_list + self.master_list)
-            for node_type in 'admin', 'storage', 'master':
-                for node in getattr(self, node_type + '_list'):
-                    if node.isAlive():
-                        node.join()
-        finally:
-            Serialized.acquire()
+            node_list.append(client.poll_thread)
+        except AttributeError: # client is None or thread is already stopped
+            pass
+        self.join(node_list)
+        logging.debug("stopped %s", self)
         self._unpatch()
 
-    @staticmethod
-    def tic(force=False):
-        # XXX: Should we automatically switch client in slave mode if it isn't ?
-        f = sys._getframe(1)
-        try:
-            logging.info('tic (%s:%u) ...', f.f_code.co_filename, f.f_lineno)
-        finally:
-            del f
-        if force:
-            Serialized.tic()
-            logging.info('forced tic')
-        while Serialized.pending:
-            Serialized.tic()
-            logging.info('tic')
-
     def getNodeState(self, node):
         uuid = node.uuid
         for node in self.neoctl.getNodeList(node.node_type):
             if node[2] == uuid:
                 return node[3]
 
-    def getOudatedCells(self):
-        return [cell for row in self.neoctl.getPartitionRowList()[1]
-                     for cell in row[1]
-                     if cell[1] == CellStates.OUT_OF_DATE]
+    def getOutdatedCells(self):
+        # Ask the admin instead of the primary master to check that it is
+        # notified of every change.
+        return [(i, cell.getUUID())
+            for i, row in enumerate(self.admin.pt.partition_list)
+            for cell in row
+            if not cell.isReadable()]
 
     def getZODBStorage(self, **kw):
-        # automatically put client in master mode
-        if self.client.em._timeout == 0:
-            self.client.setPoll(True)
-        return Storage.Storage(None, self.name, _app=self.client, **kw)
+        kw['_app'] = kw.pop('client', self.client)
+        return Storage.Storage(None, self.name, **kw)
 
     def importZODB(self, dummy_zodb=None, random=random):
         if dummy_zodb is None:
@@ -752,7 +779,6 @@ class NEOCluster(object):
             for node_type in 'admin', 'storage', 'master':
                 for node in getattr(self, node_type + '_list'):
                     node.close()
-            self.client.em.close()
         except:
             __print_exc()
             raise
@@ -761,6 +787,32 @@ class NEOCluster(object):
         return Patch(self.client.cp, getCellSortKey=lambda orig, cell:
             (orig(cell), key(cell)))
 
+    def moduloTID(self, partition):
+        """Force generation of TIDs that will be stored in given partition"""
+        partition = p64(partition)
+        master = self.primary_master
+        return Patch(master.tm, _nextTID=lambda orig, *args:
+            orig(*args) if args else orig(partition, master.pt.getPartitions()))
+
+    def sortStorageList(self):
+        """Sort storages so that storage_list[i] has partition i for all i"""
+        pt = [{x.getUUID() for x in x}
+            for x in self.primary_master.pt.partition_list]
+        r = []
+        x = [iter(pt[0])]
+        try:
+            while 1:
+                try:
+                    r.append(next(x[-1]))
+                except StopIteration:
+                    del r[-1], x[-1]
+                else:
+                    x.append(iter(pt[len(r)].difference(r)))
+        except IndexError:
+            assert len(r) == len(self.storage_list)
+        x = {x.uuid: x for x in self.storage_list}
+        self.storage_list[:] = (x[r] for r in r)
+        return self.storage_list
 
 class NEOThreadedTest(NeoTestBase):
 
@@ -777,6 +829,23 @@ class NEOThreadedTest(NeoTestBase):
                 db.execute("UPDATE packet SET body=NULL")
                 db.execute("VACUUM")
 
+    tic = Serialized.tic
+
+    def getLoopbackConnection(self):
+        app = MasterApplication(getSSL=NEOCluster.SSL,
+            getReplicas=0, getPartitions=1)
+        handler = EventHandler(app)
+        app.listening_conn = ListeningConnection(app, handler, app.server)
+        node = app.nm.createMaster(address=app.listening_conn.getAddress(),
+                                   uuid=app.uuid)
+        conn = ClientConnection.__new__(ClientConnection)
+        def reset():
+            conn.__dict__.clear()
+            conn.__init__(app, handler, node)
+            conn.reset = reset
+        reset()
+        return conn
+
     def getUnpickler(self, conn):
         reader = conn._reader
         def unpickler(data, compression=False):
@@ -804,7 +873,7 @@ class NEOThreadedTest(NeoTestBase):
 
         def join(self, timeout=None):
             threading.Thread.join(self, timeout)
-            if not self.isAlive() and self.__exc_info:
+            if not self.is_alive() and self.__exc_info:
                 etype, value, tb = self.__exc_info
                 del self.__exc_info
                 raise etype, value, tb
