@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2012  Nexedi SA
+# Copyright (C) 2012-2016  Nexedi SA
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -21,11 +21,15 @@ from collections import defaultdict
 from functools import wraps
 from neo.lib import logging
 from neo.storage.checker import CHECK_COUNT
+from neo.storage.replicator import Replicator
+from neo.lib.connector import SocketConnector
+from neo.lib.connection import ClientConnection
+from neo.lib.event import EventManager
 from neo.lib.protocol import CellStates, ClusterStates, Packets, \
     ZERO_OID, ZERO_TID, MAX_TID, uuid_str
 from neo.lib.util import p64
-from . import ConnectionFilter, NEOCluster, NEOThreadedTest, Patch, \
-    predictable_random
+from .. import Patch
+from . import ConnectionFilter, NEOCluster, NEOThreadedTest, predictable_random
 
 
 def backup_test(partitions=1, upstream_kw={}, backup_kw={}):
@@ -38,7 +42,7 @@ def backup_test(partitions=1, upstream_kw={}, backup_kw={}):
                 try:
                     backup.start()
                     backup.neoctl.setClusterState(ClusterStates.STARTING_BACKUP)
-                    backup.tic()
+                    self.tic()
                     wrapped(self, backup)
                 finally:
                     backup.stop()
@@ -86,19 +90,17 @@ class ReplicationTests(NEOThreadedTest):
             upstream.start()
             importZODB = upstream.importZODB()
             importZODB(3)
-            upstream.client.setPoll(0)
             backup = NEOCluster(partitions=np, replicas=nr-1, storage_count=5,
                                 upstream=upstream)
             try:
                 backup.start()
                 # Initialize & catch up.
                 backup.neoctl.setClusterState(ClusterStates.STARTING_BACKUP)
-                backup.tic()
+                self.tic()
                 self.assertEqual(np*nr, self.checkBackup(backup))
                 # Normal case, following upstream cluster closely.
                 importZODB(17)
-                upstream.client.setPoll(0)
-                backup.tic()
+                self.tic()
                 self.assertEqual(np*nr, self.checkBackup(backup))
             # Check that a backup cluster can be restarted.
             finally:
@@ -109,14 +111,13 @@ class ReplicationTests(NEOThreadedTest):
                 self.assertEqual(backup.neoctl.getClusterState(),
                                  ClusterStates.BACKINGUP)
                 importZODB(17)
-                upstream.client.setPoll(0)
-                backup.tic()
+                self.tic()
                 self.assertEqual(np*nr, self.checkBackup(backup))
                 backup.neoctl.checkReplicas(check_dict, ZERO_TID, None)
-                backup.tic()
+                self.tic()
                 # Stop backing up, nothing truncated.
                 backup.neoctl.setClusterState(ClusterStates.STOPPING_BACKUP)
-                backup.tic()
+                self.tic()
                 self.assertEqual(np*nr, self.checkBackup(backup))
                 self.assertEqual(backup.neoctl.getClusterState(),
                                  ClusterStates.RUNNING)
@@ -130,16 +131,15 @@ class ReplicationTests(NEOThreadedTest):
             try:
                 backup.start()
                 backup.neoctl.setClusterState(ClusterStates.STARTING_BACKUP)
-                backup.tic()
+                self.tic()
                 with backup.master.filterConnection(*backup.storage_list) as f:
                     f.add(delaySecondary)
                     while not f.filtered_count:
                         importZODB(1)
-                    upstream.client.setPoll(0)
-                    backup.tic()
+                    self.tic()
                     backup.neoctl.setClusterState(ClusterStates.STOPPING_BACKUP)
-                    backup.tic()
-                backup.tic(force=1)
+                    self.tic()
+                self.tic()
                 self.assertEqual(np*nr, self.checkBackup(backup,
                     max_tid=backup.master.getLastTransaction()))
             finally:
@@ -148,17 +148,16 @@ class ReplicationTests(NEOThreadedTest):
             try:
                 backup.start()
                 backup.neoctl.setClusterState(ClusterStates.STARTING_BACKUP)
-                backup.tic()
+                self.tic()
                 with ConnectionFilter() as f:
                     f.add(lambda conn, packet: conn.getUUID() is None and
                         isinstance(packet, Packets.AddObject))
                     while not f.filtered_count:
                         importZODB(1)
-                    upstream.client.setPoll(0)
-                    backup.tic()
+                    self.tic()
                     backup.neoctl.setClusterState(ClusterStates.STOPPING_BACKUP)
-                    backup.tic()
-                backup.tic(force=1)
+                    self.tic()
+                self.tic()
                 self.assertEqual(np*nr, self.checkBackup(backup,
                     max_tid=backup.master.getLastTransaction()))
             finally:
@@ -197,13 +196,12 @@ class ReplicationTests(NEOThreadedTest):
             # Do not start with an empty DB so that 'primary_dict' below is not
             # empty on the first iteration.
             importZODB(1)
-            upstream.client.setPoll(0)
             backup = NEOCluster(partitions=np, replicas=2, storage_count=4,
                                 upstream=upstream)
             try:
                 backup.start()
                 backup.neoctl.setClusterState(ClusterStates.STARTING_BACKUP)
-                backup.tic()
+                self.tic()
                 storage_list = [x.uuid for x in backup.storage_list]
                 slave = set(xrange(len(storage_list))).difference
                 for event in xrange(10):
@@ -224,14 +222,11 @@ class ReplicationTests(NEOThreadedTest):
                         # a second replication partially and aborts.
                         p = Patch(backup.storage_list[storage].replicator,
                                   fetchObjects=fetchObjects)
-                    try:
+                    with p:
                         importZODB(lambda x: counts[0] > 1)
-                    finally:
-                        del p
-                    upstream.client.setPoll(0)
                     if event > 5:
                         backup.neoctl.checkReplicas(check_dict, ZERO_TID, None)
-                    backup.tic()
+                    self.tic()
                     self.assertEqual(np*3, self.checkBackup(backup))
             finally:
                 backup.stop()
@@ -249,28 +244,63 @@ class ReplicationTests(NEOThreadedTest):
         """
         conn, = backup.master.getConnectionList(backup.upstream.master)
         # trigger ping
-        conn.updateTimeout(1)
         self.assertFalse(conn.isPending())
-        conn.checkTimeout(time.time())
+        conn.onTimeout()
         self.assertTrue(conn.isPending())
         # force ping to have expired
-        conn.updateTimeout(1)
         # connection will be closed before upstream master has time
         # to answer
-        backup.tic(force=1)
+        def _poll(orig, self, blocking):
+            if backup.master.em is self:
+                p.revert()
+                conn._next_timeout = 0
+                conn.onTimeout()
+            else:
+                orig(self, blocking)
+        with Patch(EventManager, _poll=_poll) as p:
+            self.tic()
         new_conn, = backup.master.getConnectionList(backup.upstream.master)
-        self.assertFalse(new_conn is conn)
+        self.assertIsNot(new_conn, conn)
+
+    @backup_test()
+    def testBackupUpstreamStorageDead(self, backup):
+        upstream = backup.upstream
+        with ConnectionFilter() as f:
+            f.add(lambda conn, packet:
+                isinstance(packet, Packets.InvalidateObjects))
+            upstream.importZODB()(1)
+        count = [0]
+        def _connect(orig, conn):
+            count[0] += 1
+            orig(conn)
+        with Patch(ClientConnection, _connect=_connect):
+            upstream.storage.listening_conn.close()
+            self.tic(step=2)
+            self.assertEqual(count[0], 0)
+            t = SocketConnector.CONNECT_LIMIT = .5
+            t += time.time()
+            self.tic()
+            # 1st attempt failed, 2nd is deferred
+            self.assertEqual(count[0], 2)
+            self.tic(check_timeout=(backup.storage,))
+            # 2nd failed, 3rd deferred
+            self.assertEqual(count[0], 4)
+            self.assertTrue(t <= time.time())
 
     @backup_test()
     def testBackupDelayedUnlockTransaction(self, backup):
+        """
+        Check that a backup storage node is put on hold by upstream if
+        the requested transaction is still locked. Such case happens when
+        the backup cluster reacts very quickly to a new transaction.
+        """
         upstream = backup.upstream
         with upstream.master.filterConnection(upstream.storage) as f:
             f.add(lambda conn, packet:
                 isinstance(packet, Packets.NotifyUnlockInformation))
             upstream.importZODB()(1)
-            upstream.client.setPoll(0)
-            backup.tic()
-        backup.tic(force=1)
+            self.tic()
+        self.tic()
         self.assertEqual(1, self.checkBackup(backup))
 
     def testReplicationAbortedBySource(self):
@@ -305,10 +335,9 @@ class ReplicationTests(NEOThreadedTest):
             try:
                 cluster.start([s0])
                 cluster.populate([range(np*2)] * np)
-                cluster.client.setPoll(0)
                 s1.start()
                 s2.start()
-                cluster.tic()
+                self.tic()
                 cluster.neoctl.enableStorageList([s1.uuid, s2.uuid])
                 cluster.neoctl.tweakPartitionTable()
                 offset, = [offset for offset, row in enumerate(
@@ -317,14 +346,44 @@ class ReplicationTests(NEOThreadedTest):
                 with ConnectionFilter() as connection_filter:
                     connection_filter.add(delayAskFetch,
                         Patch(s0.dm, changePartitionTable=changePartitionTable))
-                    cluster.tic()
+                    self.tic()
                     self.assertEqual(1, connection_filter.filtered_count)
-                cluster.tic()
+                self.tic()
                 self.checkPartitionReplicated(s1, s2, offset)
             finally:
                 cluster.stop()
             cluster.reset(True)
 
+    def testResumingReplication(self):
+        cluster = NEOCluster(replicas=1)
+        try:
+            s0, s1 = cluster.storage_list
+            cluster.start(storage_list=(s0,))
+            t, c = cluster.getTransaction()
+            r = c.root()
+            r._p_changed = 1
+            t.commit()
+            s1.start()
+            self.tic()
+            with Patch(Replicator, connected=lambda *_: None):
+                cluster.enableStorageList((s1,))
+                cluster.neoctl.tweakPartitionTable()
+                r._p_changed = 1
+                t.commit()
+                self.tic()
+                s1.stop()
+                cluster.join((s1,))
+            t0, t1, t2 = c._storage.iterator()
+            s1.resetNode()
+            s1.start()
+            self.tic()
+            self.assertEqual([], cluster.getOutdatedCells())
+            s0.stop()
+            cluster.join((s0,))
+            t0, t1, t2 = c._storage.iterator()
+        finally:
+            cluster.stop()
+
     def testCheckReplicas(self):
         from neo.storage import checker
         def corrupt(offset):
@@ -349,25 +408,24 @@ class ReplicationTests(NEOThreadedTest):
             checker.CHECK_COUNT = 2
             cluster.start()
             cluster.populate([range(np*2)] * tid_count)
-            cluster.client.setPoll(0)
             storage_dict = {x.uuid: x for x in cluster.storage_list}
             cluster.neoctl.checkReplicas(check_dict, ZERO_TID, None)
-            cluster.tic()
+            self.tic()
             check(ClusterStates.RUNNING, 0)
             source = corrupt(0)
             cluster.neoctl.checkReplicas(check_dict, p64(corrupt_tid+1), None)
-            cluster.tic()
+            self.tic()
             check(ClusterStates.RUNNING, 0)
             cluster.neoctl.checkReplicas({0: source}, ZERO_TID, None)
-            cluster.tic()
+            self.tic()
             check(ClusterStates.RUNNING, 1)
             corrupt(1)
             cluster.neoctl.checkReplicas(check_dict, p64(corrupt_tid+1), None)
-            cluster.tic()
+            self.tic()
             check(ClusterStates.RUNNING, 1)
             cluster.neoctl.checkReplicas(check_dict, ZERO_TID, None)
-            cluster.tic()
-            check(ClusterStates.VERIFYING, 4)
+            self.tic()
+            check(ClusterStates.RECOVERING, 4)
         finally:
             checker.CHECK_COUNT = CHECK_COUNT
             cluster.stop()
